/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	THIS C SOURCE CODE FILE CONTAINS FUNCTIONS FOR MANAGING THE LINKS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

#ifndef _LINK_FUNCTIONS_
#define _LINK_FUNCTIONS_

#define _DEBUG_ 0

/* standard include files */
#include <Xm/Xm.h>
#include <Xm/DrawingA.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

/* application globals include files */
#ifndef _GLOBAL_NETWORK_
#include "creation_network.h"
#endif

/* type definitions include files */
#ifndef _TYPESDEFINED_
#include "newtypes.h"
#endif

/* prototypes include files */
#ifndef _PROTOTYPES_
#include "prototypes.h"
#endif

/* dialog boxes include files */
#ifndef _MESSBOX_
#include "messbox.h"
#endif

/*******************************************************************************
	Function that calculates the absolute length in mm between two 3D entities
*******************************************************************************/
float absolute_length(float L1,float H1,float AP1,float L2,float H2,float AP2)
{
	float d_L;
	float d_H;
	float d_AP;
	
	d_L = sqrt((L1 - L2)*(L1 - L2));
	d_H = sqrt((H1 - H2)*(H1 - H2));
	d_AP = sqrt((AP1 - AP2)*(AP1 - AP2));
	
	return (float)(d_L + d_H + d_AP);

}

/*******************************************************************************
	Function for allocating a LINK_PARAMETERS structure in the network level
	a filling it with default values
*******************************************************************************/
LINK_PARAMETERS *alloc_new_link_net(void)
{
	LINK_PARAMETERS **link_array;
	int i;
	LINK_PARAMETERS *retval;
	
	if(!current_network)
		return NULL;
	
	if(!current_nucleus || !previous_selected_nucleus)
	{
		fprintf(stderr,"not enough selected nucleus in alloc_new_link_net!\n");
		exit(-1);
	}
	
	if(current_network->link_array)
	{
		for(i=0 ; i<current_network->nb_links ; i++)
		{
			if((current_network->link_array[i]->source_id
			 == previous_selected_nucleus->nucleus_id) &&
			 (current_network->link_array[i]->target_id
			 == current_nucleus->nucleus_id) && 
			 (current_network->link_array[i]->level == _NETWORK_))
			 	return NULL;
		}
		
		link_array = (LINK_PARAMETERS **)calloc(1,
			(current_network->nb_links +1)*sizeof(LINK_PARAMETERS *));
		for(i=0 ; i<current_network->nb_links ; i++)
			link_array[i] = current_network->link_array[i];
				
		retval = link_array[current_network->nb_links] = (LINK_PARAMETERS *)
			calloc(1,sizeof(LINK_PARAMETERS));
		
		retval->level = _NETWORK_;
		
		retval->source_id = previous_selected_nucleus->nucleus_id;
		retval->target_id = current_nucleus->nucleus_id;	
		
		/* default values */
		
		retval->axone_mean = 
			absolute_length(previous_selected_nucleus->L,
				previous_selected_nucleus->H,
				previous_selected_nucleus->AP,
				current_nucleus->L,
				current_nucleus->H,
				current_nucleus->AP) / _AXONE_NORMAL_SPEED_;
		retval->axone_sd = (
			absolute_length(0,
				0,
				0,
				previous_selected_nucleus->delta_L,
				previous_selected_nucleus->delta_H,
				previous_selected_nucleus->delta_AP) +
			absolute_length(0,
				0,
				0,	
				current_nucleus->delta_L,
				current_nucleus->delta_H,
				current_nucleus->delta_AP) ) / _AXONE_NORMAL_SPEED_;
		if(retval->axone_mean < retval->axone_sd)
			retval->axone_mean = retval->axone_sd;
		retval->min_knobs = 1;
		retval->max_knobs = 10;
		retval->nmda = False;
		retval->type = _FULL_EXCITATION_;		
				
				
				
		free(current_network->link_array);
		
		current_network->link_array = link_array;
	}
	else
	{
		link_array = (LINK_PARAMETERS **)calloc(1,
			sizeof(LINK_PARAMETERS *));
				
		retval = link_array[0] = (LINK_PARAMETERS *)
			calloc(1,sizeof(LINK_PARAMETERS));
		
		retval->level = _NETWORK_;
		
		retval->source_id = previous_selected_nucleus->nucleus_id;
		retval->target_id = current_nucleus->nucleus_id;	
		
		/* default values */
		
		retval->axone_mean = 
			absolute_length(previous_selected_nucleus->L,
				previous_selected_nucleus->H,
				previous_selected_nucleus->AP,
				current_nucleus->L,
				current_nucleus->H,
				current_nucleus->AP) / _AXONE_NORMAL_SPEED_;
		retval->axone_sd = (
			absolute_length(0,
				0,
				0,
				previous_selected_nucleus->delta_L,
				previous_selected_nucleus->delta_H,
				previous_selected_nucleus->delta_AP) +
			absolute_length(0,
				0,
				0,	
				current_nucleus->delta_L,
				current_nucleus->delta_H,
				current_nucleus->delta_AP) ) / _AXONE_NORMAL_SPEED_;
		if(retval->axone_mean < retval->axone_sd)
			retval->axone_mean = retval->axone_sd + 1.0;	
		retval->min_knobs = 1;
		retval->max_knobs = 10;
		retval->nmda = False;
		retval->type = _FULL_EXCITATION_;		
	
		
		current_network->link_array = link_array;
		
	}
	
	current_network->nb_links++;
	return (LINK_PARAMETERS *)retval;
	
}

/*******************************************************************************
	Function for allocating a LINK_PARAMETERS structure in the nucleus level
	a filling it with default values
*******************************************************************************/
LINK_PARAMETERS *alloc_new_link_in_nucleus(CLUSTER_WIDGET *cluster_src,
		CLUSTER_WIDGET *cluster_tgt)
{
	LINK_PARAMETERS **link_array;
	int i;
	LINK_PARAMETERS *retval;
	
	if(!current_nucleus)
	{
		fprintf(stderr,"current_nucleus is NULL in alloc_new_link_in_nucleus!\n");
		exit(-1);
	}	
	
	if(!cluster_src || !cluster_tgt)
	{
		fprintf(stderr,"one of both clusters is NULL in alloc_new_link_in_nucleus!\n");
		exit(-1);
	}
	
	if(current_nucleus->link_array)
	{
		for(i=0 ; i<current_nucleus->nb_links ; i++)
		{
			if((current_nucleus->link_array[i]->source_id
			 == cluster_src->cluster_id) &&
			 (current_nucleus->link_array[i]->target_id
			 == cluster_tgt->cluster_id) && 
			 (current_nucleus->link_array[i]->level == _NUCLEUS_))
			 	return NULL;
		}
		link_array = (LINK_PARAMETERS **)calloc(1,
			(current_nucleus->nb_links +1)*sizeof(LINK_PARAMETERS *));
		for(i=0 ; i<current_nucleus->nb_links ; i++)
			link_array[i] = current_nucleus->link_array[i];
				
		retval = link_array[current_nucleus->nb_links] = (LINK_PARAMETERS *)
			calloc(1,sizeof(LINK_PARAMETERS));
		
		retval->level = _NUCLEUS_;
		
		retval->source_id = cluster_src->cluster_id;
		retval->target_id = cluster_tgt->cluster_id;	
		
		/* default values */
		
		retval->axone_mean = 
			absolute_length(cluster_src->L,
				cluster_src->H,
				cluster_src->AP,
				cluster_tgt->L,
				cluster_tgt->H,
				cluster_tgt->AP) / _AXONE_NORMAL_SPEED_;
		retval->axone_sd = (
			absolute_length(0,
				0,
				0,
				cluster_src->delta_L,
				cluster_src->delta_H,
				cluster_src->delta_AP) +
			absolute_length(0,
				0,
				0,	
				cluster_tgt->delta_L,
				cluster_tgt->delta_H,
				cluster_tgt->delta_AP) ) / _AXONE_NORMAL_SPEED_;
		retval->min_knobs = 1;
		retval->max_knobs = 10;
		retval->nmda = False;
		retval->type = _FULL_EXCITATION_;		
				
				
				
		free(current_nucleus->link_array);
		
		current_nucleus->link_array = link_array;
	}
	else
	{
		link_array = (LINK_PARAMETERS **)calloc(1,
			sizeof(LINK_PARAMETERS *));
		
		retval = link_array[0] = (LINK_PARAMETERS *)
			calloc(1,sizeof(LINK_PARAMETERS));
		
		retval->level = _NUCLEUS_;
		
		retval->source_id = cluster_src->cluster_id;
		retval->target_id = cluster_tgt->cluster_id;	
		
		/* default values */
		
		retval->axone_mean = 
			absolute_length(cluster_src->L,
				cluster_src->H,
				cluster_src->AP,
				cluster_tgt->L,
				cluster_tgt->H,
				cluster_tgt->AP) / _AXONE_NORMAL_SPEED_;
		retval->axone_sd = (
			absolute_length(0,
				0,
				0,
				cluster_src->delta_L,
				cluster_src->delta_H,
				cluster_src->delta_AP) +
			absolute_length(0,
				0,
				0,	
				cluster_tgt->delta_L,
				cluster_tgt->delta_H,
				cluster_tgt->delta_AP) ) / _AXONE_NORMAL_SPEED_;
		retval->min_knobs = 1;
		retval->max_knobs = 10;
		retval->nmda = False;
		retval->type = _FULL_EXCITATION_;		
		
		current_nucleus->link_array = link_array;
		
	}
	
	current_nucleus->nb_links++;
	return (LINK_PARAMETERS *)retval;
	
}

/*******************************************************************************
	Function for canceling single link when newly created
*******************************************************************************/
void cancel_single_link(void)
{

	delete_link(&current_link);
}
/*******************************************************************************
	Function for canceling generic link when newly created
*******************************************************************************/
void cancel_generic_link(void)
{
	delete_link(&current_link);
}

/*******************************************************************************
	Function for deleting a specified link
*******************************************************************************/
void delete_link(LINK_PARAMETERS **link)
{

	CLUSTER_WIDGET **cluster_array;
	CLUSTER_WIDGET *cluster_tmp;
	NEURON **neuron_array;
	NEURON *neuron_src;
	NEURON *neuron_tgt;
	Nucleus_widget *nucleus_tmp;
	int i,j,k;
	LINK_PARAMETERS *link_tmp;
	CLUSTER_WIDGET *cluster_src;
	CLUSTER_WIDGET *cluster_tgt;
	Nucleus_widget *nucleus_src;
	Nucleus_widget *nucleus_tgt;
	
	if((*link)->level == _NETWORK_)
	{
		nucleus_src = search_nucleus((*link)->source_id);
		nucleus_tgt = search_nucleus((*link)->target_id);
		
		if(!nucleus_src)
			return;
		if(nucleus_src == nucleus_tgt)
			nucleus_src->color = grey;
		
		
		for(j=0 ; j<nucleus_src->nb_links ;j++)
		{
			link_tmp = nucleus_src->link_array[j];
		
			cluster_tgt = search_cluster(link_tmp->target_id,&nucleus_tmp);
			if(nucleus_tmp != nucleus_tgt)
				continue;
			
			
			delete_link(&nucleus_src->link_array[j]);
			
		}
		
		if((*link)->source)
		{
			(*link)->source->extremity = False;
			(*link)->source = NULL;
		}
		if((*link)->target)
		{
			(*link)->target->extremity = False;
			(*link)->target = NULL;
		}
		
		(*link)->type = _FULL_NOT_CONNECTED_;
		
		
		refresh_links();
		*link = NULL;
		
	}
	else if ((*link)->level == _NUCLEUS_)
	{
		cluster_src = search_cluster((*link)->source_id,&nucleus_tmp);
		cluster_tgt = search_cluster((*link)->target_id,&nucleus_tmp);
		
		
		for(i=0 ; i<cluster_src->nb_neurons ; i++)
		{
			neuron_src = cluster_src->neuron_array[i];
			if(!neuron_src->connected_neurons)
				continue;
			for(j=0 ; j<neuron_src->connected_neurons ; j++)
			{
				neuron_tgt = search_neuron(neuron_src->connections[j].target_id,
					&cluster_tmp);
				
				if(cluster_tgt == cluster_tmp)
				{
					if(neuron_tgt == neuron_src)
						neuron_src->color = grey;
					neuron_src->connections[j].target_id = 0;		
				}
			}
			neuron_src->connections = refresh_connections(neuron_src);
				
		}
		(*link)->target_id = (*link)->source_id = 0;
		if((*link)->source)
		{	
			(*link)->source = NULL;
		}
		if((*link)->target)
		{
			(*link)->target = NULL;
		}
		(*link)->type = _FULL_NOT_CONNECTED_;
		
	}
	
	
}

/*******************************************************************************
	Function for updating the connections array of the specified neuron
*******************************************************************************/
Connection *refresh_connections(NEURON *neuron)
{
	Connection *connections;
	int nb_connections = neuron->connected_neurons;
	int i,j;
	
	for(i=0 ; i<neuron->connected_neurons ; i++)
	{
		if(!neuron->connections[i].target_id)
			nb_connections--;
	}
	
	if(!nb_connections)
	{
		neuron->connected_neurons = 0;
		free(neuron->connections);
		return NULL;
	}
	
	connections = (Connection *)calloc(1,nb_connections*sizeof(Connection));
	
	j = 0;
	for(i=0 ; i<neuron->connected_neurons ; i++)
	{
		if(neuron->connections[i].target_id)
		{
			memcpy(&connections[j],&neuron->connections[i],sizeof(Connection));
			j++;
		}
	}
	neuron->connected_neurons = nb_connections;
	free(neuron->connections);	
	return (Connection *)connections;

}

/*******************************************************************************
	Function for updating the links of the whole network avoiding memory holes
*******************************************************************************/
void refresh_links(void)
{
	int i,j,k,p;
	Nucleus_widget **nucleus_array,*nucleus_tmp;
	CLUSTER_WIDGET **cluster_array,*cluster_tmp;
	NEURON **neuron_array,*neuron_tmp;
	LINK_PARAMETERS **link_array,*link_tmp;
	int nb_links;
	LINK_PARAMETERS **new_link_array;

	if(!current_network)
		return;
	
	nucleus_array = current_network->nucleus_array;
	if(nucleus_array)
	{
		for(i=0 ; i<current_network->nb_nucleus ; i++)
		{
			nucleus_tmp = nucleus_array[i];
			if(nucleus_tmp)
			{
				nb_links = nucleus_tmp->nb_links;			
				link_array = nucleus_tmp->link_array;
				if(link_array)
				{
					for(j=0 ; j<nucleus_tmp->nb_links ; j++)
					{
						link_tmp = link_array[j];
						if(link_tmp->type == _FULL_NOT_CONNECTED_)
						{
							free(nucleus_tmp->link_array[j]);
							nucleus_tmp->link_array[j] = NULL;
							nb_links--;
							
						}
					}
					
					if(!nb_links)
					{
						free(nucleus_tmp->link_array);
						nucleus_tmp->link_array = NULL;
						nucleus_tmp->nb_links = 0;
						continue;
					}
					
					new_link_array = (LINK_PARAMETERS **)calloc(1,nb_links*sizeof(
						LINK_PARAMETERS *));
					
					p = 0;	
					for(j=0 ; j<nucleus_tmp->nb_links ; j++)
					{
						link_tmp = link_array[j];
						if(link_tmp)
						{
							new_link_array[p] = (LINK_PARAMETERS *)calloc(1,
								sizeof(LINK_PARAMETERS));
							memcpy(new_link_array[p],link_array[j],sizeof(LINK_PARAMETERS));
							p++;
							free(link_array[j]);							
						}
					}
					
					free(link_array);
					nucleus_tmp->nb_links =  nb_links;	
					nucleus_tmp->link_array = new_link_array;
				}
				
				
			}
		}
		
	}	
	
	link_array = current_network->link_array;
	nb_links = current_network->nb_links;
	if(link_array)
	{
		for(j=0 ; j<current_network->nb_links ; j++)
		{
			link_tmp = link_array[j];
			if(link_tmp->type == _FULL_NOT_CONNECTED_)
			{
				if(link_tmp->target_id == link_tmp->source_id)
					(search_nucleus(link_tmp->source_id))->color = grey;
				free(current_network->link_array[j]);
				current_network->link_array[j] = NULL;
				nb_links--;

			}
		}

		if(!nb_links)
		{
			free(current_network->link_array);
			current_network->link_array = NULL;
			current_network->nb_links = 0;
			return;
		}

		new_link_array = (LINK_PARAMETERS **)calloc(1,nb_links*sizeof(
			LINK_PARAMETERS *));

		p = 0;	
		for(j=0 ; j<current_network->nb_links ; j++)
		{
			link_tmp = link_array[j];
			if(link_tmp)
			{
				new_link_array[p] = (LINK_PARAMETERS *)calloc(1,
					sizeof(LINK_PARAMETERS));
				memcpy(new_link_array[p],link_array[j],sizeof(LINK_PARAMETERS));
				p++;
				free(link_array[j]);							
			}
		}

		free(link_array);
		current_network->nb_links =  nb_links;	
		current_network->link_array = new_link_array;
	}
		
	

}

/*******************************************************************************
	Function that return the link matched with the specified ids on the 
	specified level
*******************************************************************************/
LINK_PARAMETERS *search_link(int source_id,int target_id,int level)
{
	int i,j;

	if(level == _NETWORK_)
	{
		
		for(i=0 ; i<current_network->nb_links ; i++)
		{
			if((source_id == current_network->link_array[i]->source_id) &&
				(target_id == current_network->link_array[i]->target_id))
				return (LINK_PARAMETERS *)current_network->link_array[i];
		}
		
	}
	else if(level == _NUCLEUS_)
	{
		for(j=0 ; j<current_network->nb_nucleus ; j++)
		{
			for(i=0 ; i<current_network->nucleus_array[j]->nb_links ; i++)
			{
				if((source_id == 
				current_network->nucleus_array[j]->link_array[i]->source_id) &&
					(target_id == 
				current_network->nucleus_array[j]->link_array[i]->target_id))
					return (LINK_PARAMETERS *)
				current_network->nucleus_array[j]->link_array[i];
			}
		}
		
	
	}
	
	return NULL;

}

/*******************************************************************************
	Function for modifying an existing link
*******************************************************************************/
void modify_link(int source_id,int target_id,int level)
{

	CLUSTER_WIDGET *cluster_src;
	CLUSTER_WIDGET *cluster_tgt;
	Nucleus_widget *nucleus_src;
	Nucleus_widget *nucleus_tgt;
	NEURON *neuron_src;
	NEURON *neuron_tgt;
	int pos;
	
	char buffer[255];
	
	if(level == _NETWORK_)
	{
		nucleus_src = search_nucleus(source_id);
		nucleus_tgt = search_nucleus(target_id);
		current_link = search_link(source_id,target_id,level);
	}
	else if(level == _NUCLEUS_)
	{
		neuron_src = search_neuron(source_id,&cluster_src);
		neuron_tgt = search_neuron(target_id,&cluster_tgt);
		current_link = search_link(cluster_src->cluster_id,
			cluster_tgt->cluster_id,level);
		if(!current_link)
		{
			current_link = search_link(current_nucleus->nucleus_id,
				current_nucleus->nucleus_id,_NETWORK_);
			current_link = add_new_link_in_nucleus(current_nucleus,
				cluster_src->cluster_id,cluster_tgt->cluster_id);
		}
		memcpy(&link_glb,current_link,sizeof(LINK_PARAMETERS));
	}

	
	
	if(!current_link)
	{
		display_error(widget_array[ID_MainW_network],
		"Sorry, this link doesn't exists...",
	"No Link to Modify !");
		return;
	}

	if(level == _NETWORK_)
	{
		if((nucleus_tgt->nb_neurons != 1) ||
			(nucleus_src->nb_neurons != 1))
		{
			
			
			sprintf(buffer,"Generic Link Editor: %s to %s",
				nucleus_src->nucleus_name,
				nucleus_tgt->nucleus_name);
			XtVaSetValues(XtParent(widget_array[ID_FormD_generic_link_edit]),
				XmNtitle,buffer,NULL);
			
			XmTextRemove(widget_array[ID_Text_axone_mean]);
			sprintf(buffer,"%f",current_link->axone_mean);
			XmTextReplace(widget_array[ID_Text_axone_mean],0,
			XmTextGetLastPosition(widget_array[ID_Text_axone_mean]),
				buffer);
				
			XmTextRemove(widget_array[ID_Text_axone_deviation]);
			sprintf(buffer,"%f",current_link->axone_sd);
			XmTextReplace(widget_array[ID_Text_axone_deviation],0,
			XmTextGetLastPosition(widget_array[ID_Text_axone_deviation]),
				buffer);
				
			XmTextRemove(widget_array[ID_Text_max_knobs]);
			sprintf(buffer,"%d",current_link->max_knobs);
			XmTextReplace(widget_array[ID_Text_max_knobs],0,
			XmTextGetLastPosition(widget_array[ID_Text_max_knobs]),
				buffer);
				
			XmTextRemove(widget_array[ID_Text_min_knobs]);
			sprintf(buffer,"%d",current_link->min_knobs);
			XmTextReplace(widget_array[ID_Text_min_knobs],0,
			XmTextGetLastPosition(widget_array[ID_Text_min_knobs]),
				buffer);
			
			XmToggleButtonSetState(widget_array[ID_ToggleB_NMDA],
				(current_link->nmda ? True : False),True);
			
			switch(current_link->type)
			{
				case _FULL_EXCITATION_:
					XmToggleButtonSetState(widget_array[ID_ToggleB_fully_connected],
						True,True);
					XmToggleButtonSetState(widget_array[ID_ToggleB_algo_all_excit],
						True,True);	
					break;
				
				case _FULL_INHIBITION_:
					XmToggleButtonSetState(widget_array[ID_ToggleB_fully_connected],
						True,True);
					XmToggleButtonSetState(widget_array[ID_ToggleB_algo_all_inhib],
						True,True);
					break;
				
				case _FULL_RANDOMIZED_:
					XmToggleButtonSetState(widget_array[ID_ToggleB_fully_connected],
						True,True);
					XmToggleButtonSetState(widget_array[ID_ToggleB_algo_random],
						True,True);
					break;
				
				case _PART_EXCITATION_:
					XmToggleButtonSetState(widget_array[ID_ToggleB_partially_connected],
						True,True);
					XmToggleButtonSetState(widget_array[ID_ToggleB_algo_all_excit],
						True,True);
					break;
				
				case _PART_INHIBITION_:
					XmToggleButtonSetState(widget_array[ID_ToggleB_partially_connected],
						True,True);
					XmToggleButtonSetState(widget_array[ID_ToggleB_algo_all_inhib],
						True,True);
					break;
				
				case _PART_RANDOMIZED_:
					XmToggleButtonSetState(widget_array[ID_ToggleB_partially_connected],
						True,True);
					XmToggleButtonSetState(widget_array[ID_ToggleB_algo_random],
						True,True);
					break;
				
				default:
					break;
			}
			
			if(!XtIsManaged(widget_array[ID_FormD_generic_link_edit]))
			{
				XtManageChild(widget_array[ID_FormD_generic_link_edit]);
				XSetWindowColormap(the_display, XtWindow(XtParent(widget_array[ID_FormD_generic_link_edit])),
						the_colormap);
			}
			else
				XtPopup(XtParent(widget_array[ID_FormD_generic_link_edit]),
					XtGrabNone);
		
		}
		else
		{
		
			
			sprintf(buffer,"Single Link Editor: %s to %s",
				nucleus_src->nucleus_name,
				nucleus_tgt->nucleus_name);
			XtVaSetValues(XtParent(widget_array[ID_FormD_link_simple]),
				XmNtitle,buffer,NULL);
				
			XmTextFieldRemove(widget_array[ID_TextF_axone_length]);
			sprintf(buffer,"%f",current_link->axone_mean);
			XmTextFieldReplace(widget_array[ID_TextF_axone_length],0,
			XmTextFieldGetLastPosition(widget_array[ID_TextF_axone_length]),
				buffer);
				
			XmTextFieldRemove(widget_array[ID_TextF_synaptic_knobs]);
			sprintf(buffer,"%d",current_link->max_knobs);
			XmTextFieldReplace(widget_array[ID_TextF_synaptic_knobs],0,
			XmTextFieldGetLastPosition(widget_array[ID_TextF_synaptic_knobs]),
				buffer);
		
			XmToggleButtonSetState(widget_array[ID_ToggleB_link_simple_NMDA],
				(current_link->nmda ? True : False),True);
			
			switch(current_link->type)
			{
				case _PART_EXCITATION_:
				case _FULL_EXCITATION_:
					XmToggleButtonSetState(widget_array[ID_ToggleB_dialog_excit],
						True,True);
					
					break;
				
				case _FULL_INHIBITION_:
				case _PART_INHIBITION_:
					XmToggleButtonSetState(widget_array[ID_ToggleB_dialog_inhib],
						True,True);
					break;
					
				default:
					break;
			}	
			if(!XtIsManaged(widget_array[ID_FormD_link_simple]))
			{
				XtManageChild(widget_array[ID_FormD_link_simple]);
				XSetWindowColormap(the_display, XtWindow(XtParent(widget_array[ID_FormD_link_simple])),
						the_colormap);
			}
			else
				XtPopup(XtParent(widget_array[ID_FormD_link_simple]),
					XtGrabNone);
					
		}
		
	}
	else if(level == _NUCLEUS_)
	{
		if(!search_connection(neuron_src->neuron_id,
			neuron_tgt->neuron_id,&pos))
		{
			display_error(widget_array[ID_FormD_Nucleus],
			"This connection doesn't exist yet...","No Link to Modify !");
			return;
		}
		if((cluster_tgt->nb_neurons != 1) &&
			(cluster_src->nb_neurons != 1))
		{
			sprintf(buffer,"Single Link Editor: .neuron_%d to .neuron_%d",
				neuron_src->neuron_id,
				neuron_tgt->neuron_id);
			XtVaSetValues(XtParent(widget_array[ID_FormD_link_simple]),
				XmNtitle,buffer,NULL);
		}
		else if((cluster_tgt->nb_neurons == 1) &&
			(cluster_src->nb_neurons != 1))
		{
			sprintf(buffer,"Single Link Editor: .neuron_%d to  neuron_%d",
				neuron_src->neuron_id,
				neuron_tgt->neuron_id);
			XtVaSetValues(XtParent(widget_array[ID_FormD_link_simple]),
				XmNtitle,buffer,NULL);
		}
		else if((cluster_tgt->nb_neurons != 1) &&
			(cluster_src->nb_neurons == 1))
		{
			sprintf(buffer,"Single Link Editor: neuron_%d to .neuron_%d",
				neuron_src->neuron_id,
				neuron_tgt->neuron_id);
			XtVaSetValues(XtParent(widget_array[ID_FormD_link_simple]),
				XmNtitle,buffer,NULL);
		}
		else if((cluster_tgt->nb_neurons == 1) &&
			(cluster_src->nb_neurons == 1))
		{
			sprintf(buffer,"Single Link Editor: neuron_%d to neuron_%d",
				cluster_src->ch_name,neuron_src->neuron_id,
				cluster_tgt->ch_name,neuron_tgt->neuron_id);
			XtVaSetValues(XtParent(widget_array[ID_FormD_link_simple]),
				XmNtitle,buffer,NULL);
		}
				
				
		XmTextFieldRemove(widget_array[ID_TextF_axone_length]);
		sprintf(buffer,"%f",neuron_src->connections[pos].axone_length);
		XmTextFieldReplace(widget_array[ID_TextF_axone_length],0,
		XmTextFieldGetLastPosition(widget_array[ID_TextF_axone_length]),
			buffer);

		if(neuron_src->connections[pos].synaptic_knobs > 0)
		{
			XmTextFieldRemove(widget_array[ID_TextF_synaptic_knobs]);
			sprintf(buffer,"%d",neuron_src->connections[pos].synaptic_knobs);
			XmTextFieldReplace(widget_array[ID_TextF_synaptic_knobs],0,
			XmTextFieldGetLastPosition(widget_array[ID_TextF_synaptic_knobs]),
				buffer);
			XmToggleButtonSetState(widget_array[ID_ToggleB_dialog_excit],
						True,True);
		}
		else
		{
			XmTextFieldRemove(widget_array[ID_TextF_synaptic_knobs]);
			sprintf(buffer,"%d",-neuron_src->connections[pos].synaptic_knobs);
			XmTextFieldReplace(widget_array[ID_TextF_synaptic_knobs],0,
			XmTextFieldGetLastPosition(widget_array[ID_TextF_synaptic_knobs]),
				buffer);
			XmToggleButtonSetState(widget_array[ID_ToggleB_dialog_inhib],
					True,True);
		}
		XmToggleButtonSetState(widget_array[ID_ToggleB_link_simple_NMDA],
			( neuron_src->connections[pos].nmda ? True : False),True);

		if(!XtIsManaged(widget_array[ID_FormD_link_simple]))
		{
			XtManageChild(widget_array[ID_FormD_link_simple]);
			XSetWindowColormap(the_display, XtWindow(XtParent(widget_array[ID_FormD_link_simple])),
						the_colormap);
		}
		else
			XtPopup(XtParent(widget_array[ID_FormD_link_simple]),
				XtGrabNone);
	}


}
/*******************************************************************************
	Function for applying a single neuron to neuron link
*******************************************************************************/
int apply_neuron_to_neuron_link(void)
{
	char *buffer;
	Nucleus_widget *nucleus_src;
	Nucleus_widget *nucleus_tgt;
	CLUSTER_WIDGET *cluster_src;
	CLUSTER_WIDGET *cluster_tgt;
	NEURON *neuron_src;
	NEURON *neuron_tgt;
	LINK_PARAMETERS *link_tmp;
	LINK_PARAMETERS **new_link_array;
	int i,j,pos;

	if(!current_link)
	{
		fprintf(stderr,"current_link is NULL in apply_neuron_to_neuron_link!\n");
		exit(-1);
	}

	buffer = XmTextFieldGetString(widget_array[ID_TextF_axone_length]);
	if((atof(buffer)>1000000.0)||(atof(buffer)<=0))
	{
		display_error(widget_array[ID_FormD_link_simple],
			"Sorry, the axone length value isn't available...","Bad Axone Length !");
		return -1;
	}
	current_link->axone_mean = atof(buffer);
	XtFree(buffer);	
	
	buffer = XmTextFieldGetString(widget_array[ID_TextF_synaptic_knobs]);
	if((atof(buffer)>1000000)||(atof(buffer)<=0))
	{
		display_error(widget_array[ID_FormD_link_simple],
			"Sorry, the synaptic knobs number isn't available...",
			"Bad Knobs Number !");
		return -1;
	}
	current_link->max_knobs = current_link->min_knobs = atoi(buffer);
	XtFree(buffer);
	
	if(XmToggleButtonGetState(widget_array[ID_ToggleB_dialog_excit]))
		current_link->type = _FULL_EXCITATION_;
	else if(XmToggleButtonGetState(widget_array[ID_ToggleB_dialog_inhib]))
		current_link->type = _FULL_INHIBITION_;
	
	if(XmToggleButtonGetState(widget_array[ID_ToggleB_link_simple_NMDA]))
		current_link->nmda = True;
	else
		current_link->nmda = False;	
	
	if(current_link->points_placed == False)
	{
		for(i=0 ; i<10 ; i++)
		{
			current_link->points[i].probability = 100.0;
			current_link->points[i].length = (2*i+1)*5.0;
		}
		current_link->points_placed = True;
	}
	
	if(current_link->level == _NETWORK_)
	{
		nucleus_src = search_nucleus(current_link->source_id);
		nucleus_tgt = search_nucleus(current_link->target_id);
		if(!(link_tmp = search_link(nucleus_src->cluster_array[0]->cluster_id ,
			nucleus_tgt->cluster_array[0]->cluster_id,_NUCLEUS_)))
		{
			link_tmp = add_new_link_in_nucleus(nucleus_src,
				nucleus_src->cluster_array[0]->cluster_id,
				nucleus_tgt->cluster_array[0]->cluster_id);
			
			neuron_src = nucleus_src->cluster_array[0]->neuron_array[0];
			neuron_tgt = nucleus_tgt->cluster_array[0]->neuron_array[0];

			if(!search_connection(neuron_src->neuron_id,neuron_tgt->neuron_id,&pos))
			{
				neuron_src->connections =
					 add_new_connection(neuron_src->neuron_id,neuron_tgt->neuron_id,
						&pos);
			}

			neuron_src->connections[pos].target_id = neuron_tgt->neuron_id;

			neuron_src->connections[pos].nmda = current_link->nmda;
			switch(current_link->type)
			{
				case _PART_EXCITATION_:
				case _FULL_EXCITATION_:
					neuron_src->connections[pos].synaptic_knobs
						 = current_link->max_knobs;
					break;

				case _FULL_INHIBITION_:
				case _PART_INHIBITION_:
					neuron_src->connections[pos].synaptic_knobs
						 = - current_link->max_knobs;
					break;

				default:
					break;
			}

			neuron_src->connections[pos].axone_length = current_link->axone_mean;
			
		}
		else
		{
			neuron_src = nucleus_src->cluster_array[0]->neuron_array[0];
			neuron_tgt = nucleus_tgt->cluster_array[0]->neuron_array[0];
			
			if(!search_connection(neuron_src->neuron_id,neuron_tgt->neuron_id,&pos))
			{
				fprintf(stderr,"target id is bad for one to one connec in apply..!!\n");
				exit(-1);
			}
			neuron_src->connections[pos].nmda = current_link->nmda;
			switch(current_link->type)
			{
				case _PART_EXCITATION_:
				case _FULL_EXCITATION_:
					neuron_src->connections[pos].synaptic_knobs
						 = current_link->max_knobs;
					break;
				
				case _FULL_INHIBITION_:
				case _PART_INHIBITION_:
					neuron_src->connections[pos].synaptic_knobs
						 = - current_link->max_knobs;
					break;
					
				default:
					break;
			}
			
			neuron_src->connections[pos].axone_length = current_link->axone_mean;
		}
	}	
	else if(current_link->level == _NUCLEUS_)
	{
		if(!search_connection(previous_selected_neuron->neuron_id,
			current_neuron->neuron_id,&pos))
		{
			previous_selected_neuron->connections =
				 add_new_connection(previous_selected_neuron->neuron_id,
				 	current_neuron->neuron_id,
					&pos);
		}
		previous_selected_neuron->connections[pos].nmda = current_link->nmda;
		previous_selected_neuron->connections[pos].axone_length = current_link->axone_mean;
		switch(current_link->type)
		{
			case _PART_EXCITATION_:
			case _FULL_EXCITATION_:
				previous_selected_neuron->connections[pos].synaptic_knobs
					 = current_link->max_knobs;
				break;

			case _FULL_INHIBITION_:
			case _PART_INHIBITION_:
				previous_selected_neuron->connections[pos].synaptic_knobs
					 = - current_link->max_knobs;
				break;

			default:
				break;
		}
		memcpy(current_link,&link_glb,sizeof(LINK_PARAMETERS));
						
	}
	
	return 0;
}

/*******************************************************************************
	Function for applying a generic link
*******************************************************************************/
int apply_generic_link()
{
	char *buffer;
	Nucleus_widget *nucleus_src;
	Nucleus_widget *nucleus_tgt;
	CLUSTER_WIDGET *cluster_src;
	CLUSTER_WIDGET *cluster_tgt;
	NEURON *neuron_src;
	NEURON *neuron_tgt;
	LINK_PARAMETERS *link_tmp;
	int i;

	if(!current_link)
	{
		fprintf(stderr,"current_link is NULL in apply_generic_link!\n");
		exit(-1);
	}

	buffer = XmTextGetString(widget_array[ID_Text_axone_mean]);
	if((atof(buffer)>1000000.0)||(atof(buffer)<=0))
	{
		display_error(widget_array[ID_FormD_generic_link_edit],
			"Sorry, the axone mean value isn't available...","Bad Axone Mean !");
		return -1;
	}
	current_link->axone_mean = atof(buffer);
	XtFree(buffer);	
	
	buffer = XmTextGetString(widget_array[ID_Text_axone_deviation]);
	if((atof(buffer)>current_link->axone_mean)||(atof(buffer)<0))
	{
		display_error(widget_array[ID_FormD_generic_link_edit],
			"Sorry, the axone deviation value isn't available...",
				"Bad Axone Deviation !");
		return -1;
	}
	current_link->axone_sd = atof(buffer);
	XtFree(buffer);
	
	buffer = XmTextGetString(widget_array[ID_Text_max_knobs]);
	if((atof(buffer)>1000000)||(atof(buffer)<=0))
	{
		display_error(widget_array[ID_FormD_generic_link_edit],
			"Sorry, the synaptic knobs maxmimum value isn't available...",
			"Bad Knobs Number !");
		return -1;
	}
	current_link->max_knobs = atoi(buffer);
	XtFree(buffer);
	
	buffer = XmTextGetString(widget_array[ID_Text_min_knobs]);
	if((atof(buffer)>current_link->max_knobs)||(atof(buffer)<=0))
	{
		display_error(widget_array[ID_FormD_generic_link_edit],
			"Sorry, the synaptic knobs minimum value isn't available...",
			"Bad Knobs Number !");
		return -1;
	}
	current_link->min_knobs = atoi(buffer);
	XtFree(buffer);
	
	if(XmToggleButtonGetState(widget_array[ID_ToggleB_fully_connected]))
	{
		if(XmToggleButtonGetState(widget_array[ID_ToggleB_algo_all_excit]))
			current_link->type = _FULL_EXCITATION_;
		else if(XmToggleButtonGetState(widget_array[ID_ToggleB_algo_all_inhib]))
			current_link->type = _FULL_INHIBITION_;
		else if(XmToggleButtonGetState(widget_array[ID_ToggleB_algo_random]))
			current_link->type = _FULL_RANDOMIZED_;
	}
	else if(XmToggleButtonGetState(widget_array[ID_ToggleB_partially_connected]))
	{
		if(XmToggleButtonGetState(widget_array[ID_ToggleB_algo_all_excit]))
			current_link->type = _PART_EXCITATION_;
		else if(XmToggleButtonGetState(widget_array[ID_ToggleB_algo_all_inhib]))
			current_link->type = _PART_INHIBITION_;
		else if(XmToggleButtonGetState(widget_array[ID_ToggleB_algo_random]))
			current_link->type = _PART_RANDOMIZED_;
	}
		
	if(XmToggleButtonGetState(widget_array[ID_ToggleB_NMDA]))
		current_link->nmda = True;
	else
		current_link->nmda = False;
		
	if(current_link->level == _NETWORK_)
	{
		if(!XmToggleButtonGetState(widget_array[ID_ToggleB_partially_connected]))
		{	
			if(!current_link->source)
				current_link->source = 
					place_link_extremity(current_link->source_id);
			if(!current_link->target)
				current_link->target = 
					place_link_extremity(current_link->target_id);
			if(current_link->points_placed == False)
			{
				for(i=0 ; i<10 ; i++)
				{
					current_link->points[i].probability = 100.0;
					current_link->points[i].length = i*10.0;
				}
				current_link->points_placed = False;
			}
			if(changed)
			{
				create_default_generic_link();
				
			}
		}
		else 
		{
			if(!current_link->source)
				current_link->source = 
					place_link_extremity(current_link->source_id);
			if(!current_link->target)
				current_link->target = 
					place_link_extremity(current_link->target_id);
			if(current_link->points_placed == False)
			{
				for(i=0 ; i<10 ; i++)
				{
					current_link->points[i].probability = 100.0 - (2*i+1)*5.0;
					current_link->points[i].length = (2*i+1)*5.0;
				}
				current_link->points_placed = True;
			}
			if(changed)
			{
				create_default_generic_link();

			}
		}
			
	}
	else if(current_link->level == _NUCLEUS_)
	{
	
	}	
	
	
	return 0;
}

/*******************************************************************************
	Function for allocating and filling with defaults value an whole
	generic link according to the current link structure
*******************************************************************************/
void create_default_generic_link(void)
{
	Nucleus_widget *nucleus_src;
	Nucleus_widget *nucleus_tgt;
	CLUSTER_WIDGET *cluster_src;
	CLUSTER_WIDGET *cluster_tgt;
	NEURON *neuron_src;
	NEURON *neuron_tgt;
	LINK_PARAMETERS *link_tmp;
	LINK_PARAMETERS **new_link_array;
	int i,j;	


	if(!changed)
		return;
	else
		changed = False;

	if(!current_link)
	{
		fprintf(stderr,"current_link is NULL in create_default_generic_link!!!\n");
		exit(-1);
	}

	if(current_link->level != _NETWORK_)
	{
		fprintf(stderr,"link level isn't _NETWORK_ in create_default_generic_link!!!\n");
		exit(-1);
	}
	
	nucleus_src = search_nucleus(current_link->source_id);
	nucleus_tgt = search_nucleus(current_link->target_id);
	
	for(i=0 ; i<nucleus_src->nb_clusters ; i++)
	{
		cluster_src = nucleus_src->cluster_array[i];
		for(j=0 ; j<nucleus_tgt->nb_clusters ; j++)
		{
			cluster_tgt = nucleus_tgt->cluster_array[j];
			link_tmp = search_link(cluster_src->cluster_id,
				cluster_tgt->cluster_id,_NUCLEUS_);
			if(!link_tmp)
			{
				link_tmp = add_new_link_in_nucleus(nucleus_src,
					cluster_src->cluster_id,cluster_tgt->cluster_id);
			}
			else
			{
				link_tmp->axone_mean = current_link->axone_mean;
				link_tmp->axone_sd = current_link->axone_sd;
				link_tmp->min_knobs = current_link->min_knobs;
				link_tmp->max_knobs = current_link->max_knobs;
				link_tmp->nmda = current_link->nmda;
				link_tmp->type = current_link->type;
				link_tmp->level = _NUCLEUS_;
				link_tmp->source_id = cluster_src->cluster_id;
				link_tmp->target_id = cluster_tgt->cluster_id;
				link_tmp->points_placed = current_link->points_placed;
				memcpy(link_tmp->points,current_link->points,
					_NB_INTERPOLATING_POINTS_*sizeof(PROBA_POINT));
			}
			update_connections(link_tmp);
		}
	}	

}

/*******************************************************************************
	Function for adding a new link in a nucleus level
*******************************************************************************/
LINK_PARAMETERS *add_new_link_in_nucleus(Nucleus_widget *nucleus,
	int source_id,int target_id)
{
	LINK_PARAMETERS **new_link_array;
	LINK_PARAMETERS *link_tmp;
	CLUSTER_WIDGET *cluster_src,*cluster_tgt;
	Nucleus_widget *nucleus_tmp;
	int i;
	
	if(!current_link)
	{
		fprintf(stderr,"current link is NULL in add_new_link_in_nucleus!!!!\n");
		exit(-1);

	}
	
	
	if(nucleus->nb_links>0)
	{
		new_link_array = (LINK_PARAMETERS **)calloc(1,
			(nucleus->nb_links+1)*sizeof(LINK_PARAMETERS *));
		for(i=0 ; i<nucleus->nb_links ; i++)
		{
			new_link_array[i] = nucleus->link_array[i];
		}

		new_link_array[nucleus->nb_links] = (LINK_PARAMETERS *)calloc(1,sizeof(
			LINK_PARAMETERS));

		memcpy(new_link_array[nucleus->nb_links],current_link,sizeof(LINK_PARAMETERS));

		new_link_array[nucleus->nb_links]->source_id = source_id;
			
		new_link_array[nucleus->nb_links]->target_id = target_id;
		
		new_link_array[nucleus->nb_links]->level = _NUCLEUS_;
			

		free(nucleus->link_array);
		nucleus->link_array = new_link_array;
		nucleus->nb_links++;
		return (LINK_PARAMETERS *)nucleus->link_array[nucleus->nb_links-1];


	}
	else
	{
		nucleus->link_array = (LINK_PARAMETERS **)calloc(1,sizeof(
			LINK_PARAMETERS *));
		nucleus->link_array[0] = (LINK_PARAMETERS *)calloc(1,sizeof(
			LINK_PARAMETERS));

		memcpy(nucleus->link_array[0],current_link,sizeof(LINK_PARAMETERS));

		nucleus->link_array[0]->source_id = source_id;
			
		nucleus->link_array[0]->target_id = target_id;
		
		nucleus->link_array[0]->level = _NUCLEUS_;
			

		nucleus->nb_links++;
		return (LINK_PARAMETERS *)nucleus->link_array[0];

	}
}

/*******************************************************************************
	Function for updating a nucleus level  generic link
*******************************************************************************/
void update_connections(LINK_PARAMETERS *link_tmp)
{
	
	NEURON *neuron_src;
	NEURON *neuron_tgt;
	NEURON *cnx_center;
	CLUSTER_WIDGET *cluster_src;
	CLUSTER_WIDGET *cluster_tgt;
	Nucleus_widget *nucleus_src;
	Nucleus_widget *nucleus_tgt;
	int	set;
	float axone_length;
	int synaptic_knobs;
	int i,j;
	int pos;
	float d_src,d_tgt;
	float d_src_max,d_tgt_max;
	float proba;
	float coef,res;
	int intra;
	
	
	
	
	if(link_tmp->level != _NUCLEUS_)
	{
		fprintf(stderr,"link level isn't _NUCLEUS_ in update_connections !!!!\n");
		exit(-1);
	}
	set = (int) time (0) & 077;			
		
	cluster_src = search_cluster(link_tmp->source_id,&nucleus_src);
	
	cluster_tgt = search_cluster(link_tmp->target_id,&nucleus_tgt);
	d_tgt_max = absolute_length(0,
							0,
							0,
							nucleus_tgt->delta_L,
							nucleus_tgt->delta_H,
							nucleus_tgt->delta_AP);
	if(d_tgt_max<1.0)
		d_tgt_max = 1.0;	
	
	if(nucleus_tgt == nucleus_src)
		intra = 1;
	else 
		intra = 0;
							
	for(i=0 ; i<cluster_src->nb_neurons ; i++)
	{
		neuron_src = cluster_src->neuron_array[i];
		
		cnx_center = place_link_extremity(nucleus_tgt->nucleus_id);
		cnx_center->extremity = False;
		
		for(j=0 ; j<cluster_tgt->nb_neurons ; j++)
		{
			neuron_tgt = cluster_tgt->neuron_array[j];
			
			axone_length = current_link->axone_mean
				+ (2. * ran1 (&set) - 1.)*current_link->axone_sd;
			synaptic_knobs = current_link->min_knobs
				+ (int)(ran1 (&set)*(current_link->max_knobs
					- current_link->min_knobs));
			
			d_tgt = absolute_length(neuron_tgt->anatomy.L,
							neuron_tgt->anatomy.H,
							neuron_tgt->anatomy.AP,
							cnx_center->anatomy.L,
							cnx_center->anatomy.H,
							cnx_center->anatomy.AP);
			if(!intra)
				coef = (d_tgt/(2*d_tgt_max))*100.0;
			else
				coef = (absolute_length(neuron_tgt->anatomy.L,
							neuron_tgt->anatomy.H,
							neuron_tgt->anatomy.AP,
							neuron_src->anatomy.L,
							neuron_src->anatomy.H,
							neuron_src->anatomy.AP)/(1.5*d_tgt_max))*100.0;
			
			switch(link_tmp->type)
			{
			
				case _FULL_NOT_CONNECTED_:
					if(search_connection(neuron_src->neuron_id,neuron_tgt->neuron_id,&pos))
					{
						neuron_src->connections[pos].synaptic_knobs = 0;
						neuron_src->connections[pos].nmda = 0;
						neuron_src->connections[pos].target_id = 0;
						if(neuron_src == neuron_tgt)
							neuron_src->color = grey;
					}
					break;
					
				case _FULL_EXCITATION_:
					if(neuron_tgt == neuron_src)
						neuron_src->color = red[4];
					if(!search_connection(neuron_src->neuron_id,neuron_tgt->neuron_id,&pos))
					{
						neuron_src->connections =
							 add_new_connection(neuron_src->neuron_id,neuron_tgt->neuron_id,
							 	&pos);
					}
					neuron_src->connections[pos].nmda = current_link->nmda;
					neuron_src->connections[pos].synaptic_knobs = synaptic_knobs;	
					neuron_src->connections[pos].axone_length = axone_length;
					break;
				
				case _FULL_INHIBITION_:
					if(neuron_tgt == neuron_src)
						neuron_src->color = blue[4];
					if(!search_connection(neuron_src->neuron_id,neuron_tgt->neuron_id,&pos))
					{
						neuron_src->connections =
							 add_new_connection(neuron_src->neuron_id,neuron_tgt->neuron_id,
							 	&pos);
					}
					neuron_src->connections[pos].nmda = current_link->nmda;
					neuron_src->connections[pos].synaptic_knobs = -synaptic_knobs;	
					neuron_src->connections[pos].axone_length = axone_length;
					break;
				
				case _FULL_RANDOMIZED_:
					if(!search_connection(neuron_src->neuron_id,neuron_tgt->neuron_id,&pos))
					{
						neuron_src->connections =
							 add_new_connection(neuron_src->neuron_id,neuron_tgt->neuron_id,
							 	&pos);
					}
					if((res = (2. * ran1 (&set) - 1.))>=0.0)
					{
						if(neuron_tgt == neuron_src)
							neuron_src->color = red[4];
						neuron_src->connections[pos].synaptic_knobs = synaptic_knobs;
					}
					else if(res < 0.0)
					{
						if(neuron_tgt == neuron_src)
							neuron_src->color = blue[4];
						neuron_src->connections[pos].synaptic_knobs = -synaptic_knobs;
					}
					neuron_src->connections[pos].nmda = current_link->nmda;
					neuron_src->connections[pos].axone_length = axone_length;	
					break;
				
				case _PART_EXCITATION_:
					proba = interpolate_curve(coef,link_tmp)/100.0;
					proba *= (float)synaptic_knobs;
					
					if(!search_connection(neuron_src->neuron_id,neuron_tgt->neuron_id,&pos))
					{
						neuron_src->connections =
							 add_new_connection(neuron_src->neuron_id,neuron_tgt->neuron_id,
							 	&pos);
					}
					if(proba < 1.0)
					{
						neuron_src->connections[pos].target_id = 0;
						if(neuron_tgt == neuron_src)
							neuron_src->color = grey;
						break;
					}
					if(neuron_tgt == neuron_src)
						neuron_src->color = red[4];
					neuron_src->connections[pos].nmda = current_link->nmda;
					neuron_src->connections[pos].synaptic_knobs = (short)proba;	
					neuron_src->connections[pos].axone_length = axone_length;
						
					break;
				
				case _PART_INHIBITION_:
					proba = interpolate_curve(coef,link_tmp)/100.0;
					proba *= (float)synaptic_knobs;
					if(!search_connection(neuron_src->neuron_id,neuron_tgt->neuron_id,&pos))
					{
						neuron_src->connections =
							 add_new_connection(neuron_src->neuron_id,neuron_tgt->neuron_id,
							 	&pos);
					}
					if(proba < 1.0)
					{
						neuron_src->connections[pos].target_id = 0;
						if(neuron_tgt == neuron_src)
							neuron_src->color = grey;
						break;
					}
					if(neuron_tgt == neuron_src)
						neuron_src->color = blue[4];
					neuron_src->connections[pos].nmda = current_link->nmda;
					neuron_src->connections[pos].synaptic_knobs = -(short)proba;	
					neuron_src->connections[pos].axone_length = axone_length;
							
					break;
				
				case _PART_RANDOMIZED_:
					proba = interpolate_curve(coef,link_tmp)/100.0;
					proba *= (float)synaptic_knobs;
					if(!search_connection(neuron_src->neuron_id,neuron_tgt->neuron_id,&pos))
					{
						neuron_src->connections =
							 add_new_connection(neuron_src->neuron_id,neuron_tgt->neuron_id,
							 	&pos);
					}
					if(proba < 1.0)
					{
						neuron_src->connections[pos].target_id = 0;
						if(neuron_tgt == neuron_src)
							neuron_src->color = grey;
						break;
					}
					if((res = (2. * ran1 (&set) - 1.))>=0.0)
					{
						if(neuron_tgt == neuron_src)
							neuron_src->color = red[4];
						neuron_src->connections[pos].synaptic_knobs = (short)proba;
					}
					else if(res < 0.0)
					{
						if(neuron_tgt == neuron_src)
							neuron_src->color = blue[4];
						neuron_src->connections[pos].synaptic_knobs = -(short)proba;
					}
					neuron_src->connections[pos].nmda = current_link->nmda;
					neuron_src->connections[pos].axone_length = axone_length;
						
					break;
				
				default:
					break;
			}
		
		}
		neuron_src->connections = refresh_connections(neuron_src);
	
	}
	
	
}

/*******************************************************************************
	Function for adding a new connection
*******************************************************************************/
Connection *add_new_connection(int source_id,int target_id,int *pos)
{
	CLUSTER_WIDGET *cluster_tmp;
	NEURON *neuron_src;
	NEURON *neuron_target;
	Connection *new_connections;
	int i;
	
	neuron_src = search_neuron(source_id,&cluster_tmp);
	if(!neuron_src->connected_neurons)
	{
		new_connections = (Connection *)calloc(1,sizeof(Connection));
		new_connections[0].target_id = target_id;
		*pos = 0;
	}
	else
	{
		new_connections = (Connection *)calloc(1,(neuron_src->connected_neurons+1)*sizeof(
			Connection));
		for(i=0 ; i <neuron_src->connected_neurons ; i++)
			memcpy(&new_connections[i],&(neuron_src->connections[i]),
				sizeof(Connection));
		free(neuron_src->connections);
		new_connections[neuron_src->connected_neurons].target_id = target_id;
		*pos = neuron_src->connected_neurons;
	}
	neuron_src->connected_neurons++;
	return (Connection *)new_connections;
}

/*******************************************************************************
	Function for placing a network link extremity
*******************************************************************************/
NEURON *place_link_extremity(int nucleus_id)
{
	int	set;
	Nucleus_widget *nucleus_tmp;
	NEURON *extremity;
	int number;
	int i,j;
	
	
	set = (int) time (0) & 077;			
	
	nucleus_tmp = search_nucleus(nucleus_id);
	
	number = (int)(nucleus_tmp->nb_neurons*ran1(&set));
	number += nucleus_tmp->first_neuron;
	
	for(i=0 ; i<nucleus_tmp->nb_clusters ; i++)
	{
		for(j=0 ; j<nucleus_tmp->cluster_array[i]->nb_neurons ; j++)
		{
			extremity = nucleus_tmp->cluster_array[i]->neuron_array[j];
			if(extremity->number == number)
			{
				extremity->extremity = True;
				return (NEURON *)extremity;
			}
		}
		
	}
	
	fprintf(stderr,"extremity not found in place_link_extremity!!!\n");
	exit(-1);
}

/*******************************************************************************
	Function for searching one connection
*******************************************************************************/
Connection *search_connection(int source_id,int target_id,int *pos)
{
	CLUSTER_WIDGET *cluster_tmp;
	NEURON *neuron_src;
	NEURON *neuron_target;
	Connection connection;
	int i;
	
	*pos = -1;
	neuron_src = search_neuron(source_id,&cluster_tmp);
	if(!neuron_src->connected_neurons)
		return NULL;
		
	for(i=0 ; i<neuron_src->connected_neurons ; i++)
	{
		connection = neuron_src->connections[i];
		if(connection.target_id == target_id)
		{
			*pos = i;
			return (Connection *)neuron_src->connections;
		}
	}

	return NULL;
}

/*******************************************************************************
	Function that returns an interpolated probability of connection
*******************************************************************************/
float interpolate_curve(float coef,LINK_PARAMETERS *link)
{
	float result;

	switch(interpolation_type)
	{
		case _LINEAR_:
			
			result = (float)linear_interpolate(coef,link->points);		
			return result;
			break;
			
		default:
		
			break;
	
	}

}

/*******************************************************************************
	Function for linear interpolating probability
*******************************************************************************/
float linear_interpolate(float coef,PROBA_POINT *points)
{
	int nb_points = _NB_INTERPOLATING_POINTS_;
	int born_sup;
	int born_inf;
	float prob1,prob2;
	float prob;
	float pos,pos1,pos2;
	int i;

	born_inf = born_sup = 0;
	
	if(coef < points[0].length)
		return (float)points[0].probability;
	else if (coef > points[9].length)
		return (float)points[9].probability;
	
	i = 0;
	while((coef>points[i].length) && (i<nb_points))
		born_inf = i++;
	i = nb_points - 1;			
	while((coef<points[i].length) && (i>0))
		born_sup = i--;
		
	prob1 = points[born_inf].probability;
	prob2 = points[born_sup].probability;
	prob = (prob1+prob2)/2;
	pos1 = points[born_inf].length;
	pos2 = points[born_sup].length;	
	pos = (pos1+pos2)/2;
	
	while((pos-coef)*(pos-coef)>0.000001)
	{
		if(coef>pos)
		{
			pos1 = pos;
			prob1 = prob;	
		}
		else if(coef<pos)
		{
			pos2 = pos;
			prob2 = prob;
		}
		else
			break;
		prob = (prob1+prob2)/2;
		pos = (pos1+pos2)/2;
	}
	
	return (float)prob;	
}
/*******************************************************************************
	Function for notifying a specified neuron deletion in order to erase 
	connections to that particular neuron
*******************************************************************************/
void notify_neuron_deletion(NEURON *neuron)
{
	Nucleus_widget *nucleus_src;
	Nucleus_widget *nucleus_tgt;
	CLUSTER_WIDGET * cluster_src;
	CLUSTER_WIDGET *cluster_tgt;
	Connection *connection;
	NEURON *neuron_src;
	int i,j,k,m;
	
	for(i=0 ; i<current_network->nb_nucleus ; i++)
	{
		nucleus_src = current_network->nucleus_array[i];
		for(j=0 ; j<nucleus_src->nb_clusters ; j++)
		{
			cluster_src = nucleus_src->cluster_array[j];
			for(k=0 ; k<cluster_src->nb_neurons ; k++)
			{
				neuron_src = cluster_src->neuron_array[k];
				if(!neuron_src)
					continue;
				for(m=0 ; m<neuron_src->connected_neurons ; m++)
				{
					connection = &neuron_src->connections[m];
					if(connection->target_id == neuron->neuron_id)
					{
						connection->target_id = 0;
						neuron_src->connections = 
							refresh_connections(neuron_src);
					}
				}
				
				
			}
		}
	
	}

}
/*******************************************************************************
	Function for notifying a specified cluster deletion in order to erase
	links and connections to that cluster
*******************************************************************************/
void notify_cluster_deletion(CLUSTER_WIDGET *cluster)
{

	Nucleus_widget *nucleus_src;
	Nucleus_widget *nucleus_tgt;
	CLUSTER_WIDGET * cluster_src;
	CLUSTER_WIDGET *cluster_tgt;
	Connection *connection;
	NEURON *neuron_src;
	NEURON *neuron_tgt;
	LINK_PARAMETERS *link_tmp;
	int i,j,k,m,p;
	
	for(i=0 ; i<current_network->nb_nucleus ; i++)
	{
		nucleus_src = current_network->nucleus_array[i];
		for(j=0 ; j<nucleus_src->nb_clusters ; j++)
		{
			cluster_src = nucleus_src->cluster_array[j];
			link_tmp = search_link(cluster_src->cluster_id,
				cluster->cluster_id,_NUCLEUS_);
			if(!link_tmp)
				continue;
			for(k=0 ; k<cluster->nb_neurons ; k++)
			{
				neuron_tgt = cluster->neuron_array[k];
				for(m=0 ; m<cluster_src->nb_neurons ; m++)
				{
					neuron_src = cluster_src->neuron_array[m];
					for(p=0 ; p<neuron_src->connected_neurons ; p++)
					{
						connection = &neuron_src->connections[p];
						if(connection->target_id == neuron_tgt->neuron_id)
						{
							connection->target_id = 0;
							neuron_src->connections = 
								refresh_connections(neuron_src);
						}
					}
				}
			} 
			delete_link(&link_tmp);
			refresh_links();
		}
	
	}
	for(i=0 ; i<current_network->nb_nucleus ; i++)
	{
		nucleus_tgt = current_network->nucleus_array[i];
		for(j=0 ; j<nucleus_tgt->nb_clusters ; j++)
		{
			cluster_tgt = nucleus_tgt->cluster_array[j];
			link_tmp = search_link(cluster->cluster_id,
				cluster_tgt->cluster_id,_NUCLEUS_);
			if(!link_tmp)
				continue;
			for(k=0 ; k<cluster_tgt->nb_neurons ; k++)
			{
				neuron_tgt = cluster_tgt->neuron_array[k];
				for(m=0 ; m<cluster->nb_neurons ; m++)
				{
					neuron_src = cluster->neuron_array[m];
					for(p=0 ; p<neuron_src->connected_neurons ; p++)
					{
						connection = &neuron_src->connections[p];
						if(connection->target_id == neuron_tgt->neuron_id)
						{
							connection->target_id = 0;
							neuron_src->connections = 
								refresh_connections(neuron_src);
						}
					}
				}
			} 
			delete_link(&link_tmp);
			refresh_links();
		}
	
	}
}
/*******************************************************************************
	Function for notifying a specified nucleus deletion in order to erase
	links and connections to that nucleus
*******************************************************************************/
void notify_nucleus_deletion(Nucleus_widget *nucleus)
{

	Nucleus_widget *nucleus_src;
	Nucleus_widget *nucleus_tgt;
	CLUSTER_WIDGET * cluster_src;
	CLUSTER_WIDGET *cluster_tgt;
	Connection *connection;
	NEURON *neuron_src;
	NEURON *neuron_tgt;
	LINK_PARAMETERS *link1_tmp;
	LINK_PARAMETERS *link2_tmp;
	int i,j,k,m,p,t;

	for(i=0 ; i< current_network->nb_nucleus ; i++)
	{
		nucleus_src = current_network->nucleus_array[i];
		link1_tmp = search_link(nucleus_src->nucleus_id,
			nucleus->nucleus_id,_NETWORK_);
		if(!link1_tmp)
			continue;
		
		for(j=0 ; j<nucleus->nb_clusters ; j++)
		{
			cluster_tgt = nucleus->cluster_array[j];
			for(t=0 ; t<nucleus_src->nb_clusters ; t++)
			{
				cluster_src = nucleus_src->cluster_array[t];
				link2_tmp = search_link(cluster_src->cluster_id,
					cluster_tgt->cluster_id,_NUCLEUS_);
				if(!link2_tmp)
					continue;
				for(k=0 ; k<cluster_tgt->nb_neurons ; k++)
				{
					neuron_tgt = cluster_tgt->neuron_array[k];
					for(m=0 ; m<cluster_src->nb_neurons ; m++)
					{
						neuron_src = cluster_src->neuron_array[m];
						for(p=0 ; p<neuron_src->connected_neurons ; p++)
						{
							connection = &neuron_src->connections[p];
							if(connection->target_id == neuron_tgt->neuron_id)
							{
								connection->target_id = 0;
								neuron_src->connections = 
									refresh_connections(neuron_src);
							}
						}
					}
				} 
				delete_link(&link2_tmp);
				refresh_links();
			}
			
		}
		delete_link(&link1_tmp);
	}
	for(i=0 ; i< current_network->nb_nucleus ; i++)
	{
		nucleus_tgt = current_network->nucleus_array[i];
		link1_tmp = search_link(nucleus->nucleus_id,
			nucleus_tgt->nucleus_id,_NETWORK_);
		if(!link1_tmp)
			continue;
		
		for(j=0 ; j<nucleus_tgt->nb_clusters ; j++)
		{
			cluster_tgt = nucleus_tgt->cluster_array[j];
			for(t=0 ; t<nucleus->nb_clusters ; t++)
			{
				cluster_src = nucleus->cluster_array[t];
				link2_tmp = search_link(cluster_src->cluster_id,
					cluster_tgt->cluster_id,_NUCLEUS_);
				if(!link2_tmp)
					continue;
				for(k=0 ; k<cluster_tgt->nb_neurons ; k++)
				{
					neuron_tgt = cluster_tgt->neuron_array[k];
					for(m=0 ; m<cluster_src->nb_neurons ; m++)
					{
						neuron_src = cluster_src->neuron_array[m];
						for(p=0 ; p<neuron_src->connected_neurons ; p++)
						{
							connection = &neuron_src->connections[p];
							if(connection->target_id == neuron_tgt->neuron_id)
							{
								connection->target_id = 0;
								neuron_src->connections = 
									refresh_connections(neuron_src);
							}
						}
					}
				} 
				delete_link(&link2_tmp);
				refresh_links();
			}
			
		}
		delete_link(&link1_tmp);
	}

}

/*******************************************************************************
	Function for verifying all links and update extremities
*******************************************************************************/
void verify_extremities(NEURON *neuron)
{

	int i;
	LINK_PARAMETERS *link_tmp;
	Nucleus_widget *nucleus_tmp;
	
	for(i=0 ; i<current_network->nb_links ; i++)
	{
		link_tmp = current_network->link_array[i];
		while(link_tmp->source == neuron)
		{
			link_tmp->source = NULL;
		}
		while(link_tmp->target == neuron)
		{
			link_tmp->target = NULL;
		}
	}
	neuron->extremity = False;

}

/*******************************************************************************
	Function for updating link type
*******************************************************************************/
void update_link_type(LINK_PARAMETERS **link_tmp)
{
	Nucleus_widget *nucleus_src;
	Nucleus_widget *nucleus_tgt;
	CLUSTER_WIDGET *cluster_src;
	CLUSTER_WIDGET *cluster_tgt;
	NEURON *neuron_src;
	NEURON *neuron_tgt;
	int no_connection;
	int i,j,k;
	
	no_connection = True;
	nucleus_src = search_nucleus((*link_tmp)->source_id);
	for(i=0 ; i<nucleus_src->nb_clusters ; i++)
	{
		cluster_src = nucleus_src->cluster_array[i];
		for(j=0 ; j<cluster_src->nb_neurons ; j++)
		{
			neuron_src = cluster_src->neuron_array[j];
			if(neuron_src->connected_neurons)
			{
				
				for(k=0 ; k<neuron_src->connected_neurons ; k++)
				{
					neuron_tgt = search_neuron(neuron_src->connections[k].target_id,
						&cluster_tgt);
					search_cluster(cluster_tgt->cluster_id,&nucleus_tgt);
					if(nucleus_tgt->nucleus_id == (*link_tmp)->target_id)
					{
						no_connection = False;
						break;
					}
				}
			}
			if(!no_connection)
				break;			
		}
		if(!no_connection)
				break;
	}		
	if(no_connection)
		delete_link(link_tmp);

}

/*******************************************************************************
	Function for drawing the curve axes
*******************************************************************************/
void draw_curve_axes(void)
{
	int i,j;
	int Xmin;
	int Ymin;
	int Xmax;
	int Ymax;
	int Ystep;
	int Xstep;
	short width,height;
	char buffer[255];
	
	XtVaGetValues(widget_array[ID_DrawingA_connection_curve],
		XmNwidth, &width, XmNheight ,&height,NULL);
	
	while((width-100)%100)
		width--;
	while((height-100)%100)
		height--;
		
	Xmin = 50;
	Xmax = width - 50;
	Ymin = height - 50;
	Ymax = 50;

	
	XSetForeground(the_display,the_GC,black);
	XSetBackground(the_display,the_GC,white);
	XSetFunction(the_display,the_GC,GXcopy);
	XSetLineAttributes(the_display,the_GC,1,LineSolid,CapRound,JoinRound);
	
	XDrawLine(the_display,curve_pixmap,the_GC,
		Xmin,Ymin,
		Xmax,Ymin);
	XDrawLine(the_display,curve_pixmap,the_GC,
		Xmin,Ymax,
		Xmin,Ymin);
	
	XSetLineAttributes(the_display,the_GC,1,LineOnOffDash,CapRound,JoinRound);
	XDrawLine(the_display,curve_pixmap,the_GC,
		Xmin,Ymax,
		Xmax,Ymax);
	XDrawLine(the_display,curve_pixmap,the_GC,
		Xmax,Ymax,
		Xmax,Ymin);
	XSetLineAttributes(the_display,the_GC,1,LineSolid,CapRound,JoinRound);
			
	for(i=0 ; i<=width-100 ; i++)
	{
		if(!((i*10)%(width-100)))
		{
			sprintf(buffer,"%d%%",(100*i)/(width-100));
			XDrawString(the_display,curve_pixmap,the_GC,
				Xmin+i-10,Ymin+20,buffer,strlen(buffer));
			
			XDrawLine(the_display,curve_pixmap,the_GC,
				Xmin+i,Ymin-5,
				Xmin+i,
				Ymin);
				
			
		}
	}
	for(i=0 ; i<=height-100 ; i++)
	{
		if(!((i*10)%(height-100)))
		{
			sprintf(buffer,"%d%%",(100*i)/(height-100));
			XDrawString(the_display,curve_pixmap,the_GC,
				Xmin-20,Ymin-i,buffer,strlen(buffer));
			
			XDrawLine(the_display,curve_pixmap,the_GC,
				Xmin,Ymin-i,
				Xmin+5,
				Ymin-i);
				
		}
	}
}

/*******************************************************************************
	Function for drawing the curve points widgets
*******************************************************************************/
void draw_points(void)
{
	int i;
	int X,Y;
	int Xmin;
	int Ymin;
	int Xmax;
	int Ymax;
	int Ystep;
	int Xstep;
	short width,height;
	char buffer[255];
	PROBA_POINT point;
	GRAPHIC_ITEM *g_item;
	
	XtVaGetValues(widget_array[ID_DrawingA_connection_curve],
		XmNwidth, &width, XmNheight ,&height,NULL);
	
	while((width-100)%100)
		width--;
	while((height-100)%100)
		height--;
			
	Xmin = 50;
	Xmax = width - 50;
	Ymin = height - 50;
	Ymax = 50;
	
	
	if(!current_link)
	{
		fprintf(stderr,"current_link is NULL in draw_points !!!\n");
		exit(-1);
	}
	for(i=0 ; i<_NB_INTERPOLATING_POINTS_ ; i++)
	{
		point = current_link->points[i];
		X = (int)(Xmin + point.length*((width-100)/100)) - POINT_SIZE/2;
		Y = (int)(Ymin - point.probability*((height-100)/100)) - POINT_SIZE/2;
		g_item = &curve_points[i];
		
		if(!g_item->form)	
		{
			g_item->color = blue[4];

			sprintf(buffer,"curve_point_%d",i);
			g_item->form = XtVaCreateWidget (buffer,
				xmDrawingAreaWidgetClass, widget_array[ID_DrawingA_connection_curve],
				XmNwidth, POINT_SIZE,
				XmNheight,POINT_SIZE,
				XmNborderWidth, 0,
				XmNrecomputeSize, False,
				NULL);

			
			XtAddEventHandler(g_item->form,ButtonReleaseMask,False,
				(XtEventHandler)release_point,i);
			XtAddEventHandler(g_item->form,ButtonMotionMask,False,
				(XtEventHandler)move_point,i);
			XtAddEventHandler(g_item->form,ButtonMotionMask,False,
				(XtEventHandler)info_point,i);	
			XtAddEventHandler(g_item->form,EnterWindowMask,False,
				(XtEventHandler)info_point,i);
			XtAddEventHandler(g_item->form,LeaveWindowMask,False,
				(XtEventHandler)clear_info_curve,i);

		}
		else if(XtIsManaged(g_item->form))
		{
			XtUnmanageChild(g_item->form);
		}
		if(!g_item->bgnd)
		{
			g_item->bgnd = XCreatePixmap(the_display,
		XtWindow(widget_array[ID_DrawingA_connection_curve]),POINT_SIZE,POINT_SIZE,
		DefaultDepthOfScreen(XtScreen(widget_array[ID_DrawingA_connection_curve])));
			XSetForeground(the_display,the_GC,g_item->color);
			XSetBackground(the_display,the_GC,black);
			XFillRectangle(the_display, g_item->bgnd,the_GC,
			0,0,POINT_SIZE-1,POINT_SIZE-1);
			XtVaSetValues(g_item->form,XmNbackgroundPixmap,
			g_item->bgnd,NULL);
			XSetForeground(the_display,the_GC,black);
			XSetBackground(the_display,the_GC,white);
			XSetFunction(the_display,the_GC,GXcopy);
			XSetLineAttributes(the_display,the_GC,1,LineSolid,CapRound,JoinRound);
			XDrawRectangle(the_display, g_item->bgnd,the_GC,
				0,0,POINT_SIZE-1,POINT_SIZE-1);
		}
		XtVaSetValues(g_item->form,XmNx,X,XmNy,Y,NULL);
		XtManageChild(g_item->form);		
	}	

}

/*******************************************************************************
	Function for drawing the connection curve
*******************************************************************************/
void draw_curve(void)
{
	int i;
	int X1,Y1,X2,Y2;
	int Xmin;
	int Ymin;
	int Xmax;
	int Ymax;
	int Ystep;
	int Xstep;
	short width,height;
	char buffer[255];
	PROBA_POINT point1,point2;
	
	
	
	XtVaGetValues(widget_array[ID_DrawingA_connection_curve],
		XmNwidth, &width, XmNheight ,&height,NULL);
	
	while((width-100)%100)
		width--;
	while((height-100)%100)
		height--;
			
	Xmin = 50;
	Xmax = width - 50;
	Ymin = height - 50;
	Ymax = 50;
	
	/* probability curve */
	
	point1 = current_link->points[0];
	X1 = (int)(Xmin + point1.length*((width-100)/100));
	Y1 = (int)(Ymin - point1.probability*((height-100)/100));
	
	XSetForeground(the_display,the_GC,red[4]);
	XSetBackground(the_display,the_GC,white);
	XSetFunction(the_display,the_GC,GXcopy);
	XSetLineAttributes(the_display,the_GC,1,LineSolid,CapRound,JoinRound);
	
	XDrawLine(the_display,curve_pixmap,the_GC,
		Xmin,Y1,
		X1,Y1);
	
	for(i=0 ; i<_NB_INTERPOLATING_POINTS_-1 ; i++)
	{
		point1 = current_link->points[i];
		X1 = (int)(Xmin + point1.length*((width-100)/100));
		Y1 = (int)(Ymin - point1.probability*((height-100)/100)) ;
		point2 = current_link->points[i+1];
		X2 = (int)(Xmin + point2.length*((width-100)/100));
		Y2 = (int)(Ymin - point2.probability*((height-100)/100));
		XDrawLine(the_display,curve_pixmap,the_GC,
			X1,Y1,
			X2,Y2);
	}
	XDrawLine(the_display,curve_pixmap,the_GC,
			X2,Y2,
			Xmax,Y2);
	sprintf(buffer,"Connecting Rate = f(Length Rate)");
	XDrawString(the_display,curve_pixmap,the_GC,
				Xmin + (Xmax-Xmin)/8,Ymax+(Ymin-Ymax)/2-20,buffer,strlen(buffer));
	
	
	/* connected neuron density */
	
	point1 = current_link->points[0];
	X1 = (int)(Xmin + point1.length*((width-100)/100));
	Y1 = (int)(Ymin - point1.density*((height-100)/100));
	
	XSetForeground(the_display,the_GC,green);
	XSetBackground(the_display,the_GC,white);
	XSetFunction(the_display,the_GC,GXcopy);
	XSetLineAttributes(the_display,the_GC,1,LineOnOffDash,CapRound,JoinRound);
	
	XDrawLine(the_display,curve_pixmap,the_GC,
		Xmin,Y1,
		X1,Y1);
	
	for(i=0 ; i<_NB_INTERPOLATING_POINTS_-1 ; i++)
	{
		point1 = current_link->points[i];
		X1 = (int)(Xmin + point1.length*((width-100)/100));
		Y1 = (int)(Ymin - point1.density*((height-100)/100)) ;
		point2 = current_link->points[i+1];
		X2 = (int)(Xmin + point2.length*((width-100)/100));
		Y2 = (int)(Ymin - point2.density*((height-100)/100));
		XDrawLine(the_display,curve_pixmap,the_GC,
			X1,Y1,
			X2,Y2);
	}
	XDrawLine(the_display,curve_pixmap,the_GC,
			X2,Y2,
			Xmax,Y2);
	sprintf(buffer,"Statistical Connected Neurons Density = f(Length Rate)");
	XDrawString(the_display,curve_pixmap,the_GC,
				Xmin + (Xmax-Xmin)/8,Ymax+(Ymin-Ymax)/2,buffer,strlen(buffer));
	
	
	/* neuron density */
	
	point1 = current_link->points[0];
	X1 = (int)(Xmin + point1.length*((width-100)/100));
	Y1 = (int)(Ymin - point1.real_density*((height-100)/100));
	
	XSetForeground(the_display,the_GC,blue[4]);
	XSetBackground(the_display,the_GC,white);
	XSetFunction(the_display,the_GC,GXcopy);
	XSetLineAttributes(the_display,the_GC,1,LineOnOffDash,CapRound,JoinRound);
	
	XDrawLine(the_display,curve_pixmap,the_GC,
		Xmin,Y1,
		X1,Y1);
	
	for(i=0 ; i<_NB_INTERPOLATING_POINTS_-1 ; i++)
	{
		point1 = current_link->points[i];
		X1 = (int)(Xmin + point1.length*((width-100)/100));
		Y1 = (int)(Ymin - point1.real_density*((height-100)/100)) ;
		point2 = current_link->points[i+1];
		X2 = (int)(Xmin + point2.length*((width-100)/100));
		Y2 = (int)(Ymin - point2.real_density*((height-100)/100));
		XDrawLine(the_display,curve_pixmap,the_GC,
			X1,Y1,
			X2,Y2);
	}
	XDrawLine(the_display,curve_pixmap,the_GC,
			X2,Y2,
			Xmax,Y2);
	sprintf(buffer,"Statistical Neighbours Neurons Density = f(Length Rate)");
	XDrawString(the_display,curve_pixmap,the_GC,
				Xmin + (Xmax-Xmin)/8,Ymax+(Ymin-Ymax)/2+20,buffer,strlen(buffer));
						
}

/*******************************************************************************
	Function for redrawing the connection curve
*******************************************************************************/
void redraw_connection_curve(void)
{
	short width,height;

	if(curve_pixmap)
		XFreePixmap(the_display,curve_pixmap);
	
			
	XtVaGetValues(widget_array[ID_DrawingA_connection_curve],
		XmNwidth, &width, XmNheight ,&height,NULL);
						
	curve_pixmap = XCreatePixmap(the_display,
		XtWindow(widget_array[ID_DrawingA_connection_curve]),
		width,height,
		DefaultDepth(the_display,DefaultScreen (the_display)));

	XSetForeground(the_display,the_GC,white);
	XSetBackground(the_display,the_GC,black);
	XFillRectangle(the_display,curve_pixmap,the_GC,0,0,width,height);

	XtVaSetValues(widget_array[ID_DrawingA_connection_curve],
		XmNbackgroundPixmap,curve_pixmap,NULL);
		
		
	draw_curve_axes();
	
	draw_points();
	
	update_density();
	
	draw_curve();
	
	XClearWindow(the_display,XtWindow(widget_array[ID_DrawingA_connection_curve]));	
}

/*******************************************************************************
	Function for updating a nucleus level  generic link
*******************************************************************************/
void update_density(void)
{
	
	NEURON *neuron_src;
	NEURON *neuron_tgt;
	NEURON *cnx_center;
	CLUSTER_WIDGET *cluster_src;
	CLUSTER_WIDGET *cluster_tgt;
	Nucleus_widget *nucleus_src;
	Nucleus_widget *nucleus_tgt;
	int	set;
	float axone_length;
	int synaptic_knobs;
	int i,j,k,p,q;
	int pos;
	float d_src,d_tgt;
	float d_src_max,d_tgt_max;
	float proba;
	float coef,res;
	float real_proba;
	int intra;
	
	
	
	
	if(!current_link)
	{
		fprintf(stderr,"current link is NULL in update_density  !!!!\n");
		exit(-1);
	}
	set = (int) time (0) & 077;			
	
	XDefineCursor(the_display,XtWindow(widget_array[ID_FormD_adjustement_curve]),
			working_cursor);
	
	nucleus_src = search_nucleus(current_link->source_id);
	nucleus_tgt = search_nucleus(current_link->target_id);
	d_tgt_max = absolute_length(0,
									0,
									0,
									nucleus_tgt->delta_L,
									nucleus_tgt->delta_H,
									nucleus_tgt->delta_AP);
	if(d_tgt_max<1.0)
				d_tgt_max = 1.0;
	
	if(nucleus_tgt == nucleus_src)
				intra = 1;
			else 
				intra = 0;
	
	for(i=0 ; i<10 ; i++)
	{
		current_link->points[i].density = 0;
		current_link->points[i].real_density = 0;
	}
															
	for(k=0 ; k<nucleus_src->nb_clusters ; k++)
	{
		
		cluster_src = nucleus_src->cluster_array[k];
		for(p=0 ; p<nucleus_tgt->nb_clusters ; p++)
		{
			
			cluster_tgt = nucleus_tgt->cluster_array[p];
				
									
			for(i=0 ; i<cluster_src->nb_neurons ; i++)
			{
				neuron_src = cluster_src->neuron_array[i];
				
				cnx_center = place_link_extremity(nucleus_tgt->nucleus_id);
				cnx_center->extremity = False;
				
				for(j=0 ; j<cluster_tgt->nb_neurons ; j++)
				{
					neuron_tgt = cluster_tgt->neuron_array[j];
					
					d_tgt = absolute_length(neuron_tgt->anatomy.L,
									neuron_tgt->anatomy.H,
									neuron_tgt->anatomy.AP,
									cnx_center->anatomy.L,
									cnx_center->anatomy.H,
									cnx_center->anatomy.AP);
					if(!intra)
						coef = (d_tgt/(2*d_tgt_max))*100.0;
					else
						coef = (absolute_length(neuron_tgt->anatomy.L,
									neuron_tgt->anatomy.H,
									neuron_tgt->anatomy.AP,
									neuron_src->anatomy.L,
									neuron_src->anatomy.H,
									neuron_src->anatomy.AP)/(1.5*d_tgt_max))*100.0;
					
					
						
					proba = interpolate_curve(coef,current_link)/100.0;
					proba *= (float)(ran1(&set)*
						(float)(current_link->max_knobs - current_link->min_knobs)
						+ (float)current_link->min_knobs);
					
					if(coef < (current_link->points[0].length
						+ current_link->points[1].length)/2)
					{
						if(proba >= 1.0)
							current_link->points[0].density++;
						current_link->points[0].real_density++;
					}
					else if(coef < (current_link->points[1].length
						+ current_link->points[2].length)/2)
					{
						if(proba >= 1.0)
							current_link->points[1].density++;
						current_link->points[1].real_density++;	
					}
					
					else if(coef < (current_link->points[2].length 
						+ current_link->points[3].length)/2)
					{
						if(proba >= 1.0)
							current_link->points[2].density++;
						current_link->points[2].real_density++;		
					}
					else if(coef < (current_link->points[3].length 
						+ current_link->points[4].length)/2)
					{
						if(proba >= 1.0)
							current_link->points[3].density++;
						current_link->points[3].real_density++;		
					}	
					else if(coef < (current_link->points[4].length 
						+ current_link->points[5].length)/2)
					{
						if(proba >= 1.0)
							current_link->points[4].density++;
						current_link->points[4].real_density++;		
					}
					else if(coef < (current_link->points[5].length 
						+ current_link->points[6].length)/2)
					{
						if(proba >= 1.0)
							current_link->points[5].density++;
						current_link->points[5].real_density++;		
					}
					else if(coef < (current_link->points[6].length 
						+ current_link->points[7].length)/2)
					{
						if(proba >= 1.0)
							current_link->points[6].density++;
						current_link->points[6].real_density++;		
					}
					else if(coef < (current_link->points[7].length 
						+ current_link->points[8].length)/2)
					{
						if(proba >= 1.0)
							current_link->points[7].density++;
						current_link->points[7].real_density++;		
					}	
					else if(coef < (current_link->points[8].length 
						+ current_link->points[9].length)/2)
					{
						if(proba >= 1.0)
							current_link->points[8].density++;
						current_link->points[8].real_density++;		
					}
					else if(coef >= (current_link->points[8].length 
						+ current_link->points[9].length)/2)
					{
						if(proba >= 1.0)
							current_link->points[9].density++;
						current_link->points[9].real_density++;		
					}
				}
			}
		}
	}
	for(i=1 ; i<9 ; i++)
	{
		current_link->points[i].density /= (float)nucleus_src->nb_neurons*(float)
			nucleus_tgt->nb_neurons;
		current_link->points[i].density *= 1000.0;
		current_link->points[i].density /= ((current_link->points[i+1].length + 
			current_link->points[i].length)/2 - (current_link->points[i].length +
			+ current_link->points[i-1].length)/2);	
		current_link->points[i].real_density /= (float)nucleus_src->nb_neurons*(float)
			nucleus_tgt->nb_neurons;	
		current_link->points[i].real_density *= 1000.0;
		current_link->points[i].real_density /= ((current_link->points[i+1].length + 
			current_link->points[i].length)/2 - (current_link->points[i].length +
			+ current_link->points[i-1].length)/2);
			
		if(current_link->points[i].real_density > 100.0)
			current_link->points[i].real_density = 100.0;
		if(current_link->points[i].real_density < 0.0)
			current_link->points[i].real_density = 0.0;
		if(current_link->points[i].density > 100.0)
			current_link->points[i].density = 100.0;
		if(current_link->points[i].density < 0.0)
			current_link->points[i].density = 0.0;
	}
	current_link->points[0].density /= (float)nucleus_src->nb_neurons*(float)
			nucleus_tgt->nb_neurons;
	current_link->points[0].density *= 1000.0;
	current_link->points[0].density /= ((current_link->points[1].length + 
		current_link->points[0].length)/2);
	current_link->points[0].real_density /= (float)nucleus_src->nb_neurons*(float)
			nucleus_tgt->nb_neurons;
	current_link->points[0].real_density *= 1000.0;
	current_link->points[0].real_density /= ((current_link->points[1].length + 
		current_link->points[0].length)/2);
	if(current_link->points[0].real_density > 100.0)
		current_link->points[0].real_density = 100.0;
	if(current_link->points[0].real_density < 0.0)
		current_link->points[0].real_density = 0.0;
	if(current_link->points[0].density > 100.0)
		current_link->points[0].density = 100.0;
	if(current_link->points[0].density < 0.0)
		current_link->points[0].density = 0.0;

				
	current_link->points[9].density /= (float)nucleus_src->nb_neurons*(float)
			nucleus_tgt->nb_neurons;
	current_link->points[9].real_density /= (float)nucleus_src->nb_neurons*(float)
			nucleus_tgt->nb_neurons;
	current_link->points[9].density *= 1000.0;
	current_link->points[9].density /= (100.0 - (current_link->points[9].length + 
		current_link->points[8].length)/2); 
	current_link->points[i].real_density *= 1000.0;
	current_link->points[i].real_density /= (100.0 - (current_link->points[9].length + 
		current_link->points[8].length)/2);
	if(current_link->points[9].real_density > 100.0)
		current_link->points[9].real_density = 100.0;
	if(current_link->points[9].real_density < 0.0)
		current_link->points[9].real_density = 0.0;
	if(current_link->points[9].density > 100.0)
		current_link->points[9].density = 100.0;
	if(current_link->points[9].density < 0.0)
		current_link->points[9].density = 0.0;
				
	XDefineCursor(the_display,XtWindow(widget_array[ID_FormD_adjustement_curve]),
			NULL);

}
	

/*******************************************************************************
	Function for redrawing the connection matrix
*******************************************************************************/
void redraw_connection_matrix(void)
{

	Nucleus_widget *nucleus_src;
	Nucleus_widget *nucleus_tgt;
	CLUSTER_WIDGET *cluster_tmp;
	NEURON *neuron_tmp;
	NEURON **neuron_array_src;
	NEURON **neuron_array_tgt;
	int n_src,n_tgt,width,height;
	int matrix;
	int i,j;
	int  k;
	char buffer[255];

	if(!current_link)
	{
		fprintf(stderr,"current_link is NULL in redraw_connection_matrix!!!\n");
		exit(-1);
	}

	XDefineCursor(the_display,XtWindow(widget_array[ID_FormD_generic_link_matrix]),
			working_cursor);

	if(matrix_pixmap)
	{
		XFreePixmap(the_display,matrix_pixmap);
		matrix_pixmap = 0;
	}

	nucleus_src = search_nucleus(current_link->source_id);
	nucleus_tgt = search_nucleus(current_link->target_id);
	
	n_src = nucleus_src->nb_neurons;
	n_tgt = nucleus_tgt->nb_neurons;
	
	foot = MAX_FOOT;
	
		
	test_alloc = 0;
	matrix = 1;
	foot++;
	
	XSetErrorHandler((XtPointer)(&err_alloc_pixmap));
	XSynchronize(the_display,1);
	
	while(!test_alloc)
	{
		foot--;
		if((foot) && (MAX(n_src,n_tgt) < AREA_SIZE))
		{
			test_alloc = 1;
		
			width = n_tgt * foot;
			height = n_src * foot;
			
      		matrix_pixmap = XCreatePixmap(the_display,
				XtWindow(widget_array[ID_DrawingA_matrix]),
				width,height,
				DefaultDepth(the_display,DefaultScreen (the_display)));
		
			XSetForeground(the_display,the_GC,white);
			XSetBackground(the_display,the_GC,black);
			XFillRectangle(the_display,matrix_pixmap,the_GC,0,0,width,height);
		
			XtVaSetValues(widget_array[ID_DrawingA_matrix],
				XmNbackgroundPixmap,matrix_pixmap,NULL);
			
			XtVaSetValues(widget_array[ID_DrawingA_matrix],
				XmNwidth,width,XmNheight,height,NULL);
			
      	}
      	else 
      	{
      		test_alloc = 1;
      		matrix = 0;	
      	}
	}

	
	XSetErrorHandler(NULL);
	XSynchronize(the_display,0);
	
	
	if(!matrix)
	{
		display_message(widget_array[ID_FormD_generic_link_edit],
		"Sorry, the connection matrix can't be displayed\nbut it's been calculed and updated into the storage areas...",
	"Not Enough Memory !");
		return;
	}
	
	/* fill the neuron arrays in order to manipulate neurons easily */
	
	neuron_array_src = (NEURON **)calloc(1,n_src*sizeof(NEURON *));
	
	k = 0;
	for(i=0 ; i<nucleus_src->nb_clusters ; i++)
	{
		cluster_tmp = nucleus_src->cluster_array[i];
		for(j=0 ; j<cluster_tmp->nb_neurons ; j++)
		{
			neuron_array_src[k++] = cluster_tmp->neuron_array[j];	
		}
	}
	
	neuron_array_tgt = (NEURON **)calloc(1,n_tgt*sizeof(NEURON *));
	
	k = 0;
	for(i=0 ; i<nucleus_tgt->nb_clusters ; i++)
	{
		cluster_tmp = nucleus_tgt->cluster_array[i];
		for(j=0 ; j<cluster_tmp->nb_neurons ; j++)
		{
			neuron_array_tgt[k++] = cluster_tmp->neuron_array[j];	
		}
	}	

	draw_matrix(neuron_array_src,neuron_array_tgt,n_src,n_tgt,foot);

	free(neuron_array_src);
	free(neuron_array_tgt);
	
	XtVaSetValues(widget_array[ID_Scale_knobs],XmNmaximum,current_link->max_knobs,
		XmNminimum,current_link->min_knobs,NULL);
		
	sprintf(buffer,"Connection Matrix from %s to %s",nucleus_src->nucleus_name,
		nucleus_tgt->nucleus_name);
	
	XtVaSetValues(XtParent(widget_array[ID_FormD_generic_link_matrix]),XmNtitle,buffer,NULL);
	
	XClearWindow(the_display,XtWindow(widget_array[ID_DrawingA_matrix]));
	
	XDefineCursor(the_display,XtWindow(widget_array[ID_FormD_generic_link_matrix]),
			NULL);

}

/*******************************************************************************
	Function for modifying the connection matrix
*******************************************************************************/
void modify_matrix(int X1,int Y1,int X2, int Y2)
{
	static struct{
		int i;
		int j;
		int type;
		int nmda;
		int knobs;
		} mem[200];
	short nmda;
	short type;
	int knobs;
	int i,j,k;
	static int Xref;
	static int Yref;
	Nucleus_widget *nucleus_src = search_nucleus(current_link->source_id);
	Nucleus_widget *nucleus_tgt = search_nucleus(current_link->target_id);
	
	/* first analyse the editing status */
	
	XtVaGetValues(widget_array[ID_Scale_knobs],XmNvalue,&knobs,NULL);
	
	if(XmToggleButtonGetState(widget_array[ID_ToggleB_matrix_NMDA]))
		nmda = True;
	else
		nmda = False;
	
	if(XmToggleButtonGetState(widget_array[ID_ToggleB_contype_excit]))	
		type = _FULL_EXCITATION_;
	else if(XmToggleButtonGetState(widget_array[ID_ToggleB_contype_inhib]))
		type = _FULL_INHIBITION_;
	else if(XmToggleButtonGetState(widget_array[ID_ToggleB_contype_notcon]))
		type = _FULL_NOT_CONNECTED_;
		
	/* work according current mode */
	
	switch(edit_mode)
	{
		case _REGULAR_:
			
			if((X1<=X2)&&(Y1<=Y2))
			{
				for(i=X1 ; i<=X2 ; i++)
				{
					if(i >= nucleus_tgt->nb_neurons)
						continue;
					for(j=Y1 ; j<=Y2 ; j++)
					{
						if(j >= nucleus_src->nb_neurons)
							continue;
						redraw_connection(i,j,type,knobs,nmda);
						change_connection(i,j,type,knobs,nmda);
					}
				}
			}
			else if ((X2<=X1)&&(Y2<=Y1))
			{
				for(i=X2 ; i<=X1 ; i++)
				{
					if(i >= nucleus_tgt->nb_neurons)
						continue;
					for(j=Y2 ; j<=Y1 ; j++)
					{
						if(j >= nucleus_src->nb_neurons)
							continue;
						redraw_connection(i,j,type,knobs,nmda);
						change_connection(i,j,type,knobs,nmda);
					}
				}
			}
			else if((X1<=X2)&&(Y2<=Y1))
			{
				for(i=X1 ; i<=X2 ; i++)
				{
					if(i >= nucleus_tgt->nb_neurons)
						continue;
					for(j=Y2 ; j<=Y1 ; j++)
					{
						if(j >= nucleus_src->nb_neurons)
							continue;
						redraw_connection(i,j,type,knobs,nmda);
						change_connection(i,j,type,knobs,nmda);
					}
				}
			}
			else if ((X2<=X1)&&(Y1<=Y2))
			{
				for(i=X2 ; i<=X1 ; i++)
				{
					if(i >= nucleus_tgt->nb_neurons)
						continue;
					for(j=Y1 ; j<=Y2 ; j++)
					{
						if(j >= nucleus_src->nb_neurons)
							continue;
						redraw_connection(i,j,type,knobs,nmda);
						change_connection(i,j,type,knobs,nmda);
					}
				}
			}
		
			break;
			
		case _RECORD_:
		
			if((X1<=X2)&&(Y1<=Y2))
			{
				for(i=X1 ; i<=X2 ; i++)
				{
					if(i >= nucleus_tgt->nb_neurons)
						continue;
					for(j=Y1 ; j<=Y2 ; j++)
					{
						if(j >= nucleus_src->nb_neurons)
							continue;
						if(nb_mem < 200)
						{
							if(!nb_mem)
							{
								Xref = X1;
								Yref = Y1;
							}
							k = nb_mem++;
							mem[k].i = i - Xref;
							mem[k].j = j - Yref;
							mem[k].type = type;
							mem[k].knobs = knobs;
							mem[k].nmda = nmda;
						}
						redraw_connection(i,j,type,knobs,nmda);
						change_connection(i,j,type,knobs,nmda);
					}
				}
			}
			else if ((X2<=X1)&&(Y2<=Y1))
			{
				for(i=X2 ; i<=X1 ; i++)
				{
					if(i >= nucleus_tgt->nb_neurons)
						continue;
					for(j=Y2 ; j<=Y1 ; j++)
					{
						if(j >= nucleus_src->nb_neurons)
							continue;
						if(nb_mem < 200)
						{
							if(!nb_mem)
							{
								Xref = X2;
								Yref = Y2;
							}
							k = nb_mem++;
							mem[k].i = i - Xref;
							mem[k].j = j - Yref;
							mem[k].type = type;
							mem[k].knobs = knobs;
							mem[k].nmda = nmda;
						}
						redraw_connection(i,j,type,knobs,nmda);
						change_connection(i,j,type,knobs,nmda);
					}
				}
			}
			else if((X1<=X2)&&(Y2<=Y1))
			{
				for(i=X1 ; i<=X2 ; i++)
				{
					if(i >= nucleus_tgt->nb_neurons)
						continue;
					for(j=Y2 ; j<=Y1 ; j++)
					{
						if(j >= nucleus_src->nb_neurons)
							continue;
						if(nb_mem < 200)
						{
							if(!nb_mem)
							{
								Xref = X1;
								Yref = Y2;
							}
							k = nb_mem++;
							mem[k].i = i - Xref;
							mem[k].j = j - Yref;
							mem[k].type = type;
							mem[k].knobs = knobs;
							mem[k].nmda = nmda;
						}
						redraw_connection(i,j,type,knobs,nmda);
						change_connection(i,j,type,knobs,nmda);
					}
				}
			}
			else if ((X2<=X1)&&(Y1<=Y2))
			{
				for(i=X2 ; i<=X1 ; i++)
				{
					if(i >= nucleus_tgt->nb_neurons)
						continue;
					for(j=Y1 ; j<=Y2 ; j++)
					{
						if(j >= nucleus_src->nb_neurons)
							continue;
						if(nb_mem < 200)
						{
							if(!nb_mem)
							{
								Xref = X2;
								Yref = Y1;
							}
							k = nb_mem++;
							mem[k].i = i - Xref;
							mem[k].j = j - Yref;
							mem[k].type = type;
							mem[k].knobs = knobs;
							mem[k].nmda = nmda;
						}
						redraw_connection(i,j,type,knobs,nmda);
						change_connection(i,j,type,knobs,nmda);
					}
				}
			}
			break;
			
		case _PASTE_:
			
			for(i=0 ; i<nb_mem ; i++)
			{
				if((mem[i].i + X2) >= nucleus_tgt->nb_neurons)
						continue;
				if((mem[i].j + Y2) >= nucleus_src->nb_neurons)
						continue;
				
				redraw_connection(mem[i].i+X2,mem[i].j+Y2,mem[i].type,mem[i].knobs,mem[i].nmda);
				change_connection(mem[i].i+X2,mem[i].j+Y2,mem[i].type,mem[i].knobs,mem[i].nmda);
			}
		
			break;
			
		default:
		
			break;
		
	
	}
	if(!XtIsManaged(widget_array[ID_FormD_Nucleus]))
	{
		deselect_all_nucleus();
		redisplay_network();
	}
	else
	{
		redisplay_nucleus();
	}
		
}
/*******************************************************************************
	Function for redrawing one connection
*******************************************************************************/
void redraw_connection(int X,int Y, int type, int knobs, int nmda)
{
	int pos_x,pos_y;
	int value;
	unsigned long current_color;
	
	pos_x = X*foot;
	pos_y = Y*foot;
	
	switch(type)
	{
		case _FULL_EXCITATION_:
			value = knobs;
			break;
		
		case _FULL_INHIBITION_:
			value = -knobs;
			break;
			
		case _FULL_NOT_CONNECTED_:
			value = 0;
			break;
		
		default:
			break;
	}
	
	if(!value)
		current_color = grey;
	else
		current_color = eval_color(value);
		
	XSetForeground(the_display,the_GC,current_color);
	XSetBackground(the_display,the_GC,white);
	XSetLineAttributes(the_display,the_GC,2,LineSolid,
		CapRound,JoinRound);
	XFillRectangle(the_display,matrix_pixmap,
		the_GC,pos_x,pos_y,foot,foot);

	if((nmda) && (type != _FULL_NOT_CONNECTED_))
	{
		XSetForeground(the_display,the_GC,black);	
		XFillArc(the_display,matrix_pixmap,
			the_GC,pos_x+foot/4,pos_y+foot/4,foot/2,foot/2,
			0*64,360*64);
	}

}

/*******************************************************************************
	Function for updating connection in the network structures
*******************************************************************************/
void change_connection(int X,int Y, int type, int knobs, int nmda)
{
	
	int i,j,k;
	NEURON **neuron_array_src;
	NEURON **neuron_array_tgt;
	CLUSTER_WIDGET *cluster_tmp;
	Nucleus_widget *nucleus_src;
	Nucleus_widget *nucleus_tgt;
	int pos;
	int value;
	int set = (int) time (0) & 077;	
	
	if(!current_link)
	{
		fprintf(stderr,"current_link is NULL in change_connection!!!\n");
		exit(-1);
	}

	nucleus_src = search_nucleus(current_link->source_id);
	nucleus_tgt = search_nucleus(current_link->target_id);
	
	switch(type)
	{
		case _FULL_EXCITATION_:
			value = knobs;
			break;
		
		case _FULL_INHIBITION_:
			value = -knobs;
			break;
			
		case _FULL_NOT_CONNECTED_:
			value = 0;
			break;
		
		default:
			break;
	}
	
	/* fill the neuron arrays in order to manipulate neurons easily */
	
	neuron_array_src = (NEURON **)calloc(1,nucleus_src->nb_neurons*sizeof(NEURON *));
	
	k = 0;
	for(i=0 ; i<nucleus_src->nb_clusters ; i++)
	{
		cluster_tmp = nucleus_src->cluster_array[i];
		for(j=0 ; j<cluster_tmp->nb_neurons ; j++)
		{
			neuron_array_src[k++] = cluster_tmp->neuron_array[j];	
		}
	}
	
	neuron_array_tgt = (NEURON **)calloc(1,nucleus_tgt->nb_neurons*sizeof(NEURON *));
	
	k = 0;
	for(i=0 ; i<nucleus_tgt->nb_clusters ; i++)
	{
		cluster_tmp = nucleus_tgt->cluster_array[i];
		for(j=0 ; j<cluster_tmp->nb_neurons ; j++)
		{
			neuron_array_tgt[k++] = cluster_tmp->neuron_array[j];	
		}
	}	

	if(search_connection(neuron_array_src[Y]->neuron_id,
				neuron_array_tgt[X]->neuron_id,&pos))
	{
		neuron_array_src[Y]->connections[pos].nmda = nmda;
		neuron_array_src[Y]->connections[pos].synaptic_knobs = value;
		
	}
	else
	{
		neuron_array_src[Y]->connections =
				 add_new_connection(neuron_array_src[Y]->neuron_id,
				 	neuron_array_tgt[X]->neuron_id,
					&pos);
		neuron_array_src[Y]->connections[pos].nmda = nmda;
		neuron_array_src[Y]->connections[pos].synaptic_knobs = value;
		neuron_array_src[Y]->connections[pos].axone_length = 
			 current_link->axone_mean
				+ (2. * ran1 (&set) - 1.)*current_link->axone_sd;
	
	}
	
	neuron_array_src[Y]->connections = refresh_connections(neuron_array_src[Y]);
	
	free(neuron_array_src);
	free(neuron_array_tgt);
}

/*******************************************************************************
	Function for drawing the connection matrix of between two arrays 
	of neurons with the specified resolution
*******************************************************************************/
void draw_matrix(NEURON **neuron_array_src,NEURON **neuron_array_tgt,
	int n_src, int n_tgt, int size)
{
	int i,j,k;
	NEURON *neuron_src;
	NEURON *neuron_tgt;
	Connection *connection;
	int X,Y,pos_x,pos_y;
	int pos;
	unsigned long current_color;
	
	X = Y = 1;
	pos_x = pos_y = 0;
	
	while(Y <= n_src)
	{
		neuron_src = neuron_array_src[Y - 1];
		if(!neuron_src->connected_neurons)
		{
			XSetForeground(the_display,the_GC,grey);
			XSetBackground(the_display,the_GC,white);
			XSetLineAttributes(the_display,the_GC,2,LineSolid,
				CapRound,JoinRound);
			XFillRectangle(the_display,matrix_pixmap,
				the_GC,pos_x,pos_y,n_tgt*size,size);
			Y++;
			pos_y += size;
			continue;
		}
		
		
		while(X <= n_tgt)
		{
			neuron_tgt = neuron_array_tgt[X - 1];
			if(search_connection(neuron_src->neuron_id,
				neuron_tgt->neuron_id,&pos))
			{
				connection = &neuron_src->connections[pos];
				current_color = eval_color(connection->synaptic_knobs);
				
				XSetForeground(the_display,the_GC,current_color);
				XSetBackground(the_display,the_GC,white);
				XSetLineAttributes(the_display,the_GC,2,LineSolid,
					CapRound,JoinRound);
				XFillRectangle(the_display,matrix_pixmap,
					the_GC,pos_x,pos_y,size,size);
				
				if(connection->nmda)
				{
					XSetForeground(the_display,the_GC,black);	
					XFillArc(the_display,matrix_pixmap,
						the_GC,pos_x+size/4,pos_y+size/4,size/2,size/2,
						0*64,360*64);
				}
			}
			else
			{
				XSetForeground(the_display,the_GC,grey);
				XSetBackground(the_display,the_GC,white);
				XSetLineAttributes(the_display,the_GC,2,LineSolid,
					CapRound,JoinRound);
				XFillRectangle(the_display,matrix_pixmap,
					the_GC,pos_x,pos_y,size,size);
			}
			X++;
			pos_x += size;
		
		}
		X = 1;
		pos_x = 0;
		Y++;
		pos_y += size;
	}
	

}
/*******************************************************************************
	Function for evaluating the color of a connection according to
	the max and min values of the current link
*******************************************************************************/
unsigned long eval_color(int synaptic_knobs)
{
	int  max,min,delta;
	float d;
	
	
	max = current_link->max_knobs;
	min = current_link->min_knobs;
	delta = max - min;
	
	if(!synaptic_knobs)
		return grey;
	else if(synaptic_knobs > 0)
	{
		if(!delta)
			return red[4];
		d = (float)(synaptic_knobs - min)/(float)delta;
		if(d < 0.2)
			return red[0];
		else if(d < 0.4)
			return red[1];
		else if(d < 0.6)
			return red[2];
		else if(d < 0.8)
			return red[3];
		else if(d >= 0.8)
			return red[4];
			
	}
	else
	{
		if(!delta)
			return blue[4];
		d = (float)( - synaptic_knobs - min)/(float)delta;
		if(d < 0.2)
			return blue[0];
		else if(d < 0.4)
			return blue[1];
		else if(d < 0.6)
			return blue[2];
		else if(d < 0.8)
			return blue[3];
		else if(d >= 0.8)
			return blue[4];
	}

}


/*******************************************************************************
	Function for moving one connection curve point
*******************************************************************************/
void move_point(Widget w,int message, XEvent *evt)
{
	int posX,posY;
	static int lastX=0;
	static int lastY=0;
	short X,Y;
	int i;
	int X1,Y1,X2,Y2;
	int Xmin;
	int Ymin;
	int Xmax;
	int Ymax;
	int Ystep;
	int Xstep;
	short width,height;
	char buffer[255];
	float length,probability;
	Widget label_weight = widget_array[ID_Label_connection_weight];
	XmString string;
	PROBA_POINT point1,point2;
	
	
	XtVaGetValues(widget_array[ID_DrawingA_connection_curve],
		XmNwidth, &width, XmNheight ,&height,NULL);
	
	while((width-100)%100)
		width--;
	while((height-100)%100)
		height--;
			
	Xmin = 50;
	Xmax = width - 50;
	Ymin = height - 50;
	Ymax = 50;
	
	XtVaGetValues(w,XmNx,&X,XmNy,&Y,NULL);
	
	posX = evt->xbutton.x + X+POINT_SIZE/2; 
	posY = evt->xbutton.y + Y+POINT_SIZE/2;
	
	if((posX==lastX) && (posY==lastY))
		return;
	
	length = (float)((float)(posX - Xmin)/(float)(Xmax - Xmin))*100.0;
	probability = (float)((float)(posY - Ymin)/(float)(Ymax - Ymin))*100.0;
	if(probability <0.0)
		probability = 0.0;
	else if(probability>100.0)
		probability = 100.0;
	if(length <0.0)
		length = 0.0;
	else if(length>100.0)
		length = 100.0;
			
	sprintf(buffer,"C.Rate: %.2f%%  L.Rate: %.2f%%",
		probability,
		length);
	string = XmStringCreateLocalized(buffer);
	XtVaSetValues(label_weight,XmNlabelString,string,NULL);
	XmStringFree(string);
	
	XClearWindow(the_display,XtWindow(widget_array[ID_DrawingA_connection_curve]));
	if((message>0) && (message<9))
	{
		X1 = (int)(Xmin + current_link->points[message-1].length*((width-100)/100)) - POINT_SIZE/2;
		Y1 = (int)(Ymin - current_link->points[message-1].probability*((height-100)/100)) - POINT_SIZE/2;
		X2 = (int)(Xmin + current_link->points[message+1].length*((width-100)/100)) - POINT_SIZE/2;
		Y2 = (int)(Ymin - current_link->points[message+1].probability*((height-100)/100)) - POINT_SIZE/2;
		
	}
	else if (message == 0)
	{
		X1 = (int)(Xmin);
		Y1 = (int)(posY);
		X2 = (int)(Xmin + current_link->points[message+1].length*((width-100)/100)) - POINT_SIZE/2;
		Y2 = (int)(Ymin - current_link->points[message+1].probability*((height-100)/100)) - POINT_SIZE/2;
					
	}
	else if (message == 9)
	{
		X1 = (int)(Xmin + current_link->points[message-1].length*((width-100)/100)) - POINT_SIZE/2;
		Y1 = (int)(Ymin - current_link->points[message-1].probability*((height-100)/100)) - POINT_SIZE/2;
		X2 = (int)(Xmax); 
		Y2 = (int)(posY);
	}
	
	XSetFunction(the_display,the_GC,GXcopy);
	XSetBackground(the_display,the_GC,white);
	XSetForeground(the_display,the_GC,black);
	XSetLineAttributes(the_display,the_GC,1,LineOnOffDash,CapRound,JoinRound);

	XDrawRectangle(the_display,XtWindow(widget_array[ID_DrawingA_connection_curve]),
					the_GC,posX-POINT_SIZE/2,posY-POINT_SIZE/2,
					POINT_SIZE,POINT_SIZE);

	XDrawLine(the_display,XtWindow(widget_array[ID_DrawingA_connection_curve]),
					the_GC,posX,posY,
					X1,Y1);
	XDrawLine(the_display,XtWindow(widget_array[ID_DrawingA_connection_curve]),
					the_GC,posX,posY,
					X2,Y2);	
						
	lastX = posX;
	lastY = posY;

}

/*******************************************************************************
	Function for releasing one connection curve point
*******************************************************************************/
void release_point(Widget w,int message, XEvent *evt)
{
	int posX,posY;
	static int lastX=0;
	static int lastY=0;
	short X,Y;
	float probability;
	float length;
	int i;
	int X1,Y1,X2,Y2;
	int Xmin;
	int Ymin;
	int Xmax;
	int Ymax;
	int Ystep;
	int Xstep;
	short width,height;
	char buffer[255];
	PROBA_POINT point1,point2;
	
	
	XtVaGetValues(widget_array[ID_DrawingA_connection_curve],
		XmNwidth, &width, XmNheight ,&height,NULL);
	
	while((width-100)%100)
		width--;
	while((height-100)%100)
		height--;
			
	Xmin = 50;
	Xmax = width - 50;
	Ymin = height - 50;
	Ymax = 50;
	
	
	XtVaGetValues(w,XmNx,&X,XmNy,&Y,NULL);
	
	posX = evt->xbutton.x + X+POINT_SIZE/2; 
	posY = evt->xbutton.y + Y+POINT_SIZE/2;
	
	length = (float)((float)(posX - Xmin)/(float)(Xmax - Xmin))*100.0;
	probability = (float)((float)(posY - Ymin)/(float)(Ymax - Ymin))*100.0;
	if(probability <0.0)
		probability = 0.0;
	else if(probability>100.0)
		probability = 100.0;
	
	if((message>0) && (message<9))
	{
		if((length < current_link->points[message+1].length) &&
			(length >current_link->points[message-1].length))
		{
			current_link->points[message].length = length;
			current_link->points[message].probability = probability;
			XSetFunction(the_display,the_GC,GXcopy);
			XSetBackground(the_display,the_GC,black);
			XSetForeground(the_display,the_GC,white);
			XFillRectangle(the_display,curve_pixmap,
						the_GC,0,0,
						width,height);
		
			update_density();
			
			draw_curve_axes();
	
			draw_points();
	
			draw_curve();
		}		
	}
	else if (message == 0)
	{
		if((length >= 0.0)  && (length < current_link->points[message+1].length))
		{
			current_link->points[message].length = length;
			current_link->points[message].probability = probability;
			XSetFunction(the_display,the_GC,GXcopy);
			XSetBackground(the_display,the_GC,black);
			XSetForeground(the_display,the_GC,white);
			XFillRectangle(the_display,curve_pixmap,
						the_GC,0,0,
						width,height);
			
			update_density();
			
			draw_curve_axes();
	
			draw_points();
	
			draw_curve();
		}
	}
	else if (message == 9)
	{
		if((length <= 100.0) && (length >current_link->points[message-1].length))
		{
			current_link->points[message].length = length;
			current_link->points[message].probability = probability;
			XSetFunction(the_display,the_GC,GXcopy);
			XSetBackground(the_display,the_GC,black);
			XSetForeground(the_display,the_GC,white);
			XFillRectangle(the_display,curve_pixmap,
						the_GC,0,0,
						width,height);
		
			update_density();
			
			draw_curve_axes();
	
			draw_points();
	
			draw_curve();
		}
	}
	
	changed = True;
	
	XClearWindow(the_display,XtWindow(widget_array[ID_DrawingA_connection_curve]));
	
	
}
/*******************************************************************************
	Function for help on line on connection curve 
*******************************************************************************/
void info_point(Widget w,int message, XEvent *evt)
{
	Widget label_weight = widget_array[ID_Label_connection_weight];
	Widget label_info = widget_array[ID_Label_curve_status];
	XmString string;
	char buffer[255];
	
	if(evt->type == EnterNotify)
	{
		sprintf(buffer,"C.Rate: %.2f%%  L.Rate: %.2f%% S.C.N.Density: %.2f%%",
			current_link->points[message].probability,
			current_link->points[message].length,
			current_link->points[message].density);
		string = XmStringCreateLocalized(buffer);
		XtVaSetValues(label_weight,XmNlabelString,string,NULL);
		XmStringFree(string); 
	
	
		sprintf(buffer,"     Click here,move and release to modify the curve");
		string = XmStringCreateLocalized(buffer);
		XtVaSetValues(label_info,XmNlabelString,string,NULL);
		XmStringFree(string); 
	}
}

/*******************************************************************************
	Function for clearing connection curve  label info
*******************************************************************************/
void clear_info_curve(Widget w,int message, XEvent *evt)
{
	Widget label_weight = widget_array[ID_Label_connection_weight];
	Widget label_info = widget_array[ID_Label_curve_status];
	XmString string;
	
	string = XmStringCreateLocalized("                             ");
	XtVaSetValues(label_weight,XmNlabelString,string,NULL);
	XtVaSetValues(label_info,XmNlabelString,string,NULL);
	XmStringFree(string);

}
/*******************************************************************************
	Function for resizing connection curve  
*******************************************************************************/
void resize_curve(Widget w,int message, XEvent *evt)
{

	if(XtIsManaged(widget_array[ID_FormD_adjustement_curve]))
				redraw_connection_curve();
	
}

/*******************************************************************************
	Error handler for pixmap allocation
*******************************************************************************/
int err_alloc_pixmap(Display *disp,XErrorEvent *evt)
{
	
#if DEBUG	
	printf("\n X pixmap allocation error :\n");
#endif
	
	test_alloc = 0;
	return test_alloc;
	
}

/***************************************************************************
		Function providing the help on line for the matrix editor
****************************************************************************/

void fill_label_matrix_info(Widget w,int message, XEvent *evt)
{
	XmString string;
	char buffer[256];
	Widget label_info = widget_array[ID_Label_matrix_information];
	static int lastX=0;
	static int lastY=0;
	int pos_x,pos_y,X,Y;
	Nucleus_widget *nucleus_src;
	Nucleus_widget *nucleus_tgt;
	int i,j,k;
	CLUSTER_WIDGET *cluster_tmp;
	NEURON *neuron_src;
	NEURON *neuron_tgt;
	int pos;
	Connection *connection;
	int knobs;

	
	
	switch(message)
	{
	
		case ID_DrawingA_matrix:
			pos_x = evt->xbutton.x;
			pos_y = evt->xbutton.y;
			X = pos_x/foot;
			Y = pos_y/foot;
			nucleus_src = search_nucleus(current_link->source_id);
			nucleus_tgt = search_nucleus(current_link->target_id);
			if( ((X+1) > nucleus_tgt->nb_neurons) ||
				((Y+1) >nucleus_src->nb_neurons))
				break;
			else if(((X+1) < 1) ||
				(Y+1) < 1)
				break;
			/* now identify the involved neurons */
			
			k = 0;
			for(i=0 ; i<nucleus_src->nb_clusters ; i++)
			{
				cluster_tmp = nucleus_src->cluster_array[i];
				for(j=0 ; j<cluster_tmp->nb_neurons ; j++)
				{
					k++;
					if(k == Y+1)
					{
						neuron_src = cluster_tmp->neuron_array[j];
					}
				}
			}			
			k = 0;
			for(i=0 ; i<nucleus_tgt->nb_clusters ; i++)
			{
				cluster_tmp = nucleus_tgt->cluster_array[i];
				for(j=0 ; j<cluster_tmp->nb_neurons ; j++)
				{
					k++;
					if(k == X+1)
					{
						neuron_tgt = cluster_tmp->neuron_array[j];
					}
				}
			}				

			if(connection = search_connection(neuron_src->neuron_id,
				neuron_tgt->neuron_id,&pos))
			{
				knobs = connection[pos].synaptic_knobs;
			}
			else
			{
				knobs = 0;
			}
			
			if(!knobs)
				sprintf(buffer,"Neuron %d (%s) is not connected to Neuron %d (%s)",
					neuron_src->neuron_id, nucleus_src->nucleus_name,
					neuron_tgt->neuron_id, nucleus_tgt->nucleus_name);
			else
				sprintf(buffer,"Neuron %d (%s) to Neuron %d (%s): weigth = %d , length = %.2f ms",
					neuron_src->neuron_id, nucleus_src->nucleus_name,
					neuron_tgt->neuron_id, nucleus_tgt->nucleus_name,
						knobs,connection[pos].axone_length);
			
			string = XmStringCreateLocalized(buffer);
			
			XtVaSetValues(label_info,XmNlabelString,string,NULL);
			
			XmStringFree(string);
			
			
			break;

		default:
			
			break;

	}
}

/*****************************************************************************
	Function for clearing the help on line label in the matrix editor 
*****************************************************************************/
void clear_label_matrix_info(Widget w,int message, XEvent *evt)
{
	XmString string;
	Widget label_info = widget_array[ID_Label_matrix_information];

	string = XmStringCreateLocalized("                                    ");
	XtVaSetValues(label_info,XmNlabelString,string,NULL);
	XmStringFree(string);
			

}

/*****************************************************************************
	Function for clearing the help on line label in the connectivity scope
	editor 
*****************************************************************************/
void clear_label_curve_info(Widget w,int message, XEvent *evt)
{
	XmString string;
	Widget label_info = widget_array[ID_Label_curve_status];

	string = XmStringCreateLocalized("                                    ");
	XtVaSetValues(label_info,XmNlabelString,string,NULL);
	XmStringFree(string);
			

}

/*****************************************************************************
	Function for filling the help on line label in the connectivity scope
	editor 
*****************************************************************************/
void fill_label_curve_info(Widget w,int message, XEvent *evt)
{
	XmString string;
	Widget label_info = widget_array[ID_Label_curve_status];

	
	switch(message)
	{
		case ID_push_up:
			string = XmStringCreateLocalized("Draw curve in full connection state...");
			XtVaSetValues(label_info,XmNlabelString,string,NULL);
			XmStringFree(string);
			break;
		case ID_push_down:
			string = XmStringCreateLocalized("Draw curve in NO connection state...");
			XtVaSetValues(label_info,XmNlabelString,string,NULL);
			XmStringFree(string);
			break;
		case ID_push_default:
			string = XmStringCreateLocalized("Draw default curve...");
			XtVaSetValues(label_info,XmNlabelString,string,NULL);
			XmStringFree(string);
			break;
		case ID_push_gauss:
			string = XmStringCreateLocalized("Draw Gauss curve...");
			XtVaSetValues(label_info,XmNlabelString,string,NULL);
			XmStringFree(string);
			break;
		case ID_push_plus:
			string = XmStringCreateLocalized("Translate curve higher...");
			XtVaSetValues(label_info,XmNlabelString,string,NULL);
			XmStringFree(string);
			break;
		
		case ID_push_auto:
			string = XmStringCreateLocalized("Avoid AUTO connections...");
			XtVaSetValues(label_info,XmNlabelString,string,NULL);
			XmStringFree(string);
			break;
		case ID_push_minus:
			string = XmStringCreateLocalized("Translate curve lower...");
			XtVaSetValues(label_info,XmNlabelString,string,NULL);
			XmStringFree(string);
			break;
		case ID_PushBG_curve_ok:
			string = XmStringCreateLocalized("Pops up the connection matrix...");
			XtVaSetValues(label_info,XmNlabelString,string,NULL);
			XmStringFree(string);
			break;
		case ID_PushBG_curve_apply:
			string = XmStringCreateLocalized("Pops up the connection matrix...");
			XtVaSetValues(label_info,XmNlabelString,string,NULL);
			XmStringFree(string);
			break;
		case ID_PushBG_curve_cancel:
			string = XmStringCreateLocalized("Leave with no modifications...");
			XtVaSetValues(label_info,XmNlabelString,string,NULL);
			XmStringFree(string);
			break;
		case ID_PushBG_curve_help:
			string = XmStringCreateLocalized("Help on context...");
			XtVaSetValues(label_info,XmNlabelString,string,NULL);
			XmStringFree(string);
			break;
				
		default:
			break;
	
	}
			

}

/*******************************************************************************
	Function for calculating the connecting rate for each point of the 
	connection curve of the current link according to the Gauss formula
*******************************************************************************/
void calculate_gauss(void)
{

	int i;
	float m;
	float sigma;
	float e;
	
	if(!current_link)
	{
		fprintf(stderr,"current_link is NULL in calculate_gauss!!!!!!\n");
		exit(-1);
	}
	
	m = 0.0;
	for(i=0 ; i<_NB_INTERPOLATING_POINTS_ ; i++)
	{
		m += current_link->points[i].length;	
	}
	m /= _NB_INTERPOLATING_POINTS_;

	sigma = 0.0;
	for(i=0 ; i<_NB_INTERPOLATING_POINTS_ ; i++)
	{
		sigma += current_link->points[i].length*current_link->points[i].length;
	}
	sigma /= _NB_INTERPOLATING_POINTS_;
	sigma -= m*m;
	sigma = (float)sqrt((double)sigma);
	
	for(i=0 ; i<_NB_INTERPOLATING_POINTS_ ; i++)
	{
		e = -(current_link->points[i].length+sigma)*(current_link->points[i].length - m);
		e /= 2*sigma*sigma;
		current_link->points[i].probability = (float)exp((double)e);
		current_link->points[i].probability *= 1000.0/sigma;
		if(current_link->points[i].probability < 0.0)
			current_link->points[i].probability = 0.0;
		else if(current_link->points[i].probability > 100.0)
			current_link->points[i].probability = 100.0;
	}
	
}
/*******************************************************************************
	Function for drawing a selection rectangle in the connection matrix
*******************************************************************************/
void select_rectangle(Widget w,int message, XEvent *evt)
{
	int pos_x,pos_y,pos_x2,pos_y2;
	int X1,X2,Y1,Y2;
	Nucleus_widget *nucleus_src = search_nucleus(current_link->source_id);
	Nucleus_widget *nucleus_tgt = search_nucleus(current_link->target_id);
	
	
	pos_x = x_glb * foot;
	pos_y = y_glb * foot;
	pos_x2 = evt->xbutton.x; 
    pos_y2 = evt->xbutton.y;
    X2 = pos_x2 / foot;
    Y2 = pos_y2 / foot;
    if(X2 < 0)
    	X2 = 0;
    if(X2 >= nucleus_src->nb_neurons)
    	X2 = nucleus_src->nb_neurons - 1;
    if(Y2 < 0)
    	Y2 = 0;
    if(Y2 >= nucleus_tgt->nb_neurons)
    	Y2 = nucleus_tgt->nb_neurons - 1;
    pos_x2 = X2*foot;
    pos_y2 = Y2*foot;
    
    if(X2 >= x_glb)
   		pos_x2 += foot;
   	else
   		pos_x += foot;
    if(Y2 >= y_glb)
    	pos_y2 += foot;
    else
    	pos_y += foot;
    	
    XClearWindow(the_display,XtWindow(widget_array[ID_DrawingA_matrix]));

    XSetFunction(the_display,the_GC,GXcopy);
	XSetBackground(the_display,the_GC,white);
	XSetForeground(the_display,the_GC,black);
	XSetLineAttributes(the_display,the_GC,1,LineOnOffDash,CapRound,JoinRound);

	XDrawRectangle(the_display,XtWindow(widget_array[ID_DrawingA_matrix]),
			the_GC,MIN(pos_x2,pos_x),MIN(pos_y2,pos_y),
			MAX(pos_x2,pos_x) - MIN(pos_x2,pos_x),
			MAX(pos_y2,pos_y) - MIN(pos_y2,pos_y));

        		
}

#endif
