/*
=============================================================================
	M A T R I X _ V I S U . C
=============================================================================

	Author: Denis LAMBOLEZ (Feb 1993)
		Laboratoire de Biomathematiques
		URBB B3E INSERM U263
                Faculte de Medecine Saint-Antoine
                27 rue Chaligny
                75571 Paris cedex 12
			France

	Copyright Dr JF Vibert, July 1993
	All rights reserved
	Access limited to authorized persons	
	This software was developped only for research purpose
	and is not, in any way, usable for commercial purpose.

=============================================================================
	CONTENTS
=============================================================================
	- COMPUTE_GEOMETRY_MATRIX : This function computes the geometry for
				    the matrix representation.
	- REALIZE_MATRIX : This function allocates memory for the matrix 
			   representation and initialize it.
	- DRAW_DETAILS_MATRIX : This function draw details of the matrix 
			        representation in the exposed region.
	- DRAW_MATRIX : This function draw the matrix representation in the 
			exposed region.
	- DRAW_NEURONS_VALUES_MATRIX : 	This function draws neurons with the 
					color corresponding to their value.
	- CREATE_SPIKE : This function creates new spike in the spike queue.
	- DRAW_SPIKES_MATRIX : This function draws spikes on the network 
				if the spike queue is not empty.
	- PRINT_DETAILS_MATRIX : This function prints details of the 
				 representation like synaptic knobs symbols 
				 or connexion symbols.
	- PRINT_MATRIX : This function prints the matrix representation at its 
			 current iteration. It prints entire network or just
			 the visible part.
	- FREE_MATRIX : This function free the matrix representation.
	- INFO_MATRIX : This function constructs the information string for the
			matrix representation.
=============================================================================
*/

#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <Xm/Text.h>
#include <Mrm/MrmAppl.h>                /* Motif Toolkit and MRM */
#include <X11/X.h>

#include "global_def_visu.h"
#include <plot_ps.h>

/*This macro returns True if the square (X2, Y2, S2) intersects the
rectangle (X1, Y1, W1, H1) else it returns False*/

#define INTERSECT(X1, Y1, W1, H1, X2, Y2, S2) \
		 ((((X1)+(W1)) >= (X2) && ((X2)+(S2)) >= (X1)) && \
		  (((Y1)+(H1)) >= (Y2) && ((Y2)+(S2)) >= (Y1)))

/*This macro returns True if the given point (X1, Y1) is in the window 
(X, Y, W, H)*/
#define IN_WINDOW(X, Y, W, H, X1, Y1) ((X1) >= (X) && (X1) <= (X) + (W) && \
			   (Y1) >= (Y) && (Y1) <= (Y) + (H))


/*Constantes*/
#define NO_CONNEXION       0
#define POSITIVE_CONNEXION 1
#define NEGATIVE_CONNEXION 2
#define POSNEG_CONNEXION   3

/*foreward decalaration*/
extern void COMPUTE_GEOMETRY_MATRIX();
extern void DRAW_DETAILS_MATRIX(int,int, int, int);
extern void CREATE_SPIKE(int);
extern void DRAW_SPIKES_MATRIX(int, int, int, int, int);
extern void PRINT_DETAILS_MATRIX(float);


/*
=============================================================================
Name and dates: COMPUTE_GEOMETRY_MATRIX
---------------
	Creation : 10-05-93
	Last modification : 10-05-93

Functionnal description :
-------------------------
	this function computes the geometry for the matrix representation
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	None

Explicits outputs :	None

Implicits inputs  :	NETWORK	 	NETWORK_DEF	The definition of the network

Implicits outputs :	NETWORK	 	NETWORK_DEF	The definition of the network

Sides effects :		None
---------------

=============================================================================
*/

void COMPUTE_GEOMETRY_MATRIX()
{
	XSegment *CURRENT_AXON, *CURRENT_DENDRITE;
	short *SYNAPSES_BUFFER = Null;	/*to count for each neuron the number of realized synapses*/
	XSegment TMP;
	int i, j, k;			/*not important stuff*/
	int INDEX;			/*position in geometry arrays*/
	int OFFSET;			/*offset for the first point of a dendrite line*/
	int INIT_STEP,STEP;		/*steps for the second point of a dendrite line*/
	int SEPARATE;			/*Flag. If True we draw separates dendrites else we group them*/
	int AXON_EXIST;			/*flag*/
	int DENDRITE_EXIST;		/*flag*/
	
/*computes clusters geometry*/

	/*First cluster*/
	NETWORK.MATRIX->CLUSTER_GEOMETRY->x = 0;
	NETWORK.MATRIX->CLUSTER_GEOMETRY->y = 0;
	NETWORK.MATRIX->CLUSTER_GEOMETRY->width = *NETWORK.NB_NEURONS_PER_CLUSTER *
							NETWORK.MATRIX->NEURON_SIZE;
	NETWORK.MATRIX->CLUSTER_GEOMETRY->height = NETWORK.MATRIX->CLUSTER_GEOMETRY->width;
	/*Next clusters*/
	for(i=1; i<NETWORK.NB_CLUSTERS; i++)
	{
		k = i-1;
		(NETWORK.MATRIX->CLUSTER_GEOMETRY + i)->x = (NETWORK.MATRIX->CLUSTER_GEOMETRY + k)->x +
							    (NETWORK.MATRIX->CLUSTER_GEOMETRY + k)->width;
		(NETWORK.MATRIX->CLUSTER_GEOMETRY + i)->y = (NETWORK.MATRIX->CLUSTER_GEOMETRY + k)->y +
							    (NETWORK.MATRIX->CLUSTER_GEOMETRY + k)->height;
		(NETWORK.MATRIX->CLUSTER_GEOMETRY + i)->width = *(NETWORK.NB_NEURONS_PER_CLUSTER + i) *
							        NETWORK.MATRIX->NEURON_SIZE;
		(NETWORK.MATRIX->CLUSTER_GEOMETRY + i)->height = (NETWORK.MATRIX->CLUSTER_GEOMETRY + i)->width;
	}

/*computes axons geometry*/
	/*computes offset*/
	OFFSET = (int)floor((double)NETWORK.MATRIX->NEURON_SIZE/(double)2);
	/*computes step*/
	STEP = (int)floor((double)NETWORK.MATRIX->NEURON_SIZE/(double)(NETWORK.MATRIX->MAX_SYNAPSE + 1));
	if(STEP < 3)	/*minimum for lisibility*/
	{
		STEP = 0;
		INIT_STEP = OFFSET;
		SEPARATE = False;
	}
	else
	{
		SEPARATE = True;
		INIT_STEP = STEP;
		SYNAPSES_BUFFER = (short *)calloc(sizeof(short), (size_t)NETWORK.NB_NEURONS);
		if(SYNAPSES_BUFFER == Null)
		{
			STEP = 0;
			INIT_STEP = OFFSET;
		}
	}
	/*keep the step*/
	NETWORK.MATRIX->STEP = STEP;
	INDEX = 0;		/*position in the axon to lower geometry array*/
	/*Axons to a lower neuron*/
	for(i=0; i<NETWORK.NB_NEURONS; i++)    /*From neuron i to neuron j*/
	{
		AXON_EXIST = False;
		for(j=i+1; j<NETWORK.NB_NEURONS; j++)
			if(*(NETWORK.CONNEXION_MATRIX + (i * NETWORK.NB_NEURONS) + j) != 0 && 
			   *(NETWORK.SYNAPTIC_MATRIX + (i * NETWORK.NB_NEURONS) + j) != 0)
			{
				CURRENT_AXON = NETWORK.MATRIX->AXON_GEOMETRY_TO_LOWER + INDEX;
				/*first point*/
				(CURRENT_AXON)->x1 = (i + 1) * NETWORK.MATRIX->NEURON_SIZE;
				(CURRENT_AXON)->y1 = i * NETWORK.MATRIX->NEURON_SIZE + OFFSET;
				/*second point*/
				(CURRENT_AXON)->x2 =
						(SYNAPSES_BUFFER != Null)                           ?
						   (j + 1) * NETWORK.MATRIX->NEURON_SIZE -
						   INIT_STEP - (*(SYNAPSES_BUFFER + j) * STEP)      :
						   (j + 1) * NETWORK.MATRIX->NEURON_SIZE - INIT_STEP;
				(CURRENT_AXON)->y2 = (CURRENT_AXON)->y1;
				AXON_EXIST = True;
				if(SYNAPSES_BUFFER != Null)
					(*(SYNAPSES_BUFFER + j))++;	/*next synapse for the neuron*/
			}
		if(AXON_EXIST) INDEX++;
	}
	/*Axons to a upper neuron*/
	INDEX = 0;		/*position in the axon to upper geometry array*/
	if(SYNAPSES_BUFFER != Null)
		for(i=0; i<NETWORK.NB_NEURONS; *(SYNAPSES_BUFFER + (i++)) = 0);
	for(i=0; i<NETWORK.NB_NEURONS; i++)	/*from neuron i to neuron j*/
	{
		AXON_EXIST = False;
		for(j=0; j<i; j++)
			if(*(NETWORK.CONNEXION_MATRIX + (i * NETWORK.NB_NEURONS) + j) != 0&& 
			   *(NETWORK.SYNAPTIC_MATRIX + (i * NETWORK.NB_NEURONS) + j) != 0)
			{
				CURRENT_AXON = NETWORK.MATRIX->AXON_GEOMETRY_TO_UPPER + INDEX;
				/*first point*/
				(CURRENT_AXON)->x1 = i * NETWORK.MATRIX->NEURON_SIZE;
				(CURRENT_AXON)->y1 = (CURRENT_AXON)->x1 + OFFSET;
				/*second point*/
				(CURRENT_AXON)->x2 =
						(SYNAPSES_BUFFER != Null)                           ?
						   (j + 1) * NETWORK.MATRIX->NEURON_SIZE -
						   INIT_STEP - (*(SYNAPSES_BUFFER + j) * STEP)      :
						   (j + 1) * NETWORK.MATRIX->NEURON_SIZE - INIT_STEP;
				(CURRENT_AXON)->y2 = (CURRENT_AXON)->y1;
				AXON_EXIST = True;
				if(SYNAPSES_BUFFER != Null)
					(*(SYNAPSES_BUFFER + j))++;	/*next synapse for the neuron*/
				break;	/*computes only the first connexion because this is the bigger*/
			}
		if(AXON_EXIST) INDEX++;
	}

/*computes dendrites geometry*/
	if(SEPARATE)	/*separated dendrites*/
	{
		/*dendrites to a lower neuron*/
		INDEX =0;
		if(SYNAPSES_BUFFER != Null)
			for(i=0; i<NETWORK.NB_NEURONS; *(SYNAPSES_BUFFER + (i++)) = 0);
		for(i=0; i<NETWORK.NB_NEURONS; i++)     /*From neuron i to neuron j*/
		{
			for(j=i+1; j<NETWORK.NB_NEURONS; j++)
			{
				if(*(NETWORK.CONNEXION_MATRIX + (i * NETWORK.NB_NEURONS) + j) != 0 && 
			           *(NETWORK.SYNAPTIC_MATRIX + (i * NETWORK.NB_NEURONS) + j) != 0)
				{
					CURRENT_DENDRITE = NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER + INDEX;
					/*first point*/
					(CURRENT_DENDRITE)->x1 =
							(SYNAPSES_BUFFER != Null)                         ?
							(j + 1) * NETWORK.MATRIX->NEURON_SIZE -
							INIT_STEP - (*(SYNAPSES_BUFFER + j) * STEP)       :
							(j + 1) * NETWORK.MATRIX->NEURON_SIZE - INIT_STEP;
					(CURRENT_DENDRITE)->y1 = i * NETWORK.MATRIX->NEURON_SIZE + OFFSET;
					/*second point*/
					(CURRENT_DENDRITE)->x2 = (CURRENT_DENDRITE)->x1;
					(CURRENT_DENDRITE)->y2 = j * NETWORK.MATRIX->NEURON_SIZE  - 3;
					if(SYNAPSES_BUFFER != Null)
						(*(SYNAPSES_BUFFER + j))++;	/*next synapse for the neuron*/
					/*next lower dendrite*/
					INDEX++;
				}
			}
		}
		/*class dendrites from lower x1 to greater x1*/
		for(i=0; i<NETWORK.MATRIX->NB_DENDRITES_TO_LOWER-1; i++)
			for(j=i+1; j<NETWORK.MATRIX->NB_DENDRITES_TO_LOWER; j++)
				if((NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER + j)->x1 <
				   (NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER + i)->x1)
				{
					/*save*/
					TMP.x1 = (NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER + i)->x1;
					TMP.y1 = (NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER + i)->y1;
					TMP.x2 = (NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER + i)->x2;
					TMP.y2 = (NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER + i)->y2;
					/*inverse*/
					(NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER + i)->x1 =
						(NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER + j)->x1;
					(NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER + i)->y1 =
						(NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER + j)->y1;
					(NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER + i)->x2 =
						(NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER + j)->x2;
					(NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER + i)->y2 =
						(NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER + j)->y2;
					/*restore*/
					(NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER + j)->x1 = TMP.x1;
					(NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER + j)->y1 = TMP.y1;
					(NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER + j)->x2 = TMP.x2;
					(NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER + j)->y2 = TMP.y2;
				}
		/*dendrites to a upper neuron*/
		INDEX =0;
		if(SYNAPSES_BUFFER != Null)
			for(i=0; i<NETWORK.NB_NEURONS; *(SYNAPSES_BUFFER + (i++)) = 0);
		for(i=0; i<NETWORK.NB_NEURONS; i++)  /*From neuron i to neuron j*/
		{
			for(j=0; j<i; j++)
			{
				if(*(NETWORK.CONNEXION_MATRIX + (i * NETWORK.NB_NEURONS) + j) != 0 && 
			           *(NETWORK.SYNAPTIC_MATRIX + (i * NETWORK.NB_NEURONS) + j) != 0)
				{
					CURRENT_DENDRITE = NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER + INDEX;
					/*first point*/
					(CURRENT_DENDRITE)->x1 =
							(SYNAPSES_BUFFER != Null)                      ?
							(j + 1) * NETWORK.MATRIX->NEURON_SIZE -
							INIT_STEP - (*(SYNAPSES_BUFFER + j) * STEP) :
							(j + 1) * NETWORK.MATRIX->NEURON_SIZE -
							INIT_STEP                                   ;
					(CURRENT_DENDRITE)->y1 = i * NETWORK.MATRIX->NEURON_SIZE + OFFSET;
					/*second point*/
					(CURRENT_DENDRITE)->x2 = (CURRENT_DENDRITE)->x1;
					(CURRENT_DENDRITE)->y2 =(j + 1) * NETWORK.MATRIX->NEURON_SIZE + 2;
					/*next upper dendrite*/
					INDEX++;
					if(SYNAPSES_BUFFER != Null)
						(*(SYNAPSES_BUFFER + j))++;	/*next synapse for the neuron*/
				}
			}
		}
		/*class dendrites from lower x1 to greater x1*/
		for(i=0; i<NETWORK.MATRIX->NB_DENDRITES_TO_UPPER-1; i++)
			for(j=i+1; j<NETWORK.MATRIX->NB_DENDRITES_TO_UPPER; j++)
				if((NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER + j)->x1 <
				   (NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER + i)->x1)
				{
					/*save*/
					TMP.x1 = (NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER + i)->x1;
					TMP.y1 = (NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER + i)->y1;
					TMP.x2 = (NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER + i)->x2;
					TMP.y2 = (NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER + i)->y2;
					/*inverse*/
					(NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER + i)->x1 =
						(NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER + j)->x1;
					(NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER + i)->y1 =
						(NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER + j)->y1;
					(NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER + i)->x2 =
						(NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER + j)->x2;
					(NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER + i)->y2 =
						(NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER + j)->y2;
					/*restore*/
					(NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER + j)->x1 = TMP.x1;
					(NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER + j)->y1 = TMP.y1;
					(NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER + j)->x2 = TMP.x2;
					(NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER + j)->y2 = TMP.y2;
				}
	}/*end of separated dendrites*/
	else	/*grouped dendrites*/
	{
		/*dendrites to a lower neuron (from neuron j to neuron i)*/
		INDEX =0;
		for(i=1; i<NETWORK.NB_NEURONS; i++)
		{
			for(j=0; j<i; j++)
			{
				if(*(NETWORK.CONNEXION_MATRIX + (j * NETWORK.NB_NEURONS) + i) != 0 &&
				   *(NETWORK.SYNAPTIC_MATRIX + (j * NETWORK.NB_NEURONS) + i) != 0)
				{
					CURRENT_DENDRITE = NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER + INDEX;
					/*first point*/
					(CURRENT_DENDRITE)->x1 = (i + 1) * NETWORK.MATRIX->NEURON_SIZE - INIT_STEP;
					(CURRENT_DENDRITE)->y1 = j * NETWORK.MATRIX->NEURON_SIZE + OFFSET;
					/*second point*/
					(CURRENT_DENDRITE)->x2 = (CURRENT_DENDRITE)->x1;
					(CURRENT_DENDRITE)->y2 = i  * NETWORK.MATRIX->NEURON_SIZE  - 3;
					/*next lower dendrite*/
					INDEX++;
					break; /*we draw only the first dendrite because it's the bigger*/
				}
			}
		}
		/*dendrites to a upper neuron (from neuron j to neuron i)*/
		INDEX =0;
		for(i=0; i<NETWORK.NB_NEURONS; i++)
		{
			DENDRITE_EXIST = False;
			for(j=i+1; j<NETWORK.NB_NEURONS; j++)
			{
				if(*(NETWORK.CONNEXION_MATRIX + (j * NETWORK.NB_NEURONS) + i) != 0 &&
				   *(NETWORK.SYNAPTIC_MATRIX + (j * NETWORK.NB_NEURONS) + i) != 0)
				{
					CURRENT_DENDRITE = NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER + INDEX;
					/*first point*/
					(CURRENT_DENDRITE)->x1 = (i+1) * NETWORK.MATRIX->NEURON_SIZE - INIT_STEP;
					(CURRENT_DENDRITE)->y1 = j * NETWORK.MATRIX->NEURON_SIZE + OFFSET;
					/*second point*/
					(CURRENT_DENDRITE)->x2 = (CURRENT_DENDRITE)->x1;
					(CURRENT_DENDRITE)->y2 = (i+1) * NETWORK.MATRIX->NEURON_SIZE + 2;
					DENDRITE_EXIST = True;
				}
			}
			if(DENDRITE_EXIST) INDEX++;
		}
	}/*end of grouped dendrites*/
/*
	FREE_MEMORY(SYNAPSES_BUFFER);
*/
}



/*
=============================================================================
Name and dates: REALIZE_MATRIX
---------------
	Creation : 30-04-93
	Last modification : 30-04-93

Functionnal description :
-------------------------
	This function allocates memory for the matrix 
	representation and initialize it.

Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :  None

Explicits outputs :  <return>		int		False = The selected reperesentaion
							can't be realized

Implicits inputs  :  NETWORK		NETWORK_DEF	The definition of the network
		     DISPLAY_VISU	Display *	The display

Implicits outputs :  None

Sides effects : None
---------------

=============================================================================
*/

int REALIZE_MATRIX()
{
	GC GRAPH_CTX;			/*tempory GC*/
	Arg arglist[2];
	int i, j, n;			/*not important stuff*/
	int SIZE_OF_SCREEN;		/*To computes the neuron size*/
	int NB_SYNAPSES;		/*to computes the number of synapses*/
	
	/*allocates memory for the representation*/
	if((NETWORK.MATRIX = (MATRIX_GEOMETRY *)calloc(sizeof(MATRIX_GEOMETRY), 1)) == Null)
		return(False);

	/*Create GC for clusters*/
	NETWORK.MATRIX->CLUSTER_GRAPH_CTX = XCreateGC(DISPLAY_VISU, NETWORK.WINDOW_ID, 0, Null);
	/*Create GC for neurons*/
	NETWORK.MATRIX->NEURON_GRAPH_CTX = XCreateGC(DISPLAY_VISU, NETWORK.WINDOW_ID, 0, Null);
	/*Create GC for dendrites*/
	NETWORK.MATRIX->DENDRITE_GRAPH_CTX = XCreateGC(DISPLAY_VISU, NETWORK.WINDOW_ID, 0, Null);
	/*Create GC for details*/
	NETWORK.MATRIX->DETAILS_GRAPH_CTX = XCreateGC(DISPLAY_VISU, NETWORK.WINDOW_ID, 0, Null);
	/*Create GC for spikes*/
	NETWORK.MATRIX->SPIKE_GRAPH_CTX = XCreateGC(DISPLAY_VISU, NETWORK.WINDOW_ID, 0, Null);
	/*initialize GCs*/
	XSetLineAttributes(DISPLAY_VISU, NETWORK.MATRIX->CLUSTER_GRAPH_CTX, 3, LineSolid, CapButt,
			   JoinMiter);
	XSetForeground(DISPLAY_VISU, NETWORK.MATRIX->CLUSTER_GRAPH_CTX, LUT[C_BLACK]);
	XSetForeground(DISPLAY_VISU, NETWORK.MATRIX->NEURON_GRAPH_CTX, LUT[C_GREEN]);
	XSetForeground(DISPLAY_VISU, NETWORK.MATRIX->DENDRITE_GRAPH_CTX, LUT[C_BLACK]);
	XSetForeground(DISPLAY_VISU, NETWORK.MATRIX->DETAILS_GRAPH_CTX, LUT[C_BLACK]);
	XSetLineAttributes(DISPLAY_VISU, NETWORK.MATRIX->DETAILS_GRAPH_CTX, 2, LineSolid, CapButt,
			   JoinMiter);
	if(COLORMAP_MODE)
		XSetPlaneMask(DISPLAY_VISU, NETWORK.MATRIX->SPIKE_GRAPH_CTX, 254);
	else
		XSetPlaneMask(DISPLAY_VISU, NETWORK.MATRIX->SPIKE_GRAPH_CTX, AllPlanes);

	/*Compute the maximum of synapses on a neuron*/
	NETWORK.MATRIX->MAX_SYNAPSE = 0;
	for(i=0; i<NETWORK.NB_NEURONS; i++)
	{
		NB_SYNAPSES = 0;
		for(j=0; j<NETWORK.NB_NEURONS; j++)
			if(*(NETWORK.CONNEXION_MATRIX + (j * NETWORK.NB_NEURONS) + i) != 0 &&
			   j != i)	/*doesn't count auto connexion*/
				NB_SYNAPSES++;
		if(NB_SYNAPSES > NETWORK.MATRIX->MAX_SYNAPSE)
			NETWORK.MATRIX->MAX_SYNAPSE = NB_SYNAPSES;
	}

	/*Computes neuron and window size*/
	NETWORK.MATRIX->SIZE = NETWORK.NB_NEURONS * DEFAULT_NEURON_SIZE + 1; /*default size*/
	SIZE_OF_SCREEN = (DisplayHeight(DISPLAY_VISU, DefaultScreen(DISPLAY_VISU)) <=
			  DisplayWidth(DISPLAY_VISU, DefaultScreen(DISPLAY_VISU))  ?
			  DisplayHeight(DISPLAY_VISU, DefaultScreen(DISPLAY_VISU)) :
			  DisplayWidth(DISPLAY_VISU, DefaultScreen(DISPLAY_VISU))  );
	SIZE_OF_SCREEN -= 170; /*To count of the window decoration*/

	if(NETWORK.MATRIX->SIZE > SIZE_OF_SCREEN)
		NETWORK.MATRIX->NEURON_SIZE = DEFAULT_NEURON_SIZE;
	else
	{
		/*compute the neuron size*/
		NETWORK.MATRIX->NEURON_SIZE = (int)floor((double)SIZE_OF_SCREEN/
							 (double)NETWORK.NB_NEURONS);
		/*Recompute the size of the drawing*/
		NETWORK.MATRIX->SIZE = NETWORK.NB_NEURONS * NETWORK.MATRIX->NEURON_SIZE + 1;
	}
#if DEBUG
	printf("SIZE_OF_SCREEN : %d\nNETWORK.MATRIX->NEURON_SIZE: %d\nNETWORK.MATRIX->SIZE: %d\n",
	       SIZE_OF_SCREEN, NETWORK.MATRIX->NEURON_SIZE, NETWORK.MATRIX->SIZE);
#endif

/*Actualize the drawing area size*/
	n=0;
	XtSetArg(arglist[n], XmNwidth, NETWORK.MATRIX->SIZE); n++;
	XtSetArg(arglist[n], XmNheight, NETWORK.MATRIX->SIZE); n++;
	XtSetValues(WIDGET_ARRAY[k_Window_Drawing_Area_Main], arglist, n);

	/*Computes the number of axons to a lower neuron*/
	NETWORK.MATRIX->NB_AXONS_TO_LOWER = 0;
	for(i=0; i<NETWORK.NB_NEURONS; i++)
		for(j=i+1; j<NETWORK.NB_NEURONS; j++)
			if(*(NETWORK.CONNEXION_MATRIX + (i * NETWORK.NB_NEURONS) + j) != 0 &&
	       		   *(NETWORK.SYNAPTIC_MATRIX + (i * NETWORK.NB_NEURONS) + j) != 0)
			{
				NETWORK.MATRIX->NB_AXONS_TO_LOWER++;
				break;
			}
	/*Computes the number of axons to a upper neuron*/
	NETWORK.MATRIX->NB_AXONS_TO_UPPER = 0;
	for(i=0; i<NETWORK.NB_NEURONS; i++)
		for(j=0; j<i; j++)
			if(*(NETWORK.CONNEXION_MATRIX + (i * NETWORK.NB_NEURONS) + j) != 0 &&
			   *(NETWORK.SYNAPTIC_MATRIX + (i * NETWORK.NB_NEURONS) + j) != 0)
			{
				NETWORK.MATRIX->NB_AXONS_TO_UPPER++;
				break;
			}
	/*test if we can draw separates dendrites for each connexion or*/
	/*if we must group all dendrites connected on a neuron*/
	if(((int)floor((double)NETWORK.MATRIX->NEURON_SIZE /
		       (double)(NETWORK.MATRIX->MAX_SYNAPSE + 1))) >= 3)
	{
		/*We draw all dendrites*/
		/*Computes the number of dendrites to a lower neuron*/
		NETWORK.MATRIX->NB_DENDRITES_TO_LOWER = 0;
		for(i=0; i<NETWORK.NB_NEURONS; i++)
			for(j=i+1; j<NETWORK.NB_NEURONS; j++)
				if(*(NETWORK.CONNEXION_MATRIX + (i * NETWORK.NB_NEURONS) + j) != 0 &&
			   	   *(NETWORK.SYNAPTIC_MATRIX + (i * NETWORK.NB_NEURONS) + j) != 0)
					NETWORK.MATRIX->NB_DENDRITES_TO_LOWER++;
		/*Computes the number of dendrites to a upper neuron*/
		NETWORK.MATRIX->NB_DENDRITES_TO_UPPER = 0;
		for(i=0; i<NETWORK.NB_NEURONS; i++)
			for(j=0; j<i; j++)
				if(*(NETWORK.CONNEXION_MATRIX + (i * NETWORK.NB_NEURONS) + j) != 0 &&
			   	   *(NETWORK.SYNAPTIC_MATRIX + (i * NETWORK.NB_NEURONS) + j) != 0)
					NETWORK.MATRIX->NB_DENDRITES_TO_UPPER++;
	}
	else
	{
		/*we group dendrites*/
		/*Computes the number of dendrites to a lower neuron*/
		NETWORK.MATRIX->NB_DENDRITES_TO_LOWER = 0;
		for(i=1; i<NETWORK.NB_NEURONS; i++)
			for(j=0; j<i; j++)
				if(*(NETWORK.CONNEXION_MATRIX + (j * NETWORK.NB_NEURONS) + i) != 0 &&
				   *(NETWORK.SYNAPTIC_MATRIX + (j * NETWORK.NB_NEURONS) + i) != 0)
				{
					NETWORK.MATRIX->NB_DENDRITES_TO_LOWER++;
					break;
				}
		/*Computes the number of dendrites to a upper neuron*/
		NETWORK.MATRIX->NB_DENDRITES_TO_UPPER = 0;
		for(i=0; i<NETWORK.NB_NEURONS; i++)
			for(j=i+1; j<NETWORK.NB_NEURONS; j++)
				if(*(NETWORK.CONNEXION_MATRIX + (j * NETWORK.NB_NEURONS) + i) != 0 &&
				   *(NETWORK.SYNAPTIC_MATRIX + (j * NETWORK.NB_NEURONS) + i) != 0)
				{
					NETWORK.MATRIX->NB_DENDRITES_TO_UPPER++;
					break;
				}
	}
	/*Initializes pixmaps to draw spikes*/

	GRAPH_CTX = XCreateGC(DISPLAY_VISU, NETWORK.WINDOW_ID, 0, Null);
	NETWORK.MATRIX->SPIKE_ON = XCreatePixmap(DISPLAY_VISU, NETWORK.WINDOW_ID, 6, 3,
					DefaultDepthOfScreen(DefaultScreenOfDisplay(DISPLAY_VISU)));
	NETWORK.MATRIX->SPIKE_OFF = XCreatePixmap(DISPLAY_VISU, NETWORK.WINDOW_ID, 6, 3,
					DefaultDepthOfScreen(DefaultScreenOfDisplay(DISPLAY_VISU)));
	XSetForeground(DISPLAY_VISU, GRAPH_CTX, LUT[C_RED1]);
	XFillRectangle(DISPLAY_VISU, NETWORK.MATRIX->SPIKE_ON, GRAPH_CTX, 0, 0, 6, 3);
	XSetForeground(DISPLAY_VISU, GRAPH_CTX, LUT[C_WHITE]);
	XFillRectangle(DISPLAY_VISU, NETWORK.MATRIX->SPIKE_OFF, GRAPH_CTX, 0, 0, 6, 3);
	XSetForeground(DISPLAY_VISU, GRAPH_CTX, LUT[C_BLACK]);
	XDrawLine(DISPLAY_VISU, NETWORK.MATRIX->SPIKE_OFF, GRAPH_CTX, 4, 0, 4, 3);
	XFreeGC(DISPLAY_VISU, GRAPH_CTX);

	/*Initialize spike queue*/
	NETWORK.MATRIX->FIRST_SPIKE = Null;
	/*allocates memory for the geometry*/
	NETWORK.MATRIX->CLUSTER_GEOMETRY = (XRectangle *)calloc(sizeof(XRectangle),
								(size_t)NETWORK.NB_CLUSTERS);
	NETWORK.MATRIX->AXON_GEOMETRY_TO_LOWER = (XSegment *)calloc(sizeof(XSegment),
					(size_t)(NETWORK.MATRIX->NB_AXONS_TO_LOWER));
	NETWORK.MATRIX->AXON_GEOMETRY_TO_UPPER = (XSegment *)calloc(sizeof(XSegment),
					(size_t)(NETWORK.MATRIX->NB_AXONS_TO_UPPER));
	NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER = (XSegment *)calloc(sizeof(XSegment),
					(size_t)(NETWORK.MATRIX->NB_DENDRITES_TO_LOWER));
	NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER = (XSegment *)calloc(sizeof(XSegment),
				        (size_t)(NETWORK.MATRIX->NB_DENDRITES_TO_UPPER));
	if(NETWORK.MATRIX->CLUSTER_GEOMETRY == Null ||
	   (NETWORK.MATRIX->AXON_GEOMETRY_TO_LOWER == Null &&
	    NETWORK.MATRIX->NB_AXONS_TO_LOWER != 0) ||
	   (NETWORK.MATRIX->AXON_GEOMETRY_TO_UPPER == Null &&
	    NETWORK.MATRIX->NB_AXONS_TO_UPPER != 0) ||
	   (NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER == Null &&
	    NETWORK.MATRIX->NB_DENDRITES_TO_LOWER != 0) ||
	   (NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER == Null &&
	    NETWORK.MATRIX->NB_DENDRITES_TO_UPPER != 0))
		return(False);
	/*Computes the geometry*/
	COMPUTE_GEOMETRY_MATRIX();
	return(True);
}


/*
=============================================================================
Name and dates: DRAW_DETAILS_MATRIX
---------------
	Creation : 13-05-93
	Last modification : 13-05-93

Functionnal description :
-------------------------
	This function draws details of the representation,
	like synaptic knobs symbols or connexion symbols.
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	X_ORIGIN	int		Geometry of the exposed region
			Y_ORIGIN	int			''
			WIDTH		int			''
			HEIGHT		int			''
			
Explicits outputs :	None

Implicits inputs  :	NETWORK	    	NETWORK_DEF	Definition of the network
			DISPLAY_VISU	Display *	The display
			WIDGET_ARRAY	Widget		The widget array

Implicits outputs :     None

Sides effects :		None
---------------

=============================================================================
*/

void DRAW_DETAILS_MATRIX(int X_ORIGIN, int Y_ORIGIN, int WIDTH, int HEIGHT)
{
	short *SYNAPTIC_KNOB;		/*for the current connexion*/
	short *AXON_LENGTH;
	short *SYNAPSES_BUFFER, *CURRENT_SYNAPSE;		/*to count realized connexion*/
	int i, j;
	int X1, Y1, X2, Y2, OFFSET;

	/*graphic offset*/
	OFFSET = NETWORK.MATRIX->NEURON_SIZE/2;
	SYNAPSES_BUFFER = (short *)calloc(sizeof(short), (size_t)(2*NETWORK.NB_NEURONS));
	if(SYNAPSES_BUFFER == Null)
	{
		NETWORK.FLAG.SHOW_DETAILS = False;
		XmToggleButtonSetState(WIDGET_ARRAY[k_Toggle_Show_Details], False, False);
		DISPLAY_MESSAGE("Not enough memory to draw matrix representation details", INFORMATIONAL);
		return;
	}

/*
-----------------------------------------------------------------------------
Grouped dendrites
-----------------------------------------------------------------------------
*/
	if(NETWORK.MATRIX->STEP == 0)
	{
		
		/*read the synaptic matrix*/
		for(i=0; i<NETWORK.NB_NEURONS; i++)
		{
			for(j=0; j<NETWORK.NB_NEURONS; j++)
			{
				/*synaptic knob of the current connexion*/
				SYNAPTIC_KNOB = NETWORK.SYNAPTIC_MATRIX + (i*NETWORK.NB_NEURONS) + j;
				AXON_LENGTH = NETWORK.CONNEXION_MATRIX + (i*NETWORK.NB_NEURONS) + j;
				if(*SYNAPTIC_KNOB != 0 && *AXON_LENGTH != 0)
				{
					/*connexion to a lower neuron*/
   					if (j > i ) 
					{
						CURRENT_SYNAPSE = SYNAPSES_BUFFER + j;
						/*connexion symbol*/
						X1 = NETWORK.MATRIX->NEURON_SIZE * j + OFFSET - 1;
						Y1 = NETWORK.MATRIX->NEURON_SIZE * i + OFFSET - 1;
						if(IN_WINDOW(X_ORIGIN, Y_ORIGIN, WIDTH, HEIGHT, X1, Y1))
							XFillRectangle(DISPLAY_VISU,  NETWORK.WINDOW_ID,
						             		NETWORK.MATRIX->DETAILS_GRAPH_CTX, X1, Y1, 3, 3);
						switch (*CURRENT_SYNAPSE)
						{
							case NO_CONNEXION :
								/*Synaptic knob symbol*/
								if(*SYNAPTIC_KNOB > 0)
								{
									*CURRENT_SYNAPSE = POSITIVE_CONNEXION;
									X1 = NETWORK.MATRIX->NEURON_SIZE * j + OFFSET - 4;
									Y1 = NETWORK.MATRIX->NEURON_SIZE * j;
									if(IN_WINDOW(X_ORIGIN, Y_ORIGIN, WIDTH, HEIGHT, X1, Y1))
									{
										XDrawLine(DISPLAY_VISU,  NETWORK.WINDOW_ID,
							 			 	NETWORK.MATRIX->DETAILS_GRAPH_CTX, X1 ,Y1,
							  				X1 + 5, Y1 -  4);
										XDrawLine(DISPLAY_VISU,  NETWORK.WINDOW_ID,
							 			 	NETWORK.MATRIX->DETAILS_GRAPH_CTX, X1 + 4, Y1 - 4,
							  				X1 +  8 , Y1);
									}
								}
								else
								{
									*CURRENT_SYNAPSE = NEGATIVE_CONNEXION;
									X1 = NETWORK.MATRIX->NEURON_SIZE * j + OFFSET - 4;
									Y1 = NETWORK.MATRIX->NEURON_SIZE * j - 3;
									if(IN_WINDOW(X_ORIGIN, Y_ORIGIN, WIDTH, HEIGHT, X1, Y1))
										XDrawLine(DISPLAY_VISU,  NETWORK.WINDOW_ID,
							 			 	NETWORK.MATRIX->DETAILS_GRAPH_CTX, X1,Y1,
							  				X1 +  8, Y1);
								}
								break;
							case POSITIVE_CONNEXION :
								if(*SYNAPTIC_KNOB < 0)
								{
									*CURRENT_SYNAPSE = POSNEG_CONNEXION;
									X1 = NETWORK.MATRIX->NEURON_SIZE * j + OFFSET - 4;
									Y1 = NETWORK.MATRIX->NEURON_SIZE * j - 3;
									if(IN_WINDOW(X_ORIGIN, Y_ORIGIN, WIDTH, HEIGHT, X1, Y1))
										XDrawLine(DISPLAY_VISU,  NETWORK.WINDOW_ID,
							 			 	NETWORK.MATRIX->DETAILS_GRAPH_CTX, X1,Y1,
							  				X1 +  8, Y1);
								}
								break;
							case NEGATIVE_CONNEXION :
								if(*SYNAPTIC_KNOB > 0)
								{
									*CURRENT_SYNAPSE = POSNEG_CONNEXION;
									X1 = NETWORK.MATRIX->NEURON_SIZE * j + OFFSET - 4;
									Y1 = NETWORK.MATRIX->NEURON_SIZE * j;
									if(IN_WINDOW(X_ORIGIN, Y_ORIGIN, WIDTH, HEIGHT, X1, Y1))
									{
										XDrawLine(DISPLAY_VISU,  NETWORK.WINDOW_ID,
							 			 	NETWORK.MATRIX->DETAILS_GRAPH_CTX, X1 ,Y1,
							  				X1 + 5, Y1 -  4);
										XDrawLine(DISPLAY_VISU,  NETWORK.WINDOW_ID,
							 			 	NETWORK.MATRIX->DETAILS_GRAPH_CTX, X1 + 4, Y1 - 4,
							  				X1 +  8 , Y1);
									}
								}
								break;
						}/*end of switch *curent_synapse*/
					}/*end of connexion to a lower neuron*/
					/*Connexion to a upper neuron*/
					else if( i > j)
					{
						CURRENT_SYNAPSE = SYNAPSES_BUFFER + NETWORK.NB_NEURONS + j;
						/*connexion symbol*/
						X1 = NETWORK.MATRIX->NEURON_SIZE * j + OFFSET - 1;
						Y1 = NETWORK.MATRIX->NEURON_SIZE * i + OFFSET - 1;
						if(IN_WINDOW(X_ORIGIN, Y_ORIGIN, WIDTH, HEIGHT, X1, Y1))
							XFillRectangle(DISPLAY_VISU,  NETWORK.WINDOW_ID,
						             		NETWORK.MATRIX->DETAILS_GRAPH_CTX, X1, Y1, 3, 3);
						switch (*CURRENT_SYNAPSE)
						{
							case NO_CONNEXION :
								/*Synaptic knob symbol*/
								if(*SYNAPTIC_KNOB >0)
								{
									*CURRENT_SYNAPSE = POSITIVE_CONNEXION;
									X1 = NETWORK.MATRIX->NEURON_SIZE * j + OFFSET - 4;
									Y1 = NETWORK.MATRIX->NEURON_SIZE * (j+1);
									if(IN_WINDOW(X_ORIGIN, Y_ORIGIN, WIDTH, HEIGHT, X1, Y1))
									{
										XDrawLine(DISPLAY_VISU,  NETWORK.WINDOW_ID,
							 			 	NETWORK.MATRIX->DETAILS_GRAPH_CTX, X1,Y1,
							  				X1 + 5, Y1 +  4);
										XDrawLine(DISPLAY_VISU,  NETWORK.WINDOW_ID,
							 			 	NETWORK.MATRIX->DETAILS_GRAPH_CTX, X1 + 4, Y1 + 4,
							  				X1 +  8 , Y1);
									}
								}
								else
								{
									*CURRENT_SYNAPSE = NEGATIVE_CONNEXION;
									X1 = NETWORK.MATRIX->NEURON_SIZE * j + OFFSET - 4;
									Y1 = NETWORK.MATRIX->NEURON_SIZE * (j+1) + 3;
									if(IN_WINDOW(X_ORIGIN, Y_ORIGIN, WIDTH, HEIGHT, X1, Y1))
										XDrawLine(DISPLAY_VISU,  NETWORK.WINDOW_ID,
							 			 	NETWORK.MATRIX->DETAILS_GRAPH_CTX, X1,Y1,
							  				X1 +  9, Y1);
								}
								break;
							case POSITIVE_CONNEXION :
								if(*SYNAPTIC_KNOB < 0)
								{
									*CURRENT_SYNAPSE = POSNEG_CONNEXION;
									X1 = NETWORK.MATRIX->NEURON_SIZE * j + OFFSET - 4;
									Y1 = NETWORK.MATRIX->NEURON_SIZE * (j+1) + 3;
									if(IN_WINDOW(X_ORIGIN, Y_ORIGIN, WIDTH, HEIGHT, X1, Y1))
										XDrawLine(DISPLAY_VISU,  NETWORK.WINDOW_ID,
							 			 	NETWORK.MATRIX->DETAILS_GRAPH_CTX, X1,Y1,
							  				X1 +  9, Y1);
								}
								break;
							case NEGATIVE_CONNEXION :
								if(*SYNAPTIC_KNOB >0)
								{
									*CURRENT_SYNAPSE = POSNEG_CONNEXION;
									X1 = NETWORK.MATRIX->NEURON_SIZE * j + OFFSET - 4;
									Y1 = NETWORK.MATRIX->NEURON_SIZE * (j+1);
									if(IN_WINDOW(X_ORIGIN, Y_ORIGIN, WIDTH, HEIGHT, X1, Y1))
									{
										XDrawLine(DISPLAY_VISU,  NETWORK.WINDOW_ID,
							 			 	NETWORK.MATRIX->DETAILS_GRAPH_CTX, X1,Y1,
							  				X1 + 5, Y1 +  4);
										XDrawLine(DISPLAY_VISU,  NETWORK.WINDOW_ID,
							 			 	NETWORK.MATRIX->DETAILS_GRAPH_CTX, X1 + 4, Y1 + 4,
							  				X1 +  8 , Y1);
									}
								}
						}/*end of switch *current_synapse*/
					}/*end of connexion to a upper neuron*/
				}/*end of if synaptic knob != 0*/
			}/*end of second neuron loop*/
		}/*end of first neuron loop*/
	}/*end of grouped dendrites*/
/*
-----------------------------------------------------------------------------
Separated dendrites
-----------------------------------------------------------------------------
*/
	else
	{
		/*read the synaptic matrix*/
		for(i=0; i<NETWORK.NB_NEURONS; i++)
		{
			for(j=0; j<NETWORK.NB_NEURONS; j++)
			{
				/*synaptic knob of the current connexion*/
				SYNAPTIC_KNOB = NETWORK.SYNAPTIC_MATRIX + (i*NETWORK.NB_NEURONS) + j;
				AXON_LENGTH = NETWORK.CONNEXION_MATRIX + (i*NETWORK.NB_NEURONS) + j;
				if(*SYNAPTIC_KNOB != 0 && *AXON_LENGTH != 0)
				{
					/*connexion to a lower neuron*/
   					if (j > i ) 
					{
						CURRENT_SYNAPSE = SYNAPSES_BUFFER + j;
						(*CURRENT_SYNAPSE)++;
						/*connexion symbol*/
						X1 = NETWORK.MATRIX->NEURON_SIZE * (j + 1) - 
							NETWORK.MATRIX->STEP * (*CURRENT_SYNAPSE) -1;
						Y1 = NETWORK.MATRIX->NEURON_SIZE * i + OFFSET - 1;
						if(IN_WINDOW(X_ORIGIN, Y_ORIGIN, WIDTH, HEIGHT, X1, Y1))
							XFillRectangle(DISPLAY_VISU,  NETWORK.WINDOW_ID,
						             		NETWORK.MATRIX->DETAILS_GRAPH_CTX, X1, Y1, 3, 3);
						/*Synaptic knob symbol*/
						if(*SYNAPTIC_KNOB > 0)
						{
							X1 = NETWORK.MATRIX->NEURON_SIZE * (j + 1) - 
								NETWORK.MATRIX->STEP * (*CURRENT_SYNAPSE) - 4;
							Y1 = NETWORK.MATRIX->NEURON_SIZE * j;
							if(IN_WINDOW(X_ORIGIN, Y_ORIGIN, WIDTH, HEIGHT, X1, Y1))
							{
								XDrawLine(DISPLAY_VISU,  NETWORK.WINDOW_ID,
									 	NETWORK.MATRIX->DETAILS_GRAPH_CTX, X1 ,Y1,
										X1 + 5, Y1 -  4);
								XDrawLine(DISPLAY_VISU,  NETWORK.WINDOW_ID,
							 	 	NETWORK.MATRIX->DETAILS_GRAPH_CTX, X1 + 4, Y1 - 4,
							 			X1 +  8 , Y1);
							}
						}
						else
						{
							X1 = NETWORK.MATRIX->NEURON_SIZE * (j + 1) - 
								NETWORK.MATRIX->STEP * (*CURRENT_SYNAPSE) - 4;
							Y1 = NETWORK.MATRIX->NEURON_SIZE * j - 3;
							if(IN_WINDOW(X_ORIGIN, Y_ORIGIN, WIDTH, HEIGHT, X1, Y1))
								XDrawLine(DISPLAY_VISU,  NETWORK.WINDOW_ID,
									 	NETWORK.MATRIX->DETAILS_GRAPH_CTX, X1,Y1,
						 				X1 +  8, Y1);
						}
					}/*end of connexion to a lower neuron*/
					/*connexion to a upper neuron*/
					else if(i > j)
					{
						CURRENT_SYNAPSE = SYNAPSES_BUFFER + NETWORK.NB_NEURONS + j;
						(*CURRENT_SYNAPSE)++;
						/*connexion symbol*/
						X1 = NETWORK.MATRIX->NEURON_SIZE * (j + 1) - 
							NETWORK.MATRIX->STEP * (*CURRENT_SYNAPSE)  -1;
						Y1 = NETWORK.MATRIX->NEURON_SIZE * i + OFFSET - 1;
						if(IN_WINDOW(X_ORIGIN, Y_ORIGIN, WIDTH, HEIGHT, X1, Y1))
							XFillRectangle(DISPLAY_VISU,  NETWORK.WINDOW_ID,
						             		NETWORK.MATRIX->DETAILS_GRAPH_CTX, X1, Y1, 3, 3);
						/*Synaptic knob symbol*/
						if(*SYNAPTIC_KNOB >0)
						{
							X1 = NETWORK.MATRIX->NEURON_SIZE * (j + 1) - 
								NETWORK.MATRIX->STEP * (*CURRENT_SYNAPSE)  - 4;
							Y1 = NETWORK.MATRIX->NEURON_SIZE * (j+1);
							if(IN_WINDOW(X_ORIGIN, Y_ORIGIN, WIDTH, HEIGHT, X1, Y1))
							{
								XDrawLine(DISPLAY_VISU,  NETWORK.WINDOW_ID,
					 			 	NETWORK.MATRIX->DETAILS_GRAPH_CTX, X1,Y1,
					  				X1 + 5, Y1 +  4);
								XDrawLine(DISPLAY_VISU,  NETWORK.WINDOW_ID,
					 			 	NETWORK.MATRIX->DETAILS_GRAPH_CTX, X1 + 4, Y1 + 4,
					  				X1 +  8 , Y1);
							}
						}
						else
						{
							X1 = NETWORK.MATRIX->NEURON_SIZE * (j + 1) - 
								NETWORK.MATRIX->STEP * (*CURRENT_SYNAPSE)  - 4;
							Y1 = NETWORK.MATRIX->NEURON_SIZE * (j+1) + 3;
							if(IN_WINDOW(X_ORIGIN, Y_ORIGIN, WIDTH, HEIGHT, X1, Y1))
								XDrawLine(DISPLAY_VISU,  NETWORK.WINDOW_ID,
					 			 	NETWORK.MATRIX->DETAILS_GRAPH_CTX, X1,Y1,
					  				X1 +  9, Y1);
						}
					}/*end of connexion to a upper neuron*/
				}/*end of synaptic knob != 0*/
			}/*end of second neuron loop*/
		}/*end of first neuron loop*/
	}/*end of separated dendrites*/
}


/*
=============================================================================
Name and dates: DRAW_MATRIX
---------------
	Creation : 28-04-93
	Last modification : 28-04-93

Functionnal description :
-------------------------
	This function draw recursivly the entire network.
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	None
			
Explicits outputs :	None

Implicits inputs  :	NETWORK	    NETWORK_DEF		Definition of the network
			DISPLAY_VISU	Display *	The display
			WIDGET_ARRAY	Widget		The widget array
			
Implicits outputs :	None

Sides effects :	None
---------------

=============================================================================
*/

void DRAW_MATRIX()
{
	XRectangle *FIRST_RECTANGLE;	/*First rectangle object to draw*/
	XRectangle *RECTANGLE;		/*Rectangle object to draw*/
	XSegment   *FIRST_SEGMENT;	/*first segment to draw*/
        XSegment   *SEGMENT;		/*Segment object to draw*/
	Arg arglist[3];
	int i, j, n;			/*Not important stuff*/
	int LIMIT, LIMIT_X, LIMIT_Y;
	int COUNT;			/*Number of object to draw*/
	int ORIGIN, SIZE;		/*Origin and size to draw neurons*/
	int X_STEP, Y_STEP;		/*Steps in the neuron geometry*/
	EXPOSED_REGION REGION;		/*geometry of the exposed region*/
	

/*get back exposed region*/
	GET_EXPOSED_REGION(&REGION);

/*draw clusters*/
	/*Find the first cluster to draw*/
	for(i=0; i<NETWORK.NB_CLUSTERS; i++)
	{
		FIRST_RECTANGLE = NETWORK.MATRIX->CLUSTER_GEOMETRY + i;
		if(INTERSECT(REGION.X, REGION.Y, REGION.WIDTH, REGION.HEIGHT, FIRST_RECTANGLE->x,
			     FIRST_RECTANGLE->y, FIRST_RECTANGLE->width))
			break;
	}
	/*find the number of cluster to draw*/
	COUNT = 0;
	for(; i<NETWORK.NB_CLUSTERS; i++)
	{
		RECTANGLE = NETWORK.MATRIX->CLUSTER_GEOMETRY + i;
		if(!INTERSECT(REGION.X, REGION.Y, REGION.WIDTH, REGION.HEIGHT, RECTANGLE->x, RECTANGLE->y, RECTANGLE->width))
                        break;
		else
			COUNT++;
	}
	if(COUNT != 0)
	{
		XDrawRectangles(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.MATRIX->CLUSTER_GRAPH_CTX,
				FIRST_RECTANGLE, COUNT);
		XFlush(DISPLAY_VISU);
	}

/*draw neurons*/	
	R_DRAW_NEURONS_VALUES(SAME_STEP);
	
/*Draw axones*/
	/*find the first axon to lower*/
	FIRST_SEGMENT = Null;
	for(i=0; i<NETWORK.MATRIX->NB_AXONS_TO_LOWER; i++)
		if((NETWORK.MATRIX->AXON_GEOMETRY_TO_LOWER + i)->y1 >= REGION.Y)
		{
			FIRST_SEGMENT = NETWORK.MATRIX->AXON_GEOMETRY_TO_LOWER + i;
			break;
		}
	/*find the number of axon to draw*/
	if(FIRST_SEGMENT != Null)
	{
		COUNT = 1;
		LIMIT = REGION.Y + REGION.HEIGHT;
		for(j=i+1; j<NETWORK.MATRIX->NB_AXONS_TO_LOWER; j++)
			if((NETWORK.MATRIX->AXON_GEOMETRY_TO_LOWER + j)->y1 <= LIMIT)
				COUNT++;
			else
				break;
		XDrawSegments(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.MATRIX->DENDRITE_GRAPH_CTX,
		      FIRST_SEGMENT, COUNT);
	}
	/*find the first axon to upper*/
	FIRST_SEGMENT = Null;
	for(i=0; i<NETWORK.MATRIX->NB_AXONS_TO_UPPER; i++)
		if((NETWORK.MATRIX->AXON_GEOMETRY_TO_UPPER + i)->y1 >= REGION.Y)
		{
			FIRST_SEGMENT = NETWORK.MATRIX->AXON_GEOMETRY_TO_UPPER + i;
			break;
		}
	if(FIRST_SEGMENT != Null)
	{
		COUNT = 1;
		LIMIT = REGION.Y + REGION.HEIGHT;
		for(j=i+1; j<NETWORK.MATRIX->NB_AXONS_TO_UPPER; j++)
			if((NETWORK.MATRIX->AXON_GEOMETRY_TO_UPPER + j)->y1 <= LIMIT)
				COUNT++;
			else
				break;
		XDrawSegments(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.MATRIX->DENDRITE_GRAPH_CTX,
		      FIRST_SEGMENT, COUNT);
	}
/*Draw dendrites*/
	/*find the first dendrite to lower*/
	FIRST_SEGMENT = Null;
	for(i=0; i<NETWORK.MATRIX->NB_DENDRITES_TO_LOWER; i++)
		if((NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER + i)->x1 >= REGION.X)
		{
			FIRST_SEGMENT = NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER + i;
			break;
		}
	/*find the number of DENDRITE to draw*/
	if(FIRST_SEGMENT != Null)
	{
		COUNT = 1;
		LIMIT = REGION.X + REGION.WIDTH;
		for(j=i+1; j<NETWORK.MATRIX->NB_DENDRITES_TO_LOWER; j++)
			if((NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER + j)->x1 <= LIMIT)
				COUNT++;
			 else
				break;
		XDrawSegments(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.MATRIX->DENDRITE_GRAPH_CTX,
		      FIRST_SEGMENT, COUNT);
	}

	/*find the first DENDRITE to upper*/
	FIRST_SEGMENT = Null;
	for(i=0; i<NETWORK.MATRIX->NB_DENDRITES_TO_UPPER; i++)
		if((NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER + i)->x1 >= REGION.X)
		{
			FIRST_SEGMENT = NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER + i;
			break;
		}
	if(FIRST_SEGMENT != Null)
	{
		COUNT = 1;
		LIMIT = REGION.X + REGION.WIDTH;
		for(j=i+1; j<NETWORK.MATRIX->NB_DENDRITES_TO_UPPER; j++)
			if((NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER + j)->x1 <= LIMIT)
				COUNT++;
			else
				break;
		XDrawSegments(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.MATRIX->DENDRITE_GRAPH_CTX,
			      FIRST_SEGMENT, COUNT);
	}
/*Draw details for matrix representation*/

	if(NETWORK.FLAG.SHOW_DETAILS)
		DRAW_DETAILS_MATRIX(REGION.X, REGION.Y, REGION.WIDTH, REGION.HEIGHT);

} 


/*
=============================================================================
Name and dates: DRAW_NEURONS_VALUES_MATRIX
---------------
	Creation : 28-04-93
	Last modification : 28-04-93

Functionnal description :
-------------------------
	This function draws neurons with the color corresponding to
	thier value.
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	STATE		int 		FOREWARD, BACKWARD, or SAME_STEP

Explicits outputs :	None

Implicits inputs  :	NETWORK	    	NETWORK_DEF	Definition of the network
			SIMUL	   SIMUL_PARAMETERS	Definition of the simulation
			DISPLAY_VISU	Display *	The display
			WIDGET_ARRAY	Widget		The widget array
			
Implicits outputs :	None

Sides effects :	None
---------------

=============================================================================
*/

void DRAW_NEURONS_VALUES_MATRIX(int STATE)
{
	short *LENGTH, *KNOB;
	unsigned long COLOR_INDEX;
	float VALUE;
	int i, POSITION, X1, Y1, OFFSET;
	EXPOSED_REGION REGION;
	char STRING[15];

#if DEBUG
	printf("DRAW_NEURONS_VALUES_MATRIX: %d (nb de neurones: %d)\n",
	       STATE,NETWORK.NB_NEURONS);
#endif	
	/*get back exposed region*/
	GET_EXPOSED_REGION(&REGION);
	
	if(STATE == GOTO_INDEX)
	{
		/*just create spike*/
		for(i=0; i<NETWORK.NB_NEURONS; i++)
		{
			if((SIMUL.POT + i)->EVENT == SPIKE)
			{
#if DEBUG
				printf("Appel de CREATE_SPIKE si GO_TO_INDEX (neurone %d)\n",i);
#endif
				CREATE_SPIKE(i);
			} /* fin du if */
		} /* fin du for */
	}
	else
	{
		for(i=0; i<NETWORK.NB_NEURONS; i++)
		{
			/*no simulation*/
			if(SIMUL.FLAG.SIMUL_STATE == NO_SIMULATION)
				COLOR_INDEX = C_GREEN;
			/*spike*/
			else if((SIMUL.POT + i)->EVENT == SPIKE)
			{
				if(STATE != SAME_STEP)
				{
#if DEBUG
					printf("Appel de CREATE_SPIKE si pas same step (neurone %d)\n",i);
#endif
					CREATE_SPIKE(i);
				} /* fin du if */
				COLOR_INDEX = C_RED;
			}
			/*greater than color scale top value*/
			else if((SIMUL.POT + i)->PMB >= SIMUL.CS_TOP_VALUE)
				COLOR_INDEX = C_SCALE_COLOR_OFFSET + 99;
			/*lower than color scale bottom value*/
			else if((SIMUL.POT + i)->PMB <= SIMUL.CS_BOTTOM_VALUE)
				COLOR_INDEX = C_SCALE_COLOR_OFFSET;
			/*not special value*/
			else
			{
				VALUE = (float)SIMUL.CS_TOP_VALUE - (SIMUL.POT + i)->PMB;
				COLOR_INDEX = (unsigned long)C_SCALE_COLOR_OFFSET + 99 -
					      (unsigned long)(VALUE/SIMUL.CS_STEP);
			}
			POSITION = NETWORK.MATRIX->NEURON_SIZE * i;
			/*draw neurons only in exposed region*/
			if(INTERSECT(REGION.X, REGION.Y, REGION.WIDTH, REGION.HEIGHT,
				     POSITION, POSITION, NETWORK.MATRIX->NEURON_SIZE))
			{
				XSetForeground(DISPLAY_VISU, NETWORK.MATRIX->NEURON_GRAPH_CTX, LUT[COLOR_INDEX]);
				XFillRectangle(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.MATRIX->NEURON_GRAPH_CTX,
				       POSITION, POSITION, NETWORK.MATRIX->NEURON_SIZE, 
				       NETWORK.MATRIX->NEURON_SIZE);

				/*draw auto-connexion if needed*/
				if(NETWORK.FLAG.SHOW_DETAILS)
				{
					LENGTH = NETWORK.CONNEXION_MATRIX + i * NETWORK.NB_NEURONS + i;
					KNOB = NETWORK.SYNAPTIC_MATRIX + i * NETWORK.NB_NEURONS + i;
					if(*LENGTH != 0 && *KNOB != 0)
					{
						X1 = NETWORK.MATRIX->NEURON_SIZE * i + 
							NETWORK.MATRIX->NEURON_SIZE/2;
						Y1 = NETWORK.MATRIX->NEURON_SIZE * i;
						if(*KNOB > 0)
						{
							XDrawLine(DISPLAY_VISU,  NETWORK.WINDOW_ID,
				 			 	NETWORK.MATRIX->DETAILS_GRAPH_CTX, X1, Y1,
				  				X1, Y1 + NETWORK.MATRIX->NEURON_SIZE);
							XDrawLine(DISPLAY_VISU,  NETWORK.WINDOW_ID,
				 			 	NETWORK.MATRIX->DETAILS_GRAPH_CTX, Y1, X1,
				  				Y1 + NETWORK.MATRIX->NEURON_SIZE, X1);
						}
						else
							XDrawLine(DISPLAY_VISU,  NETWORK.WINDOW_ID,
				 	 			NETWORK.MATRIX->DETAILS_GRAPH_CTX, Y1, X1,
				  				Y1 + NETWORK.MATRIX->NEURON_SIZE, X1);
					}
				}
			}/*end of draw neurons*/
		}/*end of neuron loop*/
	}/*end of not goto index*/
	if(SIMUL.FLAG.SIMUL_STATE != NO_SIMULATION)
		DRAW_SPIKES_MATRIX(STATE, REGION.X, REGION.Y, REGION.WIDTH, REGION.HEIGHT);

}


/*
=============================================================================
Name and dates: CREATE_SPIKE
---------------
	Creation : 10-05-93
	Last modification : 10-05-93

Functionnal description :
-------------------------
	this function creates new spike in the spike queue.
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	NEURON		int		the index in matrixs of the 
							current neuron

Explicits outputs :	None

Implicits inputs  :	NETWORK	 NETWORK_STRUCT		The definition of the network

Implicits outputs :	NETWORK	 NETWORK_STRUCT		The definition of the network

Sides effects :		None
---------------

=============================================================================
*/

void CREATE_SPIKE(int NEURON)
{
	SPIKE_GEOMETRY *NEW_SPIKE;
	short *SYNAPTIC_MATRIX_POS, *CONNEXION_MATRIX_POS;
	XSegment *DENDRITE;
	int i, j, OFFSET;
	int Y1, Y2;
#if DEBUG
	printf("CREATE_SPIKE for neuron %d\n",i);
#endif	
	/*computes the offset*/
	OFFSET = (int)floor((double)NETWORK.MATRIX->NEURON_SIZE/(double)2);
	/*computes the position of the current neuron in matrixs*/
	CONNEXION_MATRIX_POS = NETWORK.CONNEXION_MATRIX + NEURON * NETWORK.NB_NEURONS;
	SYNAPTIC_MATRIX_POS = NETWORK.SYNAPTIC_MATRIX + NEURON * NETWORK.NB_NEURONS;
	/*find the last spike of the queue wich is pointed by the previous spike of the first spike*/
	if(NETWORK.MATRIX->FIRST_SPIKE != Null)
		NEW_SPIKE = NETWORK.MATRIX->FIRST_SPIKE->PREV_SPIKE;
	else
		NEW_SPIKE = Null;
	/*find to which neurons the current neuron is connected*/
	for(i=0; i<NETWORK.NB_NEURONS; i++)
	{
#if DEBUG
		printf("CREATE_SPIKE: connection %d to %d\n",NEURON,i);
#endif
		/*is there a connexion ?*/
		if(*(CONNEXION_MATRIX_POS + i) != 0 && *(SYNAPTIC_MATRIX_POS + i) != 0
		   && i != NEURON) /*No auto-connexion*/
		{
			/*alloc memory for the new spike*/
			if(NEW_SPIKE == Null) /*first spike of the queue*/
			{
				if (NEW_SPIKE = (SPIKE_GEOMETRY *)calloc((size_t)1, 
								sizeof(SPIKE_GEOMETRY)))
				{
					NETWORK.MATRIX->FIRST_SPIKE = NEW_SPIKE;
					NETWORK.MATRIX->FIRST_SPIKE->PREV_SPIKE = NETWORK.MATRIX->FIRST_SPIKE;
					NETWORK.MATRIX->FIRST_SPIKE->NEXT_SPIKE = NETWORK.MATRIX->FIRST_SPIKE;
				} /* fin du if */
				else
				{
					DISPLAY_MESSAGE("No space to allocate the necessary memory, sorry",ERROR);
					continue;
				} /* fin du if */
			}
			else /*last spike of the queue*/
			{
				if ((NEW_SPIKE->NEXT_SPIKE = (SPIKE_GEOMETRY *)calloc((size_t)1, 
								     sizeof(SPIKE_GEOMETRY))) == NULL)
				{
					DISPLAY_MESSAGE("No space to allocate the necessary memory, sorry",ERROR);
					continue;
				} /* fin du if */
				NEW_SPIKE = NEW_SPIKE->NEXT_SPIKE;
			}
			/*realize queue connexions*/
			NEW_SPIKE->NEXT_SPIKE = NETWORK.MATRIX->FIRST_SPIKE;
			NEW_SPIKE->PREV_SPIKE = NETWORK.MATRIX->FIRST_SPIKE->PREV_SPIKE;
			NETWORK.MATRIX->FIRST_SPIKE->PREV_SPIKE = NEW_SPIKE;
			/*initialize new spike*/
			NEW_SPIKE->ITERATION = fabs((double)*(CONNEXION_MATRIX_POS + i));
			NEW_SPIKE->LENGTH = NEW_SPIKE->ITERATION;
			/*computes graphic step*/
			NEW_SPIKE->STEP = (i > NEURON) ?
				/*to a lower neuron*/
				((i - NEURON) * NETWORK.MATRIX->NEURON_SIZE 
					      - OFFSET) / NEW_SPIKE->ITERATION :
				/*to a upper neuron*/
				-(((NEURON-i-1) * NETWORK.MATRIX->NEURON_SIZE 
					      + OFFSET) / NEW_SPIKE->ITERATION);
			/*computes first position of the spike*/
			NEW_SPIKE->POS_Y = NEURON * NETWORK.MATRIX->NEURON_SIZE + OFFSET;
			/*when dendrites are grouped*/
			if(NETWORK.MATRIX->STEP == 0)
				NEW_SPIKE->POS_X = i * NETWORK.MATRIX->NEURON_SIZE + OFFSET - 3;
			/*when dendrites are separated*/
			else
			{
				/*connexion to lowers neurons*/
				if(i>NEURON)
				{
					Y1 = NEURON * NETWORK.MATRIX->NEURON_SIZE + OFFSET;
					Y2 = i * NETWORK.MATRIX->NEURON_SIZE - 3;
					for(j=0; j<NETWORK.MATRIX->NB_DENDRITES_TO_LOWER; j++)
					{
						DENDRITE = NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER + j;
						if(DENDRITE->y1 == Y1 && DENDRITE->y2 == Y2)
						{
							NEW_SPIKE->POS_X = DENDRITE->x1 - 3;
							break;
						}
					}	
				}
				else
				/*connexion to uppers neurons*/
				{
					Y1 = NEURON * NETWORK.MATRIX->NEURON_SIZE + OFFSET;
					Y2 = (i + 1) * NETWORK.MATRIX->NEURON_SIZE + 2;
					for(j=0; j<NETWORK.MATRIX->NB_DENDRITES_TO_UPPER; j++)
					{
						DENDRITE = NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER + j;
						if(DENDRITE->y1 == Y1 && DENDRITE->y2 == Y2)
						{
							NEW_SPIKE->POS_X = DENDRITE->x1 - 3;
							break;
						}
					}	
				}
			}/*end of when dendrites are separated*/
		}/*end of is there a connexion*/
	}/*end of neuron loop*/
/*
	FREE_MEMORY(NEW_SPIKE->NEXT_SPIKE);
	FREE_MEMORY(NEW_SPIKE);
*/
}


/*
=============================================================================
Name and dates: DRAW_SPIKES_MATRIX
---------------
	Creation : 28-04-93
	Last modification : 28-04-93

Functionnal description :
-------------------------
	This function draws spikes on the network if the spike queue is not empty.
	After this, it actualizes spikes structures for the next iteration. Spikes are
	not drawn if we are in GOTO_INDEX state.
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	STATE		int 		FOREWARD, BACKWARD, or SAME_STEP
			X_ORIGIN	int		Geometry of the exposed region
			Y_ORIGIN	int			''
			WIDTH		int			''
			HEIGHT		int			''
						
Explicits outputs :	None

Implicits inputs  :	NETWORK	    NETWORK_DEF		Definition of the network
			DISPLAY_VISU	Display *	The display
			WIDGET_ARRAY	Widget		The widget array
			
Implicits outputs :	None

Sides effects :	None
---------------

=============================================================================
*/

void DRAW_SPIKES_MATRIX(int STATE, int X_ORIGIN, int Y_ORIGIN, int WIDTH, int HEIGHT)
{
	SPIKE_GEOMETRY *CURRENT_SPIKE, *DESTROY_SPIKE;
	int CHANGE_FIRST_SPIKE = False;

	CURRENT_SPIKE = NETWORK.MATRIX->FIRST_SPIKE;

#if DEBUG
	printf("DRAW_SPIKES_MATRIX: %d %d %d %d %d\n",
	        STATE,  X_ORIGIN,  Y_ORIGIN,  WIDTH,  HEIGHT);
#endif
	/*no spike in the queue*/
	if(CURRENT_SPIKE == Null )
		return;
	/*else*/
	do
	{
#if DEBUG
		printf("Iteration courante: %d\n",CURRENT_SPIKE->ITERATION);
#endif
		/*if it is the last iteration for the spike, we destroy the spike*/
		if(CURRENT_SPIKE->ITERATION == 0 && STATE != SAME_STEP)
		{
			/*restore the undreground*/
			if(STATE != GOTO_INDEX && INTERSECT(X_ORIGIN, Y_ORIGIN, WIDTH, HEIGHT,
			   CURRENT_SPIKE->POS_X, CURRENT_SPIKE->POS_Y, 6))
				XCopyArea(DISPLAY_VISU, NETWORK.MATRIX->SPIKE_OFF, 
					  NETWORK.WINDOW_ID, NETWORK.MATRIX->SPIKE_GRAPH_CTX, 
					  0, 0, 6, 3, CURRENT_SPIKE->POS_X, 
					  CURRENT_SPIKE->POS_Y);
			/*spike to destroy*/
			DESTROY_SPIKE = CURRENT_SPIKE;
			/*next spike*/
			CURRENT_SPIKE = CURRENT_SPIKE->NEXT_SPIKE;
			/*updates the chained list*/
			DESTROY_SPIKE->PREV_SPIKE->NEXT_SPIKE = CURRENT_SPIKE;
			CURRENT_SPIKE->PREV_SPIKE = DESTROY_SPIKE->PREV_SPIKE;
			if(DESTROY_SPIKE == NETWORK.MATRIX->FIRST_SPIKE)
			{
				CHANGE_FIRST_SPIKE = True;
				NETWORK.MATRIX->FIRST_SPIKE = CURRENT_SPIKE;
			}
			if(DESTROY_SPIKE == NETWORK.MATRIX->FIRST_SPIKE &&
			   CURRENT_SPIKE == NETWORK.MATRIX->FIRST_SPIKE)
			{
				CHANGE_FIRST_SPIKE = False;
				CURRENT_SPIKE = Null;
				NETWORK.MATRIX->FIRST_SPIKE = Null;
			}
			/*Free memory*/
/*
			FREE_MEMORY(DESTROY_SPIKE);
*/
		}		
		/*if it is not the first iteration for the spike*/
		else if(CURRENT_SPIKE->ITERATION != CURRENT_SPIKE->LENGTH)	
		{
			/*restore the underground*/
			if(STATE != GOTO_INDEX && INTERSECT(X_ORIGIN, Y_ORIGIN, WIDTH, HEIGHT,
			   CURRENT_SPIKE->POS_X, CURRENT_SPIKE->POS_Y, 6))
				XCopyArea(DISPLAY_VISU, NETWORK.MATRIX->SPIKE_OFF, NETWORK.WINDOW_ID,
			  		NETWORK.MATRIX->SPIKE_GRAPH_CTX, 0, 0, 6, 3, CURRENT_SPIKE->POS_X,
			  		CURRENT_SPIKE->POS_Y);
			if(STATE != SAME_STEP)
				/*computes new position*/
				CURRENT_SPIKE->POS_Y = CURRENT_SPIKE->POS_Y + CURRENT_SPIKE->STEP;
			/*draw the spike at the new position*/
			if(STATE != GOTO_INDEX && INTERSECT(X_ORIGIN, Y_ORIGIN, WIDTH, HEIGHT,
			   CURRENT_SPIKE->POS_X, CURRENT_SPIKE->POS_Y, 6))
				XCopyArea(DISPLAY_VISU, NETWORK.MATRIX->SPIKE_ON, NETWORK.WINDOW_ID,
			  		NETWORK.MATRIX->SPIKE_GRAPH_CTX, 0, 0, 6, 3, CURRENT_SPIKE->POS_X,
			  		CURRENT_SPIKE->POS_Y);
			if(STATE != SAME_STEP)
				/* next iteration*/
				CURRENT_SPIKE->ITERATION--;
			/*next spike*/
			CURRENT_SPIKE = CURRENT_SPIKE->NEXT_SPIKE;
			CHANGE_FIRST_SPIKE = False;
		}
		/*first iteration of the spike*/
		else
		{
			/*draw the spike at the new position*/
			if(STATE != GOTO_INDEX && INTERSECT(X_ORIGIN, Y_ORIGIN, WIDTH, HEIGHT,
			   CURRENT_SPIKE->POS_X, CURRENT_SPIKE->POS_Y, 6))
				XCopyArea(DISPLAY_VISU, NETWORK.MATRIX->SPIKE_ON, NETWORK.WINDOW_ID,
			  		NETWORK.MATRIX->SPIKE_GRAPH_CTX, 0, 0, 6, 3, CURRENT_SPIKE->POS_X,
			  		CURRENT_SPIKE->POS_Y);
			if(STATE != SAME_STEP)
				/* next iteration*/
				CURRENT_SPIKE->ITERATION--;
			/*next spike*/
			CURRENT_SPIKE = CURRENT_SPIKE->NEXT_SPIKE;
			CHANGE_FIRST_SPIKE = False;
		}
	}while(CURRENT_SPIKE != NETWORK.MATRIX->FIRST_SPIKE || CHANGE_FIRST_SPIKE);
}


/*
=============================================================================
Name and dates: PRINT_DETAILS_MATRIX
---------------
	Creation : 13-05-93
	Last modification : 13-05-93

Functionnal description :
-------------------------
	This function prints details of the representation,
	like synaptic knobs symbols or connexion symbols.
	This function asumes that we are in user units (NEURON = 1 x 1)
	and that the origin is the lower left corner of the drawing.
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	SCALE		float		The scale for printing
			
Explicits outputs :	None

Implicits inputs  :	NETWORK	    	NETWORK_DEF	Definition of the network
			DISPLAY_VISU	Display *	The display
			WIDGET_ARRAY	Widget		The widget array

Implicits outputs :     None

Sides effects :		None
---------------

=============================================================================
*/

void PRINT_DETAILS_MATRIX(float SCALE)
{
	short *SYNAPTIC_KNOB;		/*for the current connexion*/
	short *AXON_LENGTH;
	short *SYNAPSES_BUFFER, *CURRENT_SYNAPSE;	/*to count realized connexion*/
	float OFFSET = 0.5;
	float X_tab[4], Y_tab[4];
	float STEP, PRINT_STEP;
	int i, j;

	ps_set_rgb_color(0., 0., 0.);
	ps_line_width(2.);
	STEP = 0.1/SCALE;
	PRINT_STEP = 1./(NETWORK.MATRIX->MAX_SYNAPSE + 1);
/*
-----------------------------------------------------------------------------
Grouped dendrites
-----------------------------------------------------------------------------
*/
	if(NETWORK.MATRIX->STEP == 0)
	{
		/*read the synaptic matrix*/
		for(i=0; i<NETWORK.NB_NEURONS; i++)
		{
			for(j=0; j<NETWORK.NB_NEURONS; j++)
			{
				/*synaptic knob of the current connexion*/
				SYNAPTIC_KNOB = NETWORK.SYNAPTIC_MATRIX + (i*NETWORK.NB_NEURONS) + j;
				AXON_LENGTH = NETWORK.CONNEXION_MATRIX + (i*NETWORK.NB_NEURONS) + j;
				if(*SYNAPTIC_KNOB != 0 && *AXON_LENGTH != 0)
				{
					/*connexion symbol*/
					X_tab[0] = (float)j + OFFSET ;
					Y_tab[0] = NETWORK.NB_NEURONS - (float)i - OFFSET;
					ps_movea(X_tab[0], Y_tab[0]);
					ps_point(2, 0.05);
					/*synaptic knobs*/
					/*connexion to a lower neuron*/
   					if (j > i ) 
					{
						if(*SYNAPTIC_KNOB > 0)
						{
							X_tab[0] = (float)j + OFFSET - STEP;
							Y_tab[0] = NETWORK.NB_NEURONS - (float)j;
							X_tab[1] = (float)j + OFFSET;
							Y_tab[1] = NETWORK.NB_NEURONS - (float)j + STEP;
							X_tab[2] = (float)j + OFFSET + STEP;
							Y_tab[2] = Y_tab[0];
							ps_polyline(X_tab, Y_tab, 3, False, 0, 0.);
						}
						else
						{
							X_tab[0] = (float)j + OFFSET - STEP;
							Y_tab[0] = NETWORK.NB_NEURONS - (float)j + STEP;
							X_tab[1] = (float)j + OFFSET + STEP;
							Y_tab[1] = Y_tab[0];
							ps_polyline(X_tab, Y_tab, 2, False, 0, 0.);
						}
					}/*end of connexion to a lower neuron*/
					/*Connexion to a upper neuron*/
					else if( i > j)
					{
						if(*SYNAPTIC_KNOB >0)
						{
							X_tab[0] = (float)j + OFFSET - STEP;
							Y_tab[0] = NETWORK.NB_NEURONS - (float)(j+1);
							X_tab[1] = (float)j + OFFSET;
							Y_tab[1] = NETWORK.NB_NEURONS - (float)(j+1) - STEP;
							X_tab[2] = (float)j + OFFSET + STEP;
							Y_tab[2] = Y_tab[0];
							ps_polyline(X_tab, Y_tab, 3, False, 0, 0.);
						}
						else
						{
							X_tab[0] = (float)j + OFFSET - STEP;
							Y_tab[0] = NETWORK.NB_NEURONS - (float)(j+1) - STEP;
							X_tab[1] = (float)j + OFFSET + STEP;
							Y_tab[1] = Y_tab[0];
							ps_polyline(X_tab, Y_tab, 2, False, 0, 0.);
						}
					}/*end of connexion to a upper neuron*/
				}/*end of if synaptic knob != 0*/
			}/*end of second neuron loop*/
		}/*end of first neuron loop*/
	}/*end of grouped dendrites*/
/*
-----------------------------------------------------------------------------
Separated dendrites
-----------------------------------------------------------------------------
*/
	else
	{
		SYNAPSES_BUFFER = (short *)calloc(sizeof(short), (size_t)(2*NETWORK.NB_NEURONS));
		/*read the synaptic matrix*/
		for(i=0; i<NETWORK.NB_NEURONS; i++)
		{
			for(j=0; j<NETWORK.NB_NEURONS; j++)
			{
				/*synaptic knob of the current connexion*/
				SYNAPTIC_KNOB = NETWORK.SYNAPTIC_MATRIX + (i*NETWORK.NB_NEURONS) + j;
				AXON_LENGTH = NETWORK.CONNEXION_MATRIX + (i*NETWORK.NB_NEURONS) + j;
				if(*SYNAPTIC_KNOB != 0 && *AXON_LENGTH != 0)
				{
					/*connexion to a lower neuron*/
   					if (j > i ) 
					{
						CURRENT_SYNAPSE = SYNAPSES_BUFFER + j;
						(*CURRENT_SYNAPSE)++;
						/*connexion symbol*/
						X_tab[0] = (float)(j+1) - PRINT_STEP * (float)(*CURRENT_SYNAPSE);
						Y_tab[0] = NETWORK.NB_NEURONS - (float)i - OFFSET;
						ps_movea(X_tab[0], Y_tab[0]);
						ps_point(2, 0.05);
						/*Synaptic knob symbol*/
						if(*SYNAPTIC_KNOB > 0)
						{
							X_tab[0] = (float)(j+1) - PRINT_STEP * (float)(*CURRENT_SYNAPSE) - STEP;
							Y_tab[0] = NETWORK.NB_NEURONS - (float)j;
							X_tab[1] = (float)(j+1) - PRINT_STEP * (float)(*CURRENT_SYNAPSE);
							Y_tab[1] = NETWORK.NB_NEURONS - (float)j + STEP;
							X_tab[2] = (float)(j+1) - PRINT_STEP * (float)(*CURRENT_SYNAPSE) + STEP;
							Y_tab[2] = Y_tab[0];
							ps_polyline(X_tab, Y_tab, 3, False, 0, 0.);
						}
						else
						{
							X_tab[0] = (float)(j+1) - PRINT_STEP * (float)(*CURRENT_SYNAPSE) - STEP;
							Y_tab[0] = NETWORK.NB_NEURONS - (float)j + STEP;
							X_tab[1] = (float)(j+1) - PRINT_STEP * (float)(*CURRENT_SYNAPSE) + STEP;
							Y_tab[1] = Y_tab[0];
							ps_polyline(X_tab, Y_tab, 2, False, 0, 0.);
						}
					}/*end of connexion to a lower neuron*/
					/*connexion to a upper neuron*/
					else if(i > j)
					{
						CURRENT_SYNAPSE = SYNAPSES_BUFFER + NETWORK.NB_NEURONS + j;
						(*CURRENT_SYNAPSE)++;
						/*connexion symbol*/
						X_tab[0] = (float)(j+1) - PRINT_STEP * (float)(*CURRENT_SYNAPSE);
						Y_tab[0] = NETWORK.NB_NEURONS - (float)i - OFFSET;
						ps_movea(X_tab[0], Y_tab[0]);
						ps_point(2, 0.05);
						/*Synaptic knob symbol*/
						if(*SYNAPTIC_KNOB >0)
						{
							X_tab[0] = (float)(j+1) - PRINT_STEP * (float)(*CURRENT_SYNAPSE) - STEP;
							Y_tab[0] = NETWORK.NB_NEURONS - (float)(j+1);
							X_tab[1] = (float)(j+1) - PRINT_STEP * (float)(*CURRENT_SYNAPSE) ;
							Y_tab[1] = NETWORK.NB_NEURONS - (float)(j+1) - STEP;
							X_tab[2] = (float)(j+1) - PRINT_STEP * (float)(*CURRENT_SYNAPSE)  + STEP;
							Y_tab[2] = Y_tab[0];
							ps_polyline(X_tab, Y_tab, 3, False, 0, 0.);
						}
						else
						{
							X_tab[0] = (float)(j+1) - PRINT_STEP * (float)(*CURRENT_SYNAPSE)  - STEP;
							Y_tab[0] = NETWORK.NB_NEURONS - (float)(j+1) - STEP;
							X_tab[1] = (float)(j+1) - PRINT_STEP * (float)(*CURRENT_SYNAPSE)  + STEP;
							Y_tab[1] = Y_tab[0];
							ps_polyline(X_tab, Y_tab, 2, False, 0, 0.);
						}
					}/*end of connexion to a upper neuron*/
				}/*end of synaptic knob != 0*/
			}/*end of second neuron loop*/
		}/*end of first neuron loop*/
	}/*end of separated dendrites*/
}



/*
=============================================================================
Name and dates: PRINT_MATRIX
---------------
	Creation : 04-06-93
	Last modification : 04-06-93

Functionnal description :
-------------------------
	This function print the matrix representation in its actual
	state if there is a simulation running. This function use the
	PLOT_PS library.
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	DRAW_ALL	int		flag : True = all the network 
							       is printed else only the 
							       visible part is printed.
			PAPER_WIDTH	float		Width of the sheet
Explicits outputs :     None

Implicits inputs  :	NETWORK		NETWORK_DEF	The definition of the network
			CMAP_ID		Colormap	The colormap
			DISPLAY_VISU	Display *	The display

Implicits outputs :	None

Sides effects :		None
---------------

=============================================================================
*/

void PRINT_MATRIX(int DRAW_ALL, float PAPER_WIDTH)
{
	XSegment *SEGMENT;
	short *KNOB, *LENGTH;
	SPIKE_GEOMETRY *CURRENT_SPIKE;
	float SCALE, VALUE;
	float X_tab[4], Y_tab[4];
	int i, j;
	EXPOSED_REGION REGION;
	XColor COLOR;
	char ACTION;
	char STRING[80];
	
	/*computes scale for drawing. A neuron is a square of one user unit side*/
	/*one user unit equal SCALE centimeters*/
	if(DRAW_ALL)
	{
		/*computes scale*/
		SCALE = (PAPER_WIDTH - 2)/(float)NETWORK.NB_NEURONS;
		/*set origin. we take count of margins for bubble jet printer*/
		ps_origin(0., 1., 0., 0.);
	}
	else
	{
		/*get back visible part geometry*/
		GET_EXPOSED_REGION(&REGION);
		/*compute scale*/
		SCALE = (PAPER_WIDTH - 2)/((float)REGION.WIDTH/(float)NETWORK.MATRIX->NEURON_SIZE);
		/*set origin. we take count of margins for bubble jet printer*/
		ps_origin(SCALE * (-(float)REGION.X/(float)NETWORK.MATRIX->NEURON_SIZE), 
			  1. - SCALE * (float)(NETWORK.MATRIX->SIZE - REGION.Y - REGION.HEIGHT)/
			  (float)NETWORK.MATRIX->NEURON_SIZE, 0., 0.);
	}

	if(SCALE < 0.008)
	{
		/*set origin. we take count of margins for bubble jet printer*/
		ps_origin(0., 1., 0., 0.);
		/*the drawing is not possible*/
		ps_writeah(0., 5., "XNBC : Visualization Printing Error", 0.7);
		ps_writeah(0., 4., NETWORK.NAME, 0.5);
		ps_writeah(0., 3., "is too large to be printed", 0.5);
	}
	/*drawing is possible*/
	else
	{
		/*set scale*/
		ps_echel(SCALE, SCALE, 0., 0., 1., 0.);
		/*use user units*/
		ps_user();

		/*draw clusters*/
		ps_movea(0., (float)NETWORK.NB_NEURONS);
		ps_line_width(3.);
		for(i=0; i<NETWORK.NB_CLUSTERS; i++)
		{
			ps_mover(0., -(float)*(NETWORK.NB_NEURONS_PER_CLUSTER + i));
			ps_frame((float)*(NETWORK.NB_NEURONS_PER_CLUSTER + i),
				 (float)*(NETWORK.NB_NEURONS_PER_CLUSTER + i));
			ps_mover((float)*(NETWORK.NB_NEURONS_PER_CLUSTER + i), 0.);
		}
		
		/*draw neurons*/
		for(i=0; i<NETWORK.NB_NEURONS; i++)
		{
			/*computes position*/
			X_tab[0] = i;    Y_tab[0] = NETWORK.NB_NEURONS - i;
			X_tab[1] = i+1;  Y_tab[1] = Y_tab[0];
			X_tab[2] = X_tab[1]; Y_tab[2] = NETWORK.NB_NEURONS - i - 1;
			X_tab[3] = X_tab[0]; Y_tab[3] = Y_tab[2];

			/*no simulation*/
			if(SIMUL.FLAG.SIMUL_STATE == NO_SIMULATION)
				COLOR.pixel = LUT[C_GREEN];
			/*spike*/
			else if((SIMUL.POT + i)->EVENT == SPIKE)
				COLOR.pixel = LUT[C_RED];
			/*greater than color scale top value*/
			else if((SIMUL.POT + i)->PMB >= SIMUL.CS_TOP_VALUE)
				COLOR.pixel = LUT[C_SCALE_COLOR_OFFSET + 99];
			/*lower than color scale bottom value*/
			else if((SIMUL.POT + i)->PMB <= SIMUL.CS_BOTTOM_VALUE)
				COLOR.pixel = LUT[C_SCALE_COLOR_OFFSET];
			/*not special value*/
			else
			{
				VALUE = (float)SIMUL.CS_TOP_VALUE - (SIMUL.POT + i)->PMB;
				COLOR.pixel = LUT[(int)C_SCALE_COLOR_OFFSET + 99 -
					      (int)(VALUE/SIMUL.CS_STEP)];
			}
			/*sets color*/
			XQueryColor(DISPLAY_VISU, CMAP_ID, &COLOR);
			ps_set_rgb_color((float)COLOR.red/65535, (float)COLOR.green/65535, 
					 (float)COLOR.blue/65535);
			/*draw neuron*/
			ps_filled_area(X_tab, Y_tab, 4);
		}/*end of neuron loop*/
		
		/*draw axons*/
		ps_set_rgb_color(0., 0., 0.);
		ps_line_width(1.);
		/*draw axons to lower*/
		for(i=0; i<NETWORK.MATRIX->NB_AXONS_TO_LOWER; i++)
		{
			SEGMENT = NETWORK.MATRIX->AXON_GEOMETRY_TO_LOWER + i;
			X_tab[0] = (float)SEGMENT->x1/(float)NETWORK.MATRIX->NEURON_SIZE; 
			Y_tab[0] = (float)NETWORK.NB_NEURONS - (float)SEGMENT->y1/(float)NETWORK.MATRIX->NEURON_SIZE;
			X_tab[1] = (float)SEGMENT->x2/(float)NETWORK.MATRIX->NEURON_SIZE; 
			Y_tab[1] = (float)NETWORK.NB_NEURONS - (float)SEGMENT->y2/(float)NETWORK.MATRIX->NEURON_SIZE;
			ps_polyline(X_tab, Y_tab, 2, False, 0, 0.);
		}
		/*draw axons to upper*/
		for(i=0; i<NETWORK.MATRIX->NB_AXONS_TO_UPPER; i++)
		{
			SEGMENT = NETWORK.MATRIX->AXON_GEOMETRY_TO_UPPER + i;
			X_tab[0] = (float)SEGMENT->x1/(float)NETWORK.MATRIX->NEURON_SIZE; 
			Y_tab[0] = (float)NETWORK.NB_NEURONS - (float)SEGMENT->y1/(float)NETWORK.MATRIX->NEURON_SIZE;
			X_tab[1] = (float)SEGMENT->x2/(float)NETWORK.MATRIX->NEURON_SIZE; 
			Y_tab[1] = (float)NETWORK.NB_NEURONS - (float)SEGMENT->y2/(float)NETWORK.MATRIX->NEURON_SIZE;
			ps_polyline(X_tab, Y_tab, 2, False, 0, 0.);
		}
		/*draw dendrites to lower*/
		for(i=0; i<NETWORK.MATRIX->NB_DENDRITES_TO_LOWER; i++)
		{
			SEGMENT = NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER + i;
			X_tab[0] = (float)SEGMENT->x1/(float)NETWORK.MATRIX->NEURON_SIZE; 
			Y_tab[0] = (float)NETWORK.NB_NEURONS - (float)SEGMENT->y1/(float)NETWORK.MATRIX->NEURON_SIZE;
			X_tab[1] = (float)SEGMENT->x2/(float)NETWORK.MATRIX->NEURON_SIZE; 
			Y_tab[1] = (float)NETWORK.NB_NEURONS - (float)SEGMENT->y2/(float)NETWORK.MATRIX->NEURON_SIZE;
			ps_polyline(X_tab, Y_tab, 2, False, 0, 0.);
		}
		/*draw dendrites to upper*/
		for(i=0; i<NETWORK.MATRIX->NB_DENDRITES_TO_UPPER; i++)
		{
			SEGMENT = NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER + i;
			X_tab[0] = (float)SEGMENT->x1/(float)NETWORK.MATRIX->NEURON_SIZE; 
			Y_tab[0] = (float)NETWORK.NB_NEURONS - (float)SEGMENT->y1/(float)NETWORK.MATRIX->NEURON_SIZE;
			X_tab[1] = (float)SEGMENT->x2/(float)NETWORK.MATRIX->NEURON_SIZE; 
			Y_tab[1] = (float)NETWORK.NB_NEURONS - (float)SEGMENT->y2/(float)NETWORK.MATRIX->NEURON_SIZE;
			ps_polyline(X_tab, Y_tab, 2, False, 0, 0.);
		}

		/*draw details*/
		ps_line_width(2.);
		if(NETWORK.FLAG.SHOW_DETAILS)
		{
			ps_movea(0., 0.);
			PRINT_DETAILS_MATRIX(SCALE);

			/*draw auto-connexion*/
			for(i=0; i<NETWORK.NB_NEURONS; i++)
			{
				LENGTH = NETWORK.CONNEXION_MATRIX + i * NETWORK.NB_NEURONS + i;
				KNOB = NETWORK.SYNAPTIC_MATRIX + i * NETWORK.NB_NEURONS + i;
				if(*LENGTH != 0 && *KNOB != 0)
				{
					if(*KNOB > 0)
					{
						X_tab[0] = (float)(i+0.5); 
						Y_tab[0] = NETWORK.NB_NEURONS - (float)i;
						X_tab[1] = X_tab[0]; 	       
						Y_tab[1] = NETWORK.NB_NEURONS - (float)(i+1);
						ps_polyline(X_tab, Y_tab, 2, False, 0., 0.);
						X_tab[0] = (float)i;       
						Y_tab[0] = NETWORK.NB_NEURONS - (float)(i+0.5);
						X_tab[1] = (float)(i+1);   
						Y_tab[1] = Y_tab[0];
						ps_polyline(X_tab, Y_tab, 2, False, 0., 0.);
					}
					else
					{
						X_tab[0] = (float)i;       
						Y_tab[0] = NETWORK.NB_NEURONS - (float)(i+0.5);
						X_tab[1] = (float)(i+1);   
						Y_tab[1] = Y_tab[0];
						ps_polyline(X_tab, Y_tab, 2, False, 0., 0.);
					}
				}
			}/*end of draw auto-connexion*/
		}/*end of draw details*/

		/*draw spikes*/
		if((CURRENT_SPIKE = NETWORK.MATRIX->FIRST_SPIKE) != Null)
		{
			ps_set_rgb_color(1., 0., 0.);
			ps_line_width(2.);
			do
			{
				X_tab[0] = (float)CURRENT_SPIKE->POS_X/(float)NETWORK.MATRIX->NEURON_SIZE;
				X_tab[0] -=  (0.05 / SCALE);
				Y_tab[0] = (float)NETWORK.NB_NEURONS - (float)CURRENT_SPIKE->POS_Y/(float)NETWORK.MATRIX->NEURON_SIZE;
				X_tab[1] = X_tab[0] + 0.2 / SCALE;
				Y_tab[1] = Y_tab[0];
				ps_polyline(X_tab, Y_tab, 2, False, 0, 0.);
			}while((CURRENT_SPIKE = CURRENT_SPIKE->NEXT_SPIKE) != NETWORK.MATRIX->FIRST_SPIKE);
		}/*end of drawing spikes*/

		/*draw bouding box if not DRAW_ALL*/
		ps_cms();
		if(!DRAW_ALL)
		{
			ps_set_rgb_color(1., 1., 1.);
			ps_origin(SCALE * (float)REGION.X/(float)NETWORK.MATRIX->NEURON_SIZE, 
			  SCALE * (float)(NETWORK.MATRIX->SIZE - REGION.Y - REGION.HEIGHT)/
			  (float)NETWORK.MATRIX->NEURON_SIZE, 0., 0.);

			/*lower rectangle*/
			X_tab[0] = -1.;
			Y_tab[0] = 0.;
			X_tab[1] = PAPER_WIDTH;
			Y_tab[1] = Y_tab[0];
			X_tab[2] = X_tab[1];
			Y_tab[2] = -5.;
			X_tab[3] = X_tab[0];
			Y_tab[3] = Y_tab[2];
			ps_filled_area(X_tab, Y_tab, 4);
			/*upper rectangle*/
			X_tab[0] = -1.;
			Y_tab[0] = SCALE * (float)REGION.HEIGHT/(float)NETWORK.MATRIX->NEURON_SIZE;
			X_tab[1] = PAPER_WIDTH;
			Y_tab[1] = Y_tab[0];
			X_tab[2] = X_tab[1];
			Y_tab[2] = 50.;
			X_tab[3] = X_tab[0];
			Y_tab[3] = Y_tab[2];
			ps_filled_area(X_tab, Y_tab, 4);
			/*left rectangle*/
			X_tab[0] = 0.;
			Y_tab[0] = 0.;
			X_tab[1] = 0.;
			Y_tab[1] = 50.;
			X_tab[2] = -2.;
			Y_tab[2] = 50.;
			X_tab[3] = -2.;
			Y_tab[3] = 0.;
			ps_filled_area(X_tab, Y_tab, 4);
			/*right rectangle*/
			X_tab[0] = PAPER_WIDTH - 2.;
			Y_tab[0] = 0.;
			X_tab[1] = X_tab[0];
			Y_tab[1] = 50.;
			X_tab[2] = PAPER_WIDTH +2.;
			Y_tab[2] = 50.;
			X_tab[3] = X_tab[2];
			Y_tab[3] = 0.;
			ps_filled_area(X_tab, Y_tab, 4);
		}			
		/*draw comments*/
		if(!DRAW_ALL)
			ps_origin(0., SCALE * (float)REGION.HEIGHT/(float)NETWORK.MATRIX->NEURON_SIZE + 0.1, 0., 0.);
		else
			ps_origin(0., SCALE * (float)NETWORK.NB_NEURONS +0.1, 0., 0.);
		/*draw color scale*/
		for(i=0; i<100; i++)
		{
			COLOR.pixel = LUT[i + C_SCALE_COLOR_OFFSET];
			XQueryColor(DISPLAY_VISU, CMAP_ID, &COLOR);
			ps_set_rgb_color((float)COLOR.red/65535, (float)COLOR.green/65535, 
					 (float)COLOR.blue/65535);
			X_tab[0] = (float)i*0.1;
			Y_tab[0] = 1.;
			X_tab[1] = X_tab[0]+0.1;
			Y_tab[1] = Y_tab[0];
			X_tab[2] = X_tab[1];
			Y_tab[2] = Y_tab[0] - 0.5;
			X_tab[3] = X_tab[0];
			Y_tab[3] = Y_tab[2];
			ps_filled_area(X_tab, Y_tab, 4);
		}
		ps_set_rgb_color(0., 0., 0.);
		ps_writeah(0., 1.1, "Color Scale in mV", 0.3);
		/*draw spike color*/
		ps_writeah(10.5, 1.1, "Spike", 0.3);
		X_tab[0] = 10.5;
		Y_tab[0] = 1.;
		X_tab[1] = 11.;
		Y_tab[1] = Y_tab[0];
		X_tab[2] = X_tab[1];
		Y_tab[2] = 0.5;
		X_tab[3] = X_tab[0];
		Y_tab[3] = Y_tab[2];
		ps_set_rgb_color(1., 0., 0.);
		ps_filled_area(X_tab, Y_tab, 4);
		/*draw values*/
		ps_set_rgb_color(0., 0., 0.);
		ps_writeintah(0., 0.1, (int)SIMUL.CS_BOTTOM_VALUE, 0.3);
		ps_writeintah(9.8, 0.1, SIMUL.CS_TOP_VALUE, 0.3);
		sprintf(STRING, "Iteration : %d    N.G.A.: %d%%", SIMUL.ITERATION, SIMUL.SPIKE_COUNT);
		ps_writeah(0., 1.6, STRING, 0.3);
		sprintf(STRING, "NETWORK : %s", NETWORK.NAME);
		ps_writeah(0., 2., STRING, 0.3);
		
	}/*end of drawing*/

}


/*
=============================================================================
Name and dates: FREE_MATRIX
---------------
	Creation : 04-06-93
	Last modification : 04-06-93

Functionnal description :
-------------------------
	This function free the matrix representation.

Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	None

Explicits outputs :     None

Implicits inputs  :	NETWORK		NETWORK_DEF	The definition of the network

Implicits outputs :	NETWORK		NETWORK_DEF	The definition of the network

Sides effects :		None
---------------

=============================================================================
*/

/*this macro free Pixmap if the given pointer isn't null*/
#define FREE_PIXMAP(PT) if((PT) != Null) \
			XFreePixmap(DISPLAY_VISU, (PT))

void FREE_MATRIX(int ACTION)
{
	SPIKE_GEOMETRY *DESTROY_SPIKE, *NEXT_SPIKE;

	/*free the entire representation*/
	if(NETWORK.MATRIX != Null && ACTION == ALL)
	{
		FREE_MEMORY(NETWORK.MATRIX->CLUSTER_GEOMETRY);
		FREE_MEMORY(NETWORK.MATRIX->AXON_GEOMETRY_TO_LOWER);
		FREE_MEMORY(NETWORK.MATRIX->AXON_GEOMETRY_TO_UPPER);
		FREE_MEMORY(NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_LOWER);
		FREE_MEMORY(NETWORK.MATRIX->DENDRITE_GEOMETRY_TO_UPPER);

		FREE_PIXMAP(NETWORK.MATRIX->SPIKE_ON);
		FREE_PIXMAP(NETWORK.MATRIX->SPIKE_OFF);

		FREE_GC(NETWORK.MATRIX->SPIKE_GRAPH_CTX);
		FREE_GC(NETWORK.MATRIX->CLUSTER_GRAPH_CTX);
		FREE_GC(NETWORK.MATRIX->NEURON_GRAPH_CTX);
		FREE_GC(NETWORK.MATRIX->DENDRITE_GRAPH_CTX);
		FREE_GC(NETWORK.MATRIX->DETAILS_GRAPH_CTX);

		/*free spike queue*/
		if((DESTROY_SPIKE = NETWORK.MATRIX->FIRST_SPIKE) != Null)
		{
			while(DESTROY_SPIKE->NEXT_SPIKE != NETWORK.MATRIX->FIRST_SPIKE)
			{
				NEXT_SPIKE = DESTROY_SPIKE->NEXT_SPIKE;
				free(DESTROY_SPIKE);
				DESTROY_SPIKE = NEXT_SPIKE;
			}
			free(DESTROY_SPIKE);
			NETWORK.MATRIX->FIRST_SPIKE = Null;
		}

		FREE_MEMORY(NETWORK.MATRIX);
		NETWORK.MATRIX = Null;
	}

	/*free only specific allocation*/
	if(NETWORK.MATRIX != Null && ACTION == SPECIAL)
		/*free spike queue*/
		if((DESTROY_SPIKE = NETWORK.MATRIX->FIRST_SPIKE) != Null)
		{
			while(DESTROY_SPIKE->NEXT_SPIKE != NETWORK.MATRIX->FIRST_SPIKE)
			{
				NEXT_SPIKE = DESTROY_SPIKE->NEXT_SPIKE;
				free(DESTROY_SPIKE);
				DESTROY_SPIKE = NEXT_SPIKE;
			}
			free(DESTROY_SPIKE);
			NETWORK.MATRIX->FIRST_SPIKE = Null;
		}
}


/*
=============================================================================
Name and dates: INFO_MATRIX
---------------
	Creation : 28-06-93
	Last modification : 28-06-93

Functionnal description :
-------------------------
	This function constructs the information string for the
	matrix representation.

Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	X		short		X position of the pointer
			Y		short		Y position of the pointer

Explicits outputs :	STRING		char *		The information string
			INDEX		int *		Index of the selected neuron

Implicits inputs  :	NETWORK		NETWORK_DEF	the definition of the network
			SIMUL		SIMUL_DEF	The definition of the simulation

Implicits outputs :	None

Sides effects :	None
---------------

=============================================================================
*/
/*static string locals to this file*/
static char STRING[256];

char *INFO_MATRIX(short X, short Y, int *INDEX)
{
	unsigned long COLOR_INDEX;
	int NEURON, CLUSTER;
	int i, TMP=0;
	char POT_STR[80];
	char NAME[80];

	/*computes neuron number*/
	NEURON = Y/NETWORK.MATRIX->NEURON_SIZE + 1;
	
	/*computes cluster number*/
	for(i=0; i<NETWORK.NB_CLUSTERS; i++)
	{
		TMP += *(NETWORK.NB_NEURONS_PER_CLUSTER + i);	
		if(NEURON <= TMP)
		{
			CLUSTER = i+1;
			break;
		}
	}

	
	/*computes potential*/
	if(SIMUL.FLAG.SIMUL_STATE == NO_SIMULATION || SIMUL.POT == Null)
		strcpy(POT_STR, ": Undefined");
	else
		sprintf(POT_STR, "= %3.2f mV", (SIMUL.POT + (NEURON - 1))->PMB);

	/*get cluster name*/
	if(NETWORK.INFO_CLUSTERS != Null)
		strcpy(NAME, NETWORK.INFO_CLUSTERS[CLUSTER-1].NAME);
	else
		sprintf(NAME, "CLUSTER_%d", CLUSTER);

	/*construct string*/
	sprintf(STRING, "CLUSTER\n %s\n No %d\n with %d neurons\nNEURON\n No %d/network\n No %d/cluster\nP.MB. %s", 
		NAME, CLUSTER, *(NETWORK.NB_NEURONS_PER_CLUSTER + (CLUSTER-1)), NEURON, 
		NEURON - (TMP - *(NETWORK.NB_NEURONS_PER_CLUSTER + (CLUSTER-1))), POT_STR);
	*INDEX = NEURON;
	return(STRING);
}
