/*
=============================================================================
	P M B _ V I S U . C
=============================================================================

	Author: Denis LAMBOLEZ (Feb 1993)
		Laboratoire de Biomathematiques
		URBB B3E INSERM U263
                Faculte de Medecine Saint-Antoine
                27 rue Chaligny
                75571 Paris cedex 12
			France

	Copyright Dr JF Vibert, July 1993
	All rights reserved
	Access limited to authorized persons	
	This software was developped only for research purpose
	and is not, in any way, usable for commercial purpose.

=============================================================================
	CONTENTS
=============================================================================
	- REALIZE_PMB :  This function allocs memory and initializes the
			 pmb representation.
	- COMPUTE_GEOMETRY_PMB : This function computes geometricals parameters
				 for clusters and details of the pmb 
				 representation.
	- DRAW_PMB : This function draws the pmb representation.
	- DRAW_NEURONS_VALUES_PMB : This function draws pmb of each cluster
				    at the current iteration.
	- PRINT_PMB : This function prints the pmb representation.
	- FREE_PMB : This function free the pmb representation.
	- INFO_PMB : This function construct the information string for the 
		     pmb representation.
=============================================================================
*/



#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <Xm/Text.h>
#include <X11/X.h>

#include "global_def_visu.h"
#include <plot_ps.h>

/*forward declarations*/
extern void COMPUTE_GEOMETRY_PMB();

/*locals constants*/
#define GRAPH_HEIGHT 265	/*size in pixels of a graphic*/
#define UPPER_OFFSET 10
#define LOWER_OFFSET 55
#define LEFT_OFFSET 40
#define RIGHT_OFFSET 20
#define Y_AXIS_LENGHT (GRAPH_HEIGHT-UPPER_OFFSET-LOWER_OFFSET)
#define X_AXIS_LENGHT (NETWORK.PMB->WIN_WIDTH-LEFT_OFFSET-RIGHT_OFFSET)
#define REFERENCE_STEP ((float)SIMUL.NB_ITERATION / 14.)

#define Y_NB_SCALE 11
#define X_NB_SCALE 14


/*
=============================================================================
Name and dates: REALIZE_PMB
---------------
	Creation : 28-06-93
	Last modification : 28-06-93

Functionnal description :
-------------------------
	This function allocs memory and initializes the pmb representation.

Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	None

Explicits outputs :	<return>	int		False => the representation can't be realized

Implicits inputs  :	NETWORK		NETWORK_DEF	the definition of the network
			DISPLAY_VISU	Display*	the display

Implicits outputs :	NETWORK		NETWORK_DEF	the definition of the network

Sides effects :	None
---------------

=============================================================================
*/
int REALIZE_PMB()
{
	Widget CLIP_WIN_ID;
	Arg arglist[2];
	int n, CLIP_WIN_SIZE;
	int SIZE_OF_SCREEN;

	/*test if there is some neuron selected else retrun to linear representation*/
	if(NETWORK.NB_SELECTED_NEURON == 0)
	{
		DISPLAY_MESSAGE("No neuron is selected! The membrane potential \
representation is now unavailable!", ERROR);
		XmToggleButtonSetState(WIDGET_ARRAY[k_Toggle_Linear_Representation], True, True);
		return(True);
	}
		
	/*allocs memory for the representation*/
	if((NETWORK.PMB = (PMB_GEOMETRY *)calloc(sizeof(PMB_GEOMETRY), 1)) == Null)
		return(False);

	/*computes and sets window size*/
	SIZE_OF_SCREEN = DisplayWidth(DISPLAY_VISU, DefaultScreen(DISPLAY_VISU));

#if 0
	/*actualize the size of the main window*/
	n=0;
	XtSetArg(arglist[n], XmNwidth, SIZE_OF_SCREEN); n++;
	XtSetValues(WIDGET_ARRAY[k_Window_Main_Visu], arglist, n);
#endif

	CLIP_WIN_SIZE = SIZE_OF_SCREEN - 245 - LEFT_OFFSET - RIGHT_OFFSET;   /*245 is corresponding to the window decoration*/
	
	NETWORK.PMB->GRAPH_STEP = (float)((float)CLIP_WIN_SIZE - (float)LEFT_OFFSET - (float)RIGHT_OFFSET)
	                          /(float)SIMUL.NB_ITERATION;

/*
	if(NETWORK.PMB->GRAPH_STEP < 1.)
	     NETWORK.PMB->GRAPH_STEP = 1.;
*/

	/*adjust the number of displayed iteration to display the maximum of iteration*/

	Taille_Dessin =  CLIP_WIN_SIZE/NETWORK.PMB->GRAPH_STEP;

	NETWORK.PMB->WIN_WIDTH = Taille_Dessin * NETWORK.PMB->GRAPH_STEP + LEFT_OFFSET + RIGHT_OFFSET;

	NETWORK.PMB->WIN_HEIGHT = GRAPH_HEIGHT * NETWORK.NB_SELECTED_NEURON;

	/*Actualize the drawing area size*/
	n=0;
	XtSetArg(arglist[n], XmNheight, NETWORK.PMB->WIN_HEIGHT); n++;
	XtSetArg(arglist[n], XmNwidth, NETWORK.PMB->WIN_WIDTH); n++;
	XtSetValues(WIDGET_ARRAY[k_Window_Drawing_Area_Main], arglist, n);
	
	/*allocs memory to keep values*/
	if((NETWORK.PMB->VALUES = (XPoint *)calloc(sizeof(XPoint), 
						 (SIMUL.NB_ITERATION + 1)*
						 NETWORK.NB_SELECTED_NEURON)) == Null)
		return(False);
	/*allocs memory to keep coordinates*/
	if((NETWORK.PMB->GRAPH_AXIS = (XSegment *)calloc(sizeof(XSegment), 
					(size_t)NETWORK.NB_SELECTED_NEURON * 2)) == Null)
		return(False);
	if((NETWORK.PMB->SCALE = (XSegment *)calloc(sizeof(XSegment), Y_NB_SCALE + X_NB_SCALE)) == Null)
		return(False);

	/*creates and initializes GC for graphics*/
	NETWORK.PMB->PMB_GRAPH_CTX = XCreateGC(DISPLAY_VISU, NETWORK.WINDOW_ID, 0, Null);
	XSetForeground(DISPLAY_VISU, NETWORK.PMB->PMB_GRAPH_CTX, LUT[C_BLACK]);

	/*computes geometry for graphics and details*/
	COMPUTE_GEOMETRY_PMB();

	return(True);
}


/*
=============================================================================
Name and dates: COMPUTE_GEOMETRY_PMB
---------------
	Creation : 28-06-93
	Last modification : 28-06-93

Functionnal description :
-------------------------
	This function computes geometricals parameters for graphics and details 
	of the pmb representation.
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	None

Explicits outputs :	None

Implicits inputs  :	NETWORK		NETWORK_DEF	the definition of the network
			DISPLAY_VISU	Display*	the display

Implicits outputs :	NETWORK		NETWORK_DEF	the definition of the network

Sides effects :	None
---------------

=============================================================================
*/
void COMPUTE_GEOMETRY_PMB()
{
	int i, j;

	/*computes axis coordinates*/
	for(i=0; i<2*NETWORK.NB_SELECTED_NEURON; i+=2)
	{
		(NETWORK.PMB->GRAPH_AXIS + i)->x1 = LEFT_OFFSET;
		(NETWORK.PMB->GRAPH_AXIS + i)->x2 = LEFT_OFFSET;
		(NETWORK.PMB->GRAPH_AXIS + i)->y1 = UPPER_OFFSET + GRAPH_HEIGHT*(i/2);
		(NETWORK.PMB->GRAPH_AXIS + i)->y2 = GRAPH_HEIGHT - LOWER_OFFSET + GRAPH_HEIGHT*(i/2);
		(NETWORK.PMB->GRAPH_AXIS + i + 1)->x1 = LEFT_OFFSET;
		(NETWORK.PMB->GRAPH_AXIS + i + 1)->x2 = NETWORK.PMB->WIN_WIDTH - RIGHT_OFFSET;
		(NETWORK.PMB->GRAPH_AXIS + i + 1)->y1 = GRAPH_HEIGHT - LOWER_OFFSET + GRAPH_HEIGHT*(i/2);
		(NETWORK.PMB->GRAPH_AXIS + i + 1)->y2 = GRAPH_HEIGHT - LOWER_OFFSET + GRAPH_HEIGHT*(i/2);
	}
	/*computes scale coordinates*/
	/*Y axis*/
	for(i=0; i< Y_NB_SCALE; i++)
	{
		(NETWORK.PMB->SCALE + i)->x1 = LEFT_OFFSET - 7;
		(NETWORK.PMB->SCALE + i)->x2 = LEFT_OFFSET;
	}
	/*X axis*/
	for(i=Y_NB_SCALE; i<=Y_NB_SCALE + X_NB_SCALE; i++)
		{
		  (NETWORK.PMB->SCALE + i)->x1 = 
		    (NETWORK.PMB->SCALE + i)->x2 = 
		      (int)( (float)LEFT_OFFSET + 
			     ((float)i-(float)Y_NB_SCALE) * 
			      (float)REFERENCE_STEP *			    
			      (float)NETWORK.PMB->GRAPH_STEP ); 
		}


	/*computes X and initials Y coordinates for values array*/

	for(i=0; i<NETWORK.NB_SELECTED_NEURON; i++)
		for(j=0; j<=SIMUL.NB_ITERATION; j++)
		{
		  (NETWORK.PMB->VALUES + i * (SIMUL.NB_ITERATION+1) + j)->x = (int)( 
		    (float)j * (float)NETWORK.PMB->GRAPH_STEP + (float)LEFT_OFFSET );
		  (NETWORK.PMB->VALUES + i * (SIMUL.NB_ITERATION+1) + j)->y = (int)(
		    (float)(i+1)*(float)GRAPH_HEIGHT - (float)LOWER_OFFSET);
		}

}


/*
=============================================================================
Name and dates: DRAW_PMB
---------------
	Creation : 28-06-93
	Last modification : 28-06-93

Functionnal description :
-------------------------
	This function draws the pmb representation.

Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	None

Explicits outputs :	None

Implicits inputs  :	NETWORK		NETWORK_DEF	the definition of the network
			DISPLAY_VISU	Display*	the display

Implicits outputs :	None

Sides effects :	None
---------------

=============================================================================
*/
void DRAW_PMB()
{
	XmString STRING;
	float Y_STEP, STEP;
	EXPOSED_REGION REGION;
	int FIRST_NEURON, LAST_NEURON;
	int i, j;
	char STR[80];

	

	/*gets back the visible part of the darwing area*/
	GET_EXPOSED_REGION(&REGION);

	/*computes the first and the last cluster to draw*/
	FIRST_NEURON = (int)floor((double)REGION.Y/(double)GRAPH_HEIGHT);
	LAST_NEURON = (int)floor((double)(REGION.Y + REGION.HEIGHT)/(double)GRAPH_HEIGHT);	
	if(LAST_NEURON == NETWORK.NB_SELECTED_NEURON) 
		LAST_NEURON--;

	/*draws axis*/
	XSetLineAttributes(DISPLAY_VISU, NETWORK.PMB->PMB_GRAPH_CTX, 3, LineSolid, CapButt,
			   JoinMiter);
	XDrawSegments(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.PMB->PMB_GRAPH_CTX, 
		      NETWORK.PMB->GRAPH_AXIS + (FIRST_NEURON * 2),
		      (LAST_NEURON - FIRST_NEURON + 1) * 2);

	/*draws details*/
	XSetLineAttributes(DISPLAY_VISU, NETWORK.PMB->PMB_GRAPH_CTX, 1, LineSolid, CapButt,
			   JoinMiter);
	for(i=FIRST_NEURON; i<=LAST_NEURON; i++)
	{
		/*draw scales*/
		/*first segments for Y axis and others for X axis*/
		/*Y axis*/
		for(j=0; j< Y_NB_SCALE; j++)
			(NETWORK.PMB->SCALE + j)->y1 = (NETWORK.PMB->SCALE + j)->y2 =
				(i+1)*GRAPH_HEIGHT - LOWER_OFFSET - j*Y_AXIS_LENGHT/10;
		/*X axis*/
		for(j=Y_NB_SCALE; j<Y_NB_SCALE + X_NB_SCALE; j++)
		{
			(NETWORK.PMB->SCALE + j)->y1 = (i+1)*GRAPH_HEIGHT - LOWER_OFFSET;
			(NETWORK.PMB->SCALE + j)->y2 = (i+1)*GRAPH_HEIGHT - LOWER_OFFSET + 7;
		}
		XDrawSegments(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.PMB->PMB_GRAPH_CTX, 
		      	      NETWORK.PMB->SCALE, Y_NB_SCALE + X_NB_SCALE);

		/*draw Y axis legend and neuron number*/
		sprintf(STR, "Membrane potential in mv for neuron %d/Network", 
			NETWORK.SELECTED_NEURON[i]);
		XDrawString(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.PMB->PMB_GRAPH_CTX, 
			    LEFT_OFFSET, i*GRAPH_HEIGHT + UPPER_OFFSET - 2, STR, strlen(STR));

		Y_STEP = (SIMUL.CS_TOP_VALUE - SIMUL.CS_BOTTOM_VALUE)/(float)(Y_NB_SCALE - 1);
		j = 0;
		for(STEP = SIMUL.CS_BOTTOM_VALUE; STEP <= SIMUL.CS_TOP_VALUE; STEP += Y_STEP)
		{
			sprintf(STR, "%5.1f", STEP);
			XDrawString(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.PMB->PMB_GRAPH_CTX, 
			    	    LEFT_OFFSET - 37, (i+1)*GRAPH_HEIGHT - LOWER_OFFSET 
				    - (j++) * Y_AXIS_LENGHT/(Y_NB_SCALE - 1), STR, strlen(STR));
		}
		
		/*draw X axis legend*/
		strcpy(STR, "Time in ms\0");
		XDrawString(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.PMB->PMB_GRAPH_CTX, 
			    LEFT_OFFSET, (i+1)*GRAPH_HEIGHT - LOWER_OFFSET + 30, STR, strlen(STR));
		
	}
	
	R_DRAW_NEURONS_VALUES(SAME_STEP);

}


/*
=============================================================================
Name and dates: DRAW_NEURONS_VALUES_PMB
---------------
	Creation : 28-06-93
	Last modification : 28-06-93

Functionnal description :
-------------------------
	This function draws global activity of each cluster at the current iteration.
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	STATE		int		FOREWARD, GOTO_INEDX, or SAME_STEP.

Explicits outputs :	None

Implicits inputs  :	NETWORK		NETWORK_DEF	the definition of the network
			DISPLAY_VISU	Display*	the display
			SIMUL	   SIMUL_PARAMETERS	Definition of the simulation
			DISPLAY_VISU	Display *	The display

Implicits outputs :	None

Sides effects :	None
---------------

=============================================================================
*/
void DRAW_NEURONS_VALUES_PMB(int STATE)
{
	Arg arglist[1];
	float PMB;
	EXPOSED_REGION REGION;
	static int PREV_MAX_IT;
	int NB_POINTS;
	int LAST_NEURON, FIRST_NEURON;
	int MIN_IT, MAX_IT;
	int i, j, n;
	float step;
	char STR[10];


	/*computes new values if needed*/
	if(SIMUL.FLAG.SIMUL_STATE != NO_SIMULATION && STATE != SAME_STEP)
	{
		/*computes new value for each cluster*/
		for(i=0; i<NETWORK.NB_SELECTED_NEURON; i++)
		{
			/*translates previous iterations if needed*/
			if(SIMUL.ITERATION > SIMUL.NB_ITERATION || STATE == GOTO_INDEX)
			{
				memmove(NETWORK.PMB->VALUES + i * (SIMUL.NB_ITERATION + 1),
					NETWORK.PMB->VALUES + i * (SIMUL.NB_ITERATION + 1) + 1,
					SIMUL.NB_ITERATION * sizeof(XPoint)/sizeof(char));
				/*restore x coordinate*/
				for(j=0; j<=SIMUL.NB_ITERATION; j++)
					(NETWORK.PMB->VALUES + i * (SIMUL.NB_ITERATION+1) + j)->x = j * 
					NETWORK.PMB->GRAPH_STEP + LEFT_OFFSET;
			}
			PMB = SIMUL.POT[NETWORK.SELECTED_NEURON[i]-1].PMB;
			if(PMB > SIMUL.CS_TOP_VALUE)
				PMB = SIMUL.CS_TOP_VALUE;
			else if(PMB < SIMUL.CS_BOTTOM_VALUE)
				PMB = SIMUL.CS_BOTTOM_VALUE;
			/*computes new point for each cluster*/
			if(SIMUL.ITERATION > SIMUL.NB_ITERATION)
				(NETWORK.PMB->VALUES + i * (SIMUL.NB_ITERATION + 1) + SIMUL.NB_ITERATION)->y = 
					(i+1)*GRAPH_HEIGHT - LOWER_OFFSET - Y_AXIS_LENGHT * 
					(PMB - SIMUL.CS_BOTTOM_VALUE) /
					(SIMUL.CS_TOP_VALUE - SIMUL.CS_BOTTOM_VALUE);
			else
				(NETWORK.PMB->VALUES + i * (SIMUL.NB_ITERATION + 1) + SIMUL.ITERATION)->y = 
					(i+1)*GRAPH_HEIGHT - LOWER_OFFSET - Y_AXIS_LENGHT *
					(PMB - SIMUL.CS_BOTTOM_VALUE) /
					(SIMUL.CS_TOP_VALUE - SIMUL.CS_BOTTOM_VALUE);
		}
	}
	
	/*computes number of points to draw*/
	if(SIMUL.ITERATION > SIMUL.NB_ITERATION)
		NB_POINTS = SIMUL.NB_ITERATION + 1;
	else
		NB_POINTS = SIMUL.ITERATION + 1;



	/*gets back the visible part of the darwing area*/
	GET_EXPOSED_REGION(&REGION);

	/*resize the window if needed*/
	if(REGION.WIDTH < NETWORK.PMB->WIN_WIDTH)
	{
		n=0;
		XtSetArg(arglist[n], XmNwidth, DisplayWidth(DISPLAY_VISU, DefaultScreen(DISPLAY_VISU))); n++;
		XtSetValues(WIDGET_ARRAY[k_Window_Main_Visu], arglist, n);
	}

	/*computes the first and the last cluster to draw*/
	FIRST_NEURON = (int)floor((double)REGION.Y/(double)GRAPH_HEIGHT);
	LAST_NEURON = (int)floor((double)(REGION.Y + REGION.HEIGHT)/(double)GRAPH_HEIGHT);
	if(LAST_NEURON == NETWORK.NB_SELECTED_NEURON) 
		LAST_NEURON--;

	/*computes max and min iteration*/
	MIN_IT = (SIMUL.ITERATION > SIMUL.NB_ITERATION) ? SIMUL.ITERATION - SIMUL.NB_ITERATION : 0;
	MAX_IT = MIN_IT + SIMUL.NB_ITERATION;
	
	/*clean and redraw time scale and graphic area*/
	for(i=FIRST_NEURON; i<=LAST_NEURON; i++)
	{
		/*clear previous time scale if it has changed */
		if(PREV_MAX_IT != MAX_IT)
			XClearArea(DISPLAY_VISU, NETWORK.WINDOW_ID, 0, (i+1)*GRAPH_HEIGHT - LOWER_OFFSET + 7,
				   NETWORK.PMB->WIN_WIDTH, 15, False);
		
		/*draw values*/
		if(STATE == FOREWARD && PREV_MAX_IT != MAX_IT)
		{
#if 0
		  XCopyArea(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.WINDOW_ID,
			    NETWORK.PMB->PMB_GRAPH_CTX, LEFT_OFFSET + NETWORK.PMB->GRAPH_STEP, 
			    i*GRAPH_HEIGHT + UPPER_OFFSET, NETWORK.PMB->WIN_WIDTH, 
			    Y_AXIS_LENGHT - 1, LEFT_OFFSET, i*GRAPH_HEIGHT + UPPER_OFFSET);
		  
		  XDrawLines(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.PMB->PMB_GRAPH_CTX, 
			     NETWORK.PMB->VALUES + i * (SIMUL.NB_ITERATION + 1) +
			     SIMUL.NB_ITERATION - 1, 2, CoordModeOrigin);
#endif
		  XClearArea(DISPLAY_VISU, NETWORK.WINDOW_ID, LEFT_OFFSET + 2,
			     i*GRAPH_HEIGHT + UPPER_OFFSET,
			     NETWORK.PMB->WIN_WIDTH, Y_AXIS_LENGHT - 1, False);
		  
		  XDrawLines(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.PMB->PMB_GRAPH_CTX, 
			     NETWORK.PMB->VALUES + i * (SIMUL.NB_ITERATION + 1),
			     NB_POINTS, CoordModeOrigin);

		}
		else if(STATE == FOREWARD && PREV_MAX_IT == MAX_IT)
		{
		  XDrawLines(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.PMB->PMB_GRAPH_CTX, 
			   	   NETWORK.PMB->VALUES + i * (SIMUL.NB_ITERATION + 1) + 
				   SIMUL.ITERATION - 1, 2, CoordModeOrigin);
		}
		else if(STATE == SAME_STEP)
		{
		  XClearArea(DISPLAY_VISU, NETWORK.WINDOW_ID, LEFT_OFFSET + 2,
			     i*GRAPH_HEIGHT + UPPER_OFFSET,
			     NETWORK.PMB->WIN_WIDTH, Y_AXIS_LENGHT - 1, False);
		  
		  XDrawLines(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.PMB->PMB_GRAPH_CTX, 
			     NETWORK.PMB->VALUES + i * (SIMUL.NB_ITERATION + 1),
			     NB_POINTS, CoordModeOrigin);
		}

		/*draw references*/

		for(step=MIN_IT; step<=MAX_IT; step+=REFERENCE_STEP)
		{
		  sprintf(STR, "%.1f", step);
		  XDrawString(DISPLAY_VISU, NETWORK.WINDOW_ID, NETWORK.PMB->PMB_GRAPH_CTX, 
			      (int)( (float)LEFT_OFFSET + ((float)step-(float)MIN_IT)*(float)NETWORK.PMB->GRAPH_STEP - 5.), 
			      (int)( ((float)i+1.)*(float)GRAPH_HEIGHT - (float)LOWER_OFFSET + 15. ), STR, strlen(STR));
		 
		}

	}

	/*keep for the next call*/
	PREV_MAX_IT = MAX_IT;

}


/*
=============================================================================
Name and dates: PRINT_PMB
---------------
	Creation : 04-06-93
	Last modification : 04-06-93

Functionnal description :
-------------------------
	This function print the pmb representation in its actual
	state if there is a simulation running. This function use the
	PLOT_PS library.
Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	DRAW_ALL	int		Not used. We always draw the 
							entire network.
			PAPER_WIDTH	float		The width of the sheet

Explicits outputs :     None

Implicits inputs  :	NETWORK		NETWORK_DEF	The definition of the network
			CMAP_ID		Colormap	The colormap
			DISPLAY_VISU	Display *	The display

Implicits outputs :	None

Sides effects :		None
---------------

=============================================================================
*/

void PRINT_PMB(int DRAW_ALL, float PAPER_WIDTH)
{
	float MAX_HEIGHT = 25.;
	float SCALE;
	float GRAPHIC_HEIGHT;
	float Y_STEP, STEP;
	EXPOSED_REGION REGION;
	int i, j, k, IT,page;
	float step;
	char STRING[80];




	if(!SIMUL.ITERATION)
	  return;

	
	if(Coord_X != NULL)
	  FREE_MEMORY(Coord_X);

	if((Coord_X = (float *)calloc(SIMUL.ITERATION + 1, sizeof(float))) == NULL)
	  {
	    printf("Synapse structure allocation impossible\nTry later\n");
	    return;
	  } 	
	if(Coord_Y != NULL)
	  FREE_MEMORY(Coord_Y);

	if((Coord_Y = (float *)calloc(SIMUL.ITERATION + 1, sizeof(float))) == NULL)
	  {
	    printf("Synapse structure allocation impossible\nTry later\n");
	    return;
	  } 	


	/*computes scale for drawing.*/
	/*one user unit equal SCALE centimeters*/
	SCALE = (PAPER_WIDTH - 2.)/ (float)(NETWORK.PMB->WIN_WIDTH); 
	/*computes height of a graphic*/
	/*we considere that the sheet drawable area is maximum 20. centimeters heigh*/
	/*and we draw 3 clusters maximum per sheet*/
	GRAPHIC_HEIGHT = MAX_HEIGHT/3./SCALE;

	/*if drawing is not possible*/
	if(SCALE < 0.008)
	{
		/*set origin. we take count of margins for bubble jet printer*/
		ps_origin(0., 1., 0., 0.);
		ps_writeah(0., 5., "XNBC : Visualization Printing Error", 0.7);
		ps_writeah(0., 4., NETWORK.NAME, 0.5);
		ps_writeah(0., 3., "is too large to be printed", 0.5);
	}
	/*drawing is possible*/
	else
	{
		if (Coord_X == NULL || Coord_Y == NULL)
			return;
		/*set scale*/
		ps_echel(SCALE, SCALE, 0., 0., 1., 0.);
		/*draw comments for the first sheet*/
		ps_cms();
		ps_origin(0., MAX_HEIGHT + 0.2, 0., 0.);
		/*draw values*/
		sprintf(STRING, "Iteration : %d    N.G.A. : %d%%", 
			SIMUL.ITERATION, SIMUL.SPIKE_COUNT);
		ps_writeah(0., 0.2, STRING, 0.4);
		sprintf(STRING, "NETWORK : %s", NETWORK.NAME);
		ps_writeah(0., 0.8, STRING, 0.4);
		/*set origin. we take count of margins for bubble jet printer*/
		ps_origin(0., -MAX_HEIGHT - 0.2, 0., 0.);
		/*use user units*/
		ps_user();
		/*neuron loop*/
		
		k = 0;     /* ces deux variables sont utilise pour recaler */
		page = 0;  /* les dessin du fichier Ps pour les differentes page */

		for(i=0; i<NETWORK.NB_SELECTED_NEURON; i++)
		{
		  k++;
		  if (k==4)
		    {
		      k=1;
		      page++;
		    }
			/* draw comments on each sheet*/
			if(i != 0 && i%3 == 0)
			{
				/*new page*/
				ps_newpage();
				ps_cms();
				ps_origin(0., MAX_HEIGHT + 0.2, 0., 0.);
				/*draw values*/
				sprintf(STRING, "Iteration : %d    N.G.A. : %d%%", 
					SIMUL.ITERATION, SIMUL.SPIKE_COUNT);
				ps_writeah(0., 0.2, STRING, 0.4);
				sprintf(STRING, "NETWORK : %s", NETWORK.NAME);
				ps_writeah(0., 0.8, STRING, 0.4);
				ps_origin(0., -MAX_HEIGHT - 0.2, 0., 0.);
				ps_user();
			}

			/*draw axis*/ 

			ps_line_width(2.);
			Coord_Y[0] = MAX_HEIGHT/SCALE - ((i%3+1)*GRAPH_HEIGHT - LOWER_OFFSET - Y_AXIS_LENGHT)*GRAPHIC_HEIGHT/GRAPH_HEIGHT;
			Coord_X[0] = LEFT_OFFSET;
			Coord_Y[1] = MAX_HEIGHT/SCALE - ((i%3+1)*GRAPH_HEIGHT - LOWER_OFFSET)*GRAPHIC_HEIGHT/GRAPH_HEIGHT;
			Coord_X[1] = LEFT_OFFSET;
			Coord_Y[2] = Coord_Y[1];
			Coord_X[2] = LEFT_OFFSET + X_AXIS_LENGHT;
			/*X and Y axis*/
			ps_polyline(Coord_X, Coord_Y, 3, False, 0, 0.);

			/*draw ticked*/
			ps_line_width(1.);

			/*Y axis*/
			for(j=0; j<Y_NB_SCALE; j++)
			{
				Coord_X[0] = LEFT_OFFSET;
				Coord_Y[0] = MAX_HEIGHT/SCALE - ((i%3+1)*(float)GRAPH_HEIGHT - (float)LOWER_OFFSET - 
				       (float)Y_AXIS_LENGHT/(float)(Y_NB_SCALE-1)*j)*GRAPHIC_HEIGHT/GRAPH_HEIGHT;
				Coord_X[1] = LEFT_OFFSET - 7.;
				Coord_Y[1] = Coord_Y[0];

				ps_polyline(Coord_X, Coord_Y, 2, False, 0, 0.);
			} 
			/*X axis*/
			for(step=0; step<SIMUL.NB_ITERATION; step+=REFERENCE_STEP)
			{
				Coord_X[0] = LEFT_OFFSET + step * NETWORK.PMB->GRAPH_STEP;
				Coord_Y[0] = MAX_HEIGHT/SCALE - ((i%3+1)*GRAPH_HEIGHT - LOWER_OFFSET)*GRAPHIC_HEIGHT/GRAPH_HEIGHT;
				Coord_X[1] = Coord_X[0];
				Coord_Y[1] = Coord_Y[0] - 5.0*GRAPHIC_HEIGHT/GRAPH_HEIGHT;
				ps_polyline(Coord_X, Coord_Y, 2, False, 0, 0.);
			}

			
			
			/*draw values*/
			
			for(j=0 ; j < SIMUL.NB_ITERATION; j++)
			{
				Coord_X[j] = LEFT_OFFSET + j * NETWORK.PMB->GRAPH_STEP;
				Coord_Y[j] = ( page + 1 ) * MAX_HEIGHT/SCALE - 
				       (NETWORK.PMB->VALUES + 
					i * (SIMUL.NB_ITERATION+1) + j)->y * 
					GRAPHIC_HEIGHT/GRAPH_HEIGHT;
			}


			ps_polyline(Coord_X, Coord_Y, SIMUL.NB_ITERATION, False, 0, 0.);


			/*draw legend and references*/
			ps_cms();
			/*Y legend*/
			sprintf(STRING, "Membrane potential in mv for neuron %d/Network", 
				NETWORK.SELECTED_NEURON[i]);
			ps_writeah((float)LEFT_OFFSET * SCALE, 
				   MAX_HEIGHT - ((i%3+1)*GRAPH_HEIGHT - LOWER_OFFSET - Y_AXIS_LENGHT) * 
				   GRAPHIC_HEIGHT/GRAPH_HEIGHT * SCALE + 0.2, STRING, 0.3);
			/*X legend*/
			ps_writeah((float)LEFT_OFFSET * SCALE - 1.5, 
				   MAX_HEIGHT - ((i%3+1)*GRAPH_HEIGHT - LOWER_OFFSET) * 
				   GRAPHIC_HEIGHT/GRAPH_HEIGHT * SCALE -0.4, "Time in", 0.3);
			ps_writeah((float)LEFT_OFFSET * SCALE - 1, 
				   MAX_HEIGHT - ((i%3+1)*GRAPH_HEIGHT - LOWER_OFFSET) * 
				   GRAPHIC_HEIGHT/GRAPH_HEIGHT * SCALE - 0.7, "ms", 0.3);
			/*Y references*/
			Y_STEP = (SIMUL.CS_TOP_VALUE - SIMUL.CS_BOTTOM_VALUE)/(float)(Y_NB_SCALE - 1);
			j = 0;
		

			for(STEP = SIMUL.CS_BOTTOM_VALUE; STEP <= SIMUL.CS_TOP_VALUE; STEP += Y_STEP)
			{
				sprintf(STRING, "%5.1f", STEP);
				ps_writeah((float)(LEFT_OFFSET)*SCALE - 1.1,
					   MAX_HEIGHT - ((i%3+1)*(float)GRAPH_HEIGHT - (float)LOWER_OFFSET - 
					   (float)Y_AXIS_LENGHT/(float)(Y_NB_SCALE -1)* (j++)) * 
				   	   GRAPHIC_HEIGHT/GRAPH_HEIGHT * SCALE, STRING, 0.3);
			}
			/*X references*/
			IT = (SIMUL.ITERATION > SIMUL.NB_ITERATION) ? SIMUL.ITERATION - SIMUL.NB_ITERATION : 0;
			for(step=0; step<SIMUL.NB_ITERATION; step+=REFERENCE_STEP)
			{
				sprintf(STRING, "%d", IT + (int)step);
				ps_writeah((float)(LEFT_OFFSET + step*NETWORK.PMB->GRAPH_STEP)*SCALE - 0.1,
					   MAX_HEIGHT - ((i%3+1)*(float)GRAPH_HEIGHT - (float)LOWER_OFFSET + 5.) * 
				   	   GRAPHIC_HEIGHT/GRAPH_HEIGHT * SCALE - 0.3, STRING, 0.3);
			}

			ps_user();


		} /*end of neuron loop*/
	}/*end of drawing*/

}


/*
=============================================================================
Name and dates: FREE_PMB
---------------
	Creation : 28-06-93
	Last modification : 28-06-93

Functionnal description :
-------------------------
	This function free the pmb representation.

Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	None

Explicits outputs :	None

Implicits inputs  :	NETWORK		NETWORK_DEF	the definition of the network

Implicits outputs :	NETWORK		NETWORK_DEF	the definition of the network

Sides effects :	None
---------------

=============================================================================
*/


void FREE_PMB(int ACTION)
{
	int n, i, j;

	/*free the entire representation*/
	if(NETWORK.PMB != Null && ACTION == ALL)
	{
		FREE_GC(NETWORK.PMB->PMB_GRAPH_CTX);
		FREE_MEMORY(NETWORK.PMB->VALUES);
		FREE_MEMORY(NETWORK.PMB->GRAPH_AXIS);
		FREE_MEMORY(NETWORK.PMB->SCALE);

		FREE_MEMORY(NETWORK.PMB);
		NETWORK.PMB = Null;
	}
	
	/*reinitializes values array*/
	if(NETWORK.PMB != Null && ACTION == SPECIAL)
		/*computes X and initials Y coordinates for values array*/
		for(i=0; i<NETWORK.NB_SELECTED_NEURON; i++)
			for(j=0; j<=SIMUL.NB_ITERATION; j++)
			{
				(NETWORK.PMB->VALUES + i * (SIMUL.NB_ITERATION+1) + j)->x = j * 
					NETWORK.PMB->GRAPH_STEP + LEFT_OFFSET;
				(NETWORK.PMB->VALUES + i * (SIMUL.NB_ITERATION+1) + j)->y = (i+1) *
					GRAPH_HEIGHT - LOWER_OFFSET;
			}
}


/*
=============================================================================
Name and dates: INFO_PMB
---------------
	Creation : 28-06-93
	Last modification : 28-06-93

Functionnal description :
-------------------------
	This function construct the information string for the pmb representation.

Inputs - Outputs :
------------------
			Name		Type		Description
Explicits inputs  :	X		short		X position of the pointer
			Y		short		Y position of the pointer

Explicits outputs :	STRING		char *		The information string
			INDEX		int *		Not Used

Implicits inputs  :	NETWORK		NETWORK_DEF	the definition of the network
			SIMUL		SIMUL_DEF	The definition of the simulation

Implicits outputs :	None

Sides effects :	None
---------------

=============================================================================
*/
/*static string locals to this file*/
static char STRING[256];

char *INFO_PMB(short X, short Y, int *INDEX)
{	
	float VALUE, TMP_VALUE;
	int NEURON, ITERATION, CLUSTER;
	int i, TMP=0, MIN_IT;
	char POT_STR[80];
	char NAME[80];


	/*computes neuron number*/
	NEURON = Y/GRAPH_HEIGHT;
	*INDEX = NETWORK.SELECTED_NEURON[NEURON];
	/*compute cluster number*/
	for(i=0; i<NETWORK.NB_CLUSTERS; i++)
	{
		TMP += *(NETWORK.NB_NEURONS_PER_CLUSTER + i);	
		if(*INDEX <= TMP)
		{
			CLUSTER = i+1;
			break;
		}
	}
	
	/*computes iteration*/
	MIN_IT = (SIMUL.ITERATION > SIMUL.NB_ITERATION) ? SIMUL.ITERATION - SIMUL.NB_ITERATION : 0;
	ITERATION = MIN_IT + (X-LEFT_OFFSET)/NETWORK.PMB->GRAPH_STEP;
	if(ITERATION < MIN_IT )
		ITERATION = MIN_IT;
	if(ITERATION > SIMUL.ITERATION)
		ITERATION = SIMUL.ITERATION;
	
	/*computes activity*/
	if(SIMUL.FLAG.SIMUL_STATE == NO_SIMULATION || NETWORK.PMB->VALUES == Null
	   || ITERATION > SIMUL.ITERATION)
		strcpy(POT_STR, ": Undefined");
	else
	{
		TMP_VALUE = (float)(NEURON+1) * (float)GRAPH_HEIGHT - (float)LOWER_OFFSET
			 - (float)(NETWORK.PMB->VALUES + NEURON * (SIMUL.NB_ITERATION+1)
			 + ITERATION - MIN_IT)->y;
		VALUE = TMP_VALUE * (SIMUL.CS_TOP_VALUE - SIMUL.CS_BOTTOM_VALUE)/(float)Y_AXIS_LENGHT 
			+ SIMUL.CS_BOTTOM_VALUE;
		sprintf(POT_STR, "= %3.2f", VALUE);
	}
	
	/*get cluster name*/
	if(NETWORK.INFO_CLUSTERS != Null)
		strcpy(NAME, NETWORK.INFO_CLUSTERS[CLUSTER-1].NAME);
	else
		sprintf(NAME, "CLUSTER_%d", CLUSTER);

	/*construct string*/
	
	sprintf(STRING, "TIME : %d ms\nCLUSTER\n %s\n No %d\n with %d neurons\nNEURON\n No %d/network\n No %d/cluster\nP.MB. %s",
		ITERATION, NAME, CLUSTER,
		*(NETWORK.NB_NEURONS_PER_CLUSTER + (CLUSTER-1)), *INDEX, 
		*INDEX - (TMP - *(NETWORK.NB_NEURONS_PER_CLUSTER + (CLUSTER-1))), POT_STR);
	 
	
	return(STRING);
}



















